// Dependencies
const support = require( './support.js' ) ;
const latinize = require( 'latinize' ) ;

// List common spellings of numbers
const number_list = [
	[ 1,	'een' ],
	[ 2,	'twee' ],
	[ 3,	'drie' ],
	[ 4,	'vier' ],
	[ 5,	'vijf' ],
	[ 6,	'zes' ],
	[ 7,	'zeven' ],
	[ 8,	'acht' ],
	[ 9,	'negen' ],
	[ 10,	'tien' ],
	[ 11,	'elf' ],
	[ 12,	'twaalf' ],
	[ 13,	'dertien' ],
	[ 14,	'veertien' ],
	[ 15,	'vijftien' ],
	[ 15,	'vijften' ],
	[ 16,	'zestien' ],
	[ 17,	'zeventien' ],
	[ 18,	'achttien' ],
	[ 19,	'negentien' ],
	[ 20,	'twintig' ],
	[ 24,	'vierentwintig' ],
	[ 30,	'dertig' ],
	[ 60,	'zestig' ],
	[ 120,	'honderdtwintig' ],
	[ 200,	'tweehonderd' ],
	[ 240,	'tweehonderdenveertig' ]
] ;

// Replace the words by numbers
function normalise_txt( txt ) {
	var result = latinize( txt ) ;
	number_list.map( item => {
		result = result.replace( new RegExp( '\\b' + item[ 1 ] + '\\b', 'g' ), item[ 0 ] ) ;
	} ) ;
	return result
		.replace( /(\d+) *(?:jaar|jaren) *en *6 *(?:maand|maanden)\b/g, '$1.5 jaar' )
		.replace( /(\d+) *(?:jaar|jaren) *en *4 *(?:maand|maanden)\b/g, '$1.3333333 jaar' )
		.replace( /(\d+) *(?:jaar|jaren) *en *8 *(?:maand|maanden)\b/g, '$1.6666666 jaar' ) ;
}

// List common units for sentences
const unit_list = [
	[ 'uur', 'uren' ],
	[ 'dag', 'dagen' ],
	[ 'week', 'weken' ],
	[ 'maand', 'maanden' ],
	[ 'jaar', 'jaren' ]
] ;

// Extract the maxima from the text
function extract_max( txt ) {
	var result = [ ] ;
	const txt_list = txt.split( /(?=ten hoogste)/ ) ;
	if( txt_list.length > 1 ) {
		txt_list.slice( 1 ).map( txt_item => {
			unit_list.map( unit_list => {
				const pat = new RegExp( '.*ten *hoogste *([0-9\.]+) *(?:achtereenvolgende)? *(?:\\b' + unit_list.join( '\\b|\\b' ) + '\\b).*', 'i' ) ;
				const number = txt_item.replace( pat, '$1' ) ;
				const unit = unit_list[ 0 ] ;
				if( number !== txt_item  && unit !== 'uur' ) {
					result.push( { soort: 'gevangenisstraf', eenheid: unit, aantal: Number.parseFloat( number ) } ) ;
				}
			} ) ;
		} ) ;
	}
	if( result.length === 0 ) {
		return undefined
	}
	const sorted_result = result.sort( ( a, b ) => support.unit2hours( a.aantal, a.eenheid ) - support.unit2hours( b.aantal, b.eenheid ) ) ;
	return sorted_result[ sorted_result.length  - 1 ] ;
}

// Pretty print
function show_data( data ) {
	var txt_list = [ ] ;
	txt_list.push( '"Wet": ' + JSON.stringify( data.law ) ) ;
	txt_list.push( '"Nummer": ' + JSON.stringify( data.number ) ) ;
	txt_list.push( '"Artikel": ' +  JSON.stringify( data.article ).replace( /","/g, '",\n      "' ) ) ;
	if( typeof data.max !== 'undefined' ) {
		txt_list.push( '"Maximum": ' +  JSON.stringify( data.max ) ) ;
	}
	return '  {\n    ' + txt_list.join( ',\n    ' ) + '\n  }' ;
}

// Process one law text
function process_law_text( filename ) {
	const input = support
		.read_txt( filename )
		.replace( /\nTitel [IVX0-9].*/g, '' ) 
		.replace( /\nHoofdstuk [IVX0-9].*/g, '' ) 
		.replace( /\nParagraaf [IVX0-9].*/g, '' ) 
		.replace( /\nAfdeling [IVX0-9].*/g, '' ) 
		.replace( /\n§ [IVX0-9].*/g, '' )
		.replace( /\n+/g, '\n' ) ;
	const art_list = input
		.split( /(?=\nArtikel [0-9][0-9a-z\.]*\n)/s ) 
		.slice( 1 ) ;
	const result_list = art_list.map( item => {
		const txt = normalise_txt( item.trim() )
		const data = {
			law: filename.replace( /(?:.*\/)?([A-Za-z0-9]+)\.txt/, '$1' ),
			number:	txt.replace( /Artikel ([0-9][a-z0-9\.]*)\n.*/s, '$1' ),
			article: item.replace( /Artikel ([0-9][a-z0-9\.]*)\n/s, '' ).trim().split( '\n' )
		} ;
		const max = extract_max( txt ) ;
		if( typeof max !== 'undefined' ) {
			data.max = max ;
		}
		return data ;
	} ) ;
	return result_list ;
}

// Main function
function main() {
	var output_filename ;
	var input_filename_list = [] ;
	for( var i = 2; i < process.argv.length; i++ ) {
		const arg = process.argv[ i ] ;
		if( arg === '-o' ) {
			i++ ;
			if( i < process.argv.length ) {
				output_filename = process.argv[ i ] ;
			}
		} else {
			input_filename_list.push( arg ) ;
		}
	}
	if( input_filename_list.length === 0 || typeof output_filename === 'undefined' ) {
		console.error( 'usage: node %s input_file... -o output_file', process.argv[ 1 ] ) ;
		process.exit( 1 ) ;
	}
	const result_list = input_filename_list.map( process_law_text ).flat() ;
	const txt = result_list.map( show_data ).join( ',\n' ) ;
	support.write_txt( output_filename, '[\n' + txt + '\n]' ) ;
}

main() ;
