// Dependencies
const latinize = require( 'latinize' ) ;
const support = require( './support.js' ) ;
const truncate_max = 150 ;

// CLI options
var debug = false ;
var debug_debug = false ;
var trace = false ;
var sections = false ;

// Trace_match is functionally equivalent to the .match() method but in addition it traces the call and the results
function trace_match( txt, pat ) {
	if( typeof txt === 'undefined' ) {
		return false ;
	} else {
		const result = txt.match( pat ) ;
		if( trace && result ) {
			console.log( 'trace_match	%s	%s',
				support.truncate( JSON.stringify( pat.source ), truncate_max ), support.truncate( JSON.stringify( result ), truncate_max ) ) ;
		}
		return result ;
	}
}

// Trace_match_list takes a list of objects with an id and a pattern field, and returns a list of matches, sorted by the position of the matches.
// The id field of the list of patterns is copied into the matches.
// Setting last_match = true will only work for patterns with the g flag.
function trace_match_list( txt, pat_list, last_match = false ) {
	if( typeof txt === 'undefined' ) {
		return null ;
	} else {
		const result_list = pat_list
			.map( item => {
				// console.log( 'trace_match_list %s: %s', item.id, txt ) ;
				if( last_match ) {
					const match_list = [ ...txt.matchAll( item.pattern ) ] ;
					if( match_list.length > 0 ) {
						return Object.assign( match_list[ match_list.length - 1 ], { id: item.id } ) ;
					}
				} else {
					const match = txt.match( item.pattern ) ;
					if( match ) {
						return Object.assign( match, { id: item.id } ) ;
					}
				}
				return null ;
			} )
			.filter( item => item )
			// sort by position and break ties by the shortest match.
			.sort( ( a, b ) => a.index !== b.index ? a.index - b.index : a[ 0 ].length - b[ 0 ].length ) ;
		if( result_list.length === 0 ) {
			return null ;
		} else {
			var result = result_list[ last_match ? result_list.length - 1 : 0 ] ;
			if( trace ) {
				console.log( 'trace_match_list	%s	%d:%s',
					result_list.map( item => item.id ).join( '/' ), result.index, support.truncate( JSON.stringify( result ), truncate_max ) ) ;
			}
			return result ;
		}
	}
}

// Try the lists in the given list of lists in order.
function trace_match_list_list( txt, pat_list_list, last_match = false ) {
	if( typeof txt === 'undefined' ) {
		return null ;
	} else {
		const result_list = pat_list_list
			.map( pat_list => trace_match_list( txt, pat_list, last_match ) )
			.filter( item => item )
			.sort( ( a, b ) => a.index !== b.index ? a.index - b.index : a[ 0 ].length - b[ 0 ].length ) ;
		if( trace ) {
			result_list.map( item => {
				console.log( 'trace_match_list_list	%s	%d:%s',
					item.id, item.index, support.truncate( JSON.stringify( item ), truncate_max ) ) ;
			} ) ;
		}
		if( result_list.length > 0 ) {
			return result_list[ last_match ? result_list.length - 1 : 0 ] ;
		} else {
			return null ;
		}
	}
}

// Turn a list of strings into a list of patterns, suitable for trace_match_list.
function mk_pat_list( tag, before, list, after, warning = true, flags = 'i' ) {
	const pat_list	= list.map( (item, index) => { return { id: tag + '_' + index, pattern: new RegExp( before + item + after, flags ) } } ) ;
	pat_list.map( item => {
		if( warning && item.pattern.toString().match( /[<>]/ ) ) {
			console.log( 'mk_pat_list', item.id, item.pattern ) ;
		}
	} ) ;
	return pat_list ;
}

// Look for the sections with relevant headers.
// We expect the header to start at the beginning of the section, i.e. in the first 20 characters.
// In some cases (i.e. when sections have been poorly structured, a header may occur somewhere else, as controlled by start_limit.
// In any case the begining of the section is stripped, so that it starts with the heading.
function extract_headers( pattern, section_list, start_limit=20, header_limit=0 ) {
	var result = '' ;
	for( var index = 0; index < section_list.length; index++ ) {
		const txt = section_list[ index ].replace( /^[0-9\. ]*/, '' ) ;
		const pos = txt.search( pattern ) ;
		if( pos >= 0 && ( start_limit === -1 || pos <= start_limit ) ) {
			if( header_limit === 0 ){
				result = txt.substring( pos ) ;
			} else {
				const end = Math.max( index+header_limit, section_list.length ) ;
				result = txt.substring( pos ) + ' ' + section_list.slice( index + 1, end ).join( ' ' ) ;
			}
			break ;
		}
	}
	if( result.length > 0 ) {
		if( debug_debug) console.log( 'extract_headers_1	/%s/ %d,%d\nresult: "%s"',
			support.truncate( pattern.source, truncate_max ), start_limit, header_limit, support.truncate( result, truncate_max ) ) ;
		return result ;
	} else {
		return undefined ;
	}
}

// Where do we find demographics of the defendant?
const demographics_pat	= /\bgeboren\b[^\.]*/i ;

// What year was the defandant born?
const year_pat		= /\bgeboren\b[^\.]*?([12]\d\d\d)/i ;

// Is there an age in the contents.info section?
const age_txt		= '\\b((?:[1-9]\\d|[a-z]+))[ -]*jarige *(?:verdachte|jongens?|mann?e?n?|vrouwe?n?|inwoners?|hackers?|\\[inwoner|[A-Z][a-z]*)'
const age_pat		= new RegExp( '(?:' + age_txt + '[^\\.]*(?:veroordeeld|opgelegd)|(?:[Vv]eroordeling|[Vv]eroordeelt)[^\\.]*' + age_txt + ')' ) ;
const jarige_pat	= /\b(?:[1-9]\d|[a-z]+)[ -]*jarige/g ;

// Is the defendant from abroad?
const country_pat	= /\bgeboren[^\.]*?(?:geboorte[a-z ]*?land|land van herkomst|\[land\]|\( *[A-Z][a-zA-Z ]+\))/i ;

// List text fragments that indicate a female defendant
const gender_list = [
	'verdachte (?:is|was) (?:daardoor )?(?:niet )?in haar (?:belang|verdediging)',
	'overtuigend bewezen dat([:0-9\\. -]*)zij',
	'verdachte, haar',
	'[^\\\/]haar (?:medeverdachte|mededader)',
	'verdachte (?:met|samen met|alleen met|en|of|en\\\/of|zonder) haar',
] ;
const gender_pat_list	= mk_pat_list( 'gender', '\\b', gender_list, '\\b' ) ;

// List the beginning and the end of the text with the considerations about first/repeat offendership
const offender_list = [
	'verdachte[^\.]*(?:eerder|vaker|onlangs|recent)[^\.]*(?:in aanraking|veroordeeld|gepleegd)',
	'verdachte[^\.]*(?:strafrechtelijke? antecedente?n?|eerdere? antecedente?n?|eerdere? veroordelinge?n?)',
	'verdachte zich[^\.]*eerder schuldig (?:heeft )?gemaakt',
	'(?:strafblad|documentatie)[^\.]*(?:in aanraking|schuldig gemaakt|bestraft|veroordeeld|veroordelinge?n?)',
	'(?:strafblad|documentatie)[^\.]*\.?[^\.]*(?:in aanraking (?:is |was )?(?:gekomen|geweest)|schuldig gemaakt|bestraft|veroordeeld|veroordelinge?n?)'
] ;
const offender_pat_list	= mk_pat_list( 'offender', '\\b', offender_list, '\\b' ) ;

// List special phrases that the courts have used at least once to indicate first offendership
const first_offender_list = [
	'verdachte\]?zich [^\.]* niet eerder schuldig heeft gemaakt',
	'over verdachte\]?geen justitiele documentatie bekend is',
	'blanco strafblad[^\.]*van verdachte',
	'blanco Nederlandse, Spaanse en Engelse strafbladen',
	'in het voordeel van de verdachte\]? is gelet op zijn blanco strafblad',
	'verdachte\]? blijkens[^\.]*een blanco strafblad heeft',
	'verdachte\]? en (?:zijn|haar) blanco strafblad',
	'heeft verdachte\]? een (?:zogenaamd )?blanco strafblad',
	'verdachte\]? heeft een (?:zogenaamd )?blanco strafblad',
	'verdachte\]? een (?:zogenaamd )?blanco strafblad heeft',
	'heeft verdachte\]? geen[^\.]*(?:justitiele documentatie|strafblad)',
	'verdachte\]? heeft geen[^\.]*(?:justitiele documentatie|strafblad)',
	'verdachte\]? geen [^\.]*(?:justitiele documentatie|strafblad) heeft',
	'is verdachte\]? first[ -]*offender',
	'verdachte\]? is first[ -]*offender',
	'verdachte\]? first[ -]*offender is'
] ;
const first_offender_pat_list	= mk_pat_list( 'first_offender', '\\b', first_offender_list, '\\b' ) ;

// List special phrases that the courts have used at least once to indicate repeat offendership
const repeat_offender_list = [
	'waarvoor verdachte in \d* is veroordeeld',
	'gezien de ernst van de feiten en het strafblad van de verdachte',
	'strafblad van de verdachte kan niet anders worden gereageerd',
	'justitiele documentatie[^\.]*heeft verdachte zich eerder schuldig gemaakt',
	'verdachte[^\.]*met een uitgebreid strafblad',
	'in zijn nadeel werkt zijn strafblad, de verdachte[^\.]*veroordeeld',
	'ten voordele van verdachte\]? rekening met het beperkte strafblad van verdachte',
	'rol van verdachte\]? en zijn uitgebreide documentatie',
	'verdachte kent recidive',
	'verdachte moet worden beschouwd als recidivist',
	'verdachte\]? is gerecidiveerd',
	'eerdere contacten met justitie'
] ;
const repeat_offender_pat_list	= mk_pat_list( 'repeat_offender', '\\b', repeat_offender_list, '\\b' ) ;
const yes_no_pat	= /\b(?:geen|nooit|niet)\b/i ;

// Check if each article number in the list is in the range 1..599.
function is_plausible_article( item ) {
	const value = Number.parseInt( item ) ;
	return ! isNaN( value ) && value >= 1 && value <= 599 ;
}

// List the laws that may occur in criminal cases.
// https://nl.wikipedia.org/wiki/Lijst_van_afkortingen_voor_wetten_en_regelingen/
const law_list = [
	['AAW',		'Algemene Arbeidsongeschiktheidswet'],
	['ABW',		'Algemene Bijstandswet'],
	['AKW',		'Algemene Kinderbijslagwet'],
	['AOW',		'Algemene Ouderdomswet'],
	['AWBZ',	'Algemene Wet Bijzondere Ziektekosten'],
	['AWR',		'Algemene wet inzake rijksbelastingen'],
	['AWW',		'Algemene Weduwen en Wezenwet'],
	['Advw',	'Advocatenwet'],
	['Anw',		'Algemene nabestaandenwet'],
	['Arbowet',	'Arbeidsomstandighedenwet'],
	['Aw',		'Auteurswet'],
	['Awb',		'Algemene wet bestuursrecht'],
	['BW',		'Burgerlijk Wetboek'],
	['EUV',		'Europese uitleveringsverdrag', 'Europees uitleveringsverdrag', 'Europees Verdrag betreffende de uitlevering', 'Europees Verdrag uitlevering', 'EU-uitleveringsverdrag', 'Europees Verdrag betreffende uitlevering' ],
	['EUV-2',	'Tweede Aanvullend Protocol bij het Europees Verdrag betreffende uitlevering', 'Tweede Aanvullend Protocol' ],
	['EUR',		'Europees Verdrag aangaande de wederzijdse rechtshulp','Europees Verdrag aangaande de wederzijdse rechtshulp in strafzaken'],
	['FinBES',	'Wet financiën openbare lichamen'],
	['Fw',		'Faillissementswet'],
	['Gnw',		'Geneesmiddelenwet'],
	['HPW',		'Huurprijzenwet Woonruimte'],
	['Hvw',		'Huisvestingswet'],
	['Hw',		'Huurwet'],
	['IB',		'Wet inkomstenbelasting'],
	['LW69',	'Leerplichtwet'],
	['Lw',		'Leegstandswet'],
	['MOT',		'Wet melding ongebruikelijke transacties'],
	['OLW',		'Overleveringswet'],
	['Opw',		'Ow', 'Opiumwet'],
	['PW',		'Participatiewet'],
	['Pw',		'Pachtwet'],
	['RO',		'Wet op de rechterlijke organisatie'],
	['RVV',		'Regeling voertuigen'],
	['RWN',		'Rijkswet op het Nederlanderschap'],
	['Rv',		'Wetboek van Burgerlijke Rechtsvordering'],
	['Sr',		'WvS', 'WvSR', 'Wetboek Str?afrecht', 'wetboek van Strafrecht'], // Take care of typos
	['Sv',		'Wetboek Strafvordering', 'wetboek van Strafvordering'],
	['Sw',		'Sanctiewet'],
	['Tw',		'Telecommunicatiewet'],
	['UHW',		'Uitvoeringswet huurprijzen woonruimte'],
	['UW',		'Uitleveringswet'],
	['VWB',		'Vuurwerkbesluit', 'Vuurwerkbesluit Wet milieugevaarlijke stoffen'],
	['Vpb',		'Wet op de vennootschapsbelasting'],
	['Vw',		'Vreemdelingenwet'],
	['WA',		'Wet op accijns', 'wet accijns', 'wet op de accijns'],
	['WABM',	'Wet algemene bepalingen milieuhygiëne'],
	['WAM',		'Wet aansprakelijkheidsverzekering motorrijtuigen'],
	['WAO',		'Wet op de arbeidsongeschiktheidsverzekering'],
	['WAT',		'Wet op de architectentitel'],
	['WBO',		'Wet op het bevolkingsonderzoek'],
	['WBSO',	'Wet Bevordering Speur en Ontwikkelingswerk'],
	['WEB',		'Wet educatie en beroepsonderwijs'],
	['WED',		'Wet op de economische delicten','Wet op economische delicten'],
	['WGR',		'Wet Gemeenschappelijke Regelingen'],
	['WGR',		'Wet Gemeenschappelijke Regelingen'],
	['WHC',		'Wet op de Huurcommissies'],
	['WHC',		'Wet op de Huurcommissies'],
	['WHV',		'Wet herverdeling wegenbeheer'],
	['WHV',		'Wet herverdeling wegenbeheer'],
	['WHW',		'Wet op het hoger onderwijs en wetenschappelijk onderzoek'],
	['WHW',		'Wet op het hoger onderwijs en wetenschappelijk onderzoek'],
	['WIA',		'Wet Werk en Inkomen naar Arbeidsvermogen'],
	['WIA',		'Wet Werk en Inkomen naar Arbeidsvermogen'],
	['WION',	'Wet informatieuitwisseling ondergrondse netten'],
	['WION',	'Wet informatieuitwisseling ondergrondse netten'],
	['WKA',		'Wet ketenaansprakelijkheid'],
	['WKA',		'Wet ketenaansprakelijkheid'],
	['WKCZ',	'Wet klachtrecht cliënten zorgsector'],
	['WKCZ',	'Wet klachtrecht cliënten zorgsector'],
	['WM',		'Wet Milieubeheer'],
	['WM',		'Wet Milieubeheer'],
	['WMO',		'Wet medischwetenschappelijk onderzoek met mensen'],
	['WMO',		'Wet medischwetenschappelijk onderzoek met mensen'],
	['WNRA',	'Ambtenarenwet','Wet Normalisering Rechtspositie Ambtenaren'],
	['WOB',		'Wet openbaarheid van bestuur'],
	['WOHV',	'Wet op het overleg huurders verhuurder'],
	['WOR',		'Wet op de ondernemingsraden'],
	['WOZ',		'Wet waardering onroerende zaken'],
	['WRO',		'Wet op de Ruimtelijke Ordening'],
	['WTU',		'Wet toelating en uitzetting'],
	['WVGS',	'Wet vervoer gevaarlijke stoffen'],
	['WVMK',	'Wet voorkoming misbruik chemicalien'],
	['WVW',		'Wegenverkeerswet', 'Wegenverkeersweg'],
	['WW',		'Werkloosheidswet'],
	['WWB',		'Wet werk en bijstand'],
	['WaW',		'Warenwet'],
	['Wabo',	'Wet algemene bepalingen omgevingsrecht'],
	['Wajong',	'Wet werk en arbeidsondersteuning jonggehandicapten'],
	['Walvis',	'Wet administratieve lastenverlichting'],
	['Waz',		'Wet arbeidsongeschiktheidsverzekering zelfstandigen'],
	['Wazo',	'Wet arbeid en zorg'],
	['Wbb',		'Wet Bodembescherming'],
	['Wbp',		'Wet bescherming persoonsgegevens'],
	['Wd',		'Wet Dieren'],
	['Wft',		'Wet op het financieel toezicht', 'Wet op financieel toezicht'],
	['Wgb',		'Wet Gewasbeschermingsmiddelen'],
	['Wiv',		'Wet op de inlichtingen en veiligheidsdiensten'],
	['Wlz',		'Wet langdurige zorg'],
	['Wmo',		'Wet maatschappelijke ondersteuning'],
	['Wok',		'Wet op de Kansspelen',	'Wet op Kansspelen'],
	['WolBES',	'Wet openbare lichamen'],
	['Wonw',	'Woningwet'],
	['Wp2000',	'Wet Personenvervoer'],
	['Wpg',		'Wet Politiegegevens'],
	['Wro',		'Wet ruimtelijke ordening'],
	['Wspn',	'Wet Schuldsanering'],
	['Wth',		'Wet tijdelijk huisverbod'],
	['Wtt',		'Wet Toezicht trustkantoren'],
	['WvK',		'Wetboek van Koophandel'],
	['WvMSr',	'Wetboek Militair Strafrecht', 'Wetboek van Militair Strafrecht'],
	['Wwft',	'Wet ter voorkoming witwassen en financieren terrorisme', 'Wet ter voorkoming van Witwassen'],
	['Wwik',	'Wet werk en inkomen kunstenaars'],
	['Wwm',		'Wet Wapen en munitie', 'wet Wapens en munitie'],
	['Wwz',		'Wet Werk en zekerheid'],
	['ZFW',		'Ziekenfondswet'],
	['ZW',		'Ziektewet'],
	['Zvw',		'Zorgverzekeringswet']

] ;

const only_law_pat	= new RegExp( '^' + law_list.flat().join( '$|^' ) + '$', 'i' ) ;
const all_law_pat	= new RegExp( '(?<=(\\b' + law_list.flat().join( '\\b|\\b' ) + '\\b))', 'i' ) ;
const law_pat_list	= law_list.map( (item, index) => new RegExp( '(\\b' + item.join( '\\b|\\b' ) + '\\b)', 'i' ) ) ;

const art_law_pat	= new RegExp( '((?:[1-9][0-9\.]*[a-z]*)(?:, *[1-9][0-9\.]*[a-z]*)* *(?:\\b' + law_list.flat().join( '\\b|\\b' ) + '\\b))', 'i' ) ;
const law_art_pat	= new RegExp( '((?:\\b' + law_list.flat().join( '\\b|\\b' ) + '\\b) *(?:[1-9][0-9\.]*[a-z]*)(?:, *[1-9][0-9\.]*[a-z]*)*\\b)', 'i' ) ;

const sub_art_list	= [ '[a-z]','bis','ter','quater','quinquies','sexies','aa','ab','ad','ag','cd','dd','ee','fa','gg','hh','lb','ma','mm','ng','ta','tb','wa','wc','we','za','zz' ] ;
const art_pat		= /([1-9]\d*[a-z\/]*(?:\.\d+)*)/i ;

// Given the text of the wettelijke voorschrtiften section, return the articles of each law that occurs in the text, only once!
function extract_articles( txt ) {
	var prv = txt ;
	// Fix spacing and spelling issues
	var cur = prv
		.replace( /[˚:]/g, ' ' )
		.replace( / *,[, ]*/g, ', ' )
		.replace( /(\d)en\b/gi, '$1, ' )
		.replace( /toegepast, zoals deze luidden ten tijde van het bewezenverklaarde/gi, ' ' )
		.replace( /\b *(?:juncto|jo|in verbinding met)\b/gi, ', ' )
		.replace( /([, ]*\[?(\bart\b\.?|\bartikel\b|\bartikelen\b)\]?|toegepaste?|(?:, *)?\bvan\b|\bden?\b|\bhet\b|\(?\b(?:oud|nieuw|oud en nieuw)\b\)?[, ]*)/gi, ' ' )
		.replace( /\(geldend[^\)]*\)/gi, ' ' )
		.replace( /ten (?:eerste|tweede|derde)/gi, ' ' )
		.replace( /[, ]*\(feit[^\)]*\),/gi, ',' )
		.replace( /[, ]*\(feit[^\)]*\)[^,]/gi, ' ' )
		.replace( /(\d),\.(\d)/gi, '$1.$2' )
		.replace( /[, ]*eerste lid/gi, '.1' )
		.replace( /[, ]*tweede lid/gi, '.2' )
		.replace( /[, ]*derde lid/gi, '.3' )
		.replace( /[, ]*lid *(\d+)/gi, '.$1' )
		.replace( /[, ]*lid */gi, ' ' )
		.replace( /[, ]*onderdeel */gi, ' ' ) // Added 20201223
		.replace( /(WVW|Wegenverkeerswet) 1994/gi, '$1' ) // Added 20201223
		.replace( /13 december 1957 \(Trb. 1965, 9\)/gi, ' ' ) // Added 20201224
		.replace( /20 april 1959 \(Straatsburg 20 april 1959, Trb. 1965\)?/gi, ' ' ) // Added 20201226
		.replace( /bij dat verdrag \(Trb. 1979, 120\)/gi, ' ' ) // Added 20201224
		.replace( /(Sw|Sanctiewet) 1997/gi, '$1' )
		.replace( /[, ]*(?:sub|onder|aanhef en onder) *([a-z])/gi, '$1' ) // Added 20201223
		.replace( / +(?:sub|onder|aanhef en onder) +/gi, '' )
		.replace( /(\d) +(\d)/gi, '$1, $2' )
		.replace( new RegExp( '(\\d) *(' + sub_art_list.join( '|' ) + ')\\b\.\\d', 'gi'), '$1$2' )
		.replace( new RegExp( '(\\d) *(' + sub_art_list.join( '|' ) + ')\\b *(\\d)', 'gi'), '$1$2, $3' )
		.replace( new RegExp( '(\\d) *\\(?(' + sub_art_list.join( '|' ) + ')\\b\\)?', 'gi'), '$1$2' )
		.replace( new RegExp( '(\\d)(' + sub_art_list.join( '|' ) + '),* +en', 'gi'), '$1$2, ' )
		.replace( new RegExp( '(\\d)(' + sub_art_list.join( '|' ) + ') *\\\/ *(\\d)', 'gi'), '$1$2, $3' )
		.replace( /(\d),* +en/gi, '$1, ' )
		.replace( /([0-9\.]+)a en b/gi, '$1a, $1b' )
		.replace( /  +/g, ' ' ) ;
	if( cur !== prv ) {
		if( debug_debug) console.log( '%s\n->\n%s', prv, cur )
	}
	// Unfold repeated articles, e.g. 420bis/quater -> 420bis, 420quater
	prv = cur ;
	for( var i = 1 ; i < 10 ; i++ ) {
		var tmp = cur ;
		cur = tmp.replace( new RegExp( '(\\d+) *(' + sub_art_list.join( '|' ) + ') */ *(' + sub_art_list.join( '|' ) + '\\b)', 'gi'), '$1$2, $1$3' )
		if( tmp === cur ) {
			break ;
		}
	}
	if( cur !== prv ) {
		if( debug_debug) console.log( '%s\n-->\n%s', prv, cur )
	}
	// Remove abbreviations of the laws, e.g. Opiumwet (Opw) -> Opiumwet
	prv = cur ;
	cur = prv.replace( new RegExp ( '(\\b' + law_list.flat().join( '\\b|\\b' ) + '\\b) *\\( *(' + law_list.flat().join( '|' ) + ') *\\)', 'gi' ), '$1' ) ;
	var result_3 = [] ;
	// Split the text into sentences 
	const cur_list = cur.split( /(?:\.|;) / ) ;
	cur_list.map( cur => {
		if( debug_debug) {
			console.log( 'extract_articles_0	%s', support.truncate( cur, truncate_max ) ) ;
		}
		const result_1 = cur.split( art_law_pat ).filter( item => trace_match( item, art_law_pat ) ) ;
		if( debug_debug && result_1.length > 0 ) {
			console.log( 'extract_articles_1	#%d %s', result_1.length, support.truncate( JSON.stringify( result_1 ), truncate_max ) ) ;
		}
		const result_2 = cur.split( law_art_pat ).filter( item => trace_match( item, law_art_pat ) ) ;
		if( debug_debug && result_2.length > 0 ) {
			console.log( 'extract_articles_2	#%d %s', result_2.length, support.truncate( JSON.stringify( result_2 ), truncate_max ) ) ;
		}
		// Decide whether in the current sentence laws precede the articles or the other way round
		if( result_2.length > result_1.length ) {
			result_3 = result_3.concat( result_2 ) ;
		} else if( result_2.length < result_1.length ) {
			result_3 = result_3.concat( result_1 ) ;
		} else {
			result_3 = result_3.concat( result_1, result_2 ) ;
		}
	} ) ;
	if( debug_debug) {
		console.log( 'extract_articles_3	#%d %s', result_3.length, support.truncate( JSON.stringify( result_3 ), truncate_max ) ) ;
	}
	var result_4 = {} ;
	result_3.map( law_art_or_art_law =>
		law_pat_list.map( ( pattern, index ) => {
			const key = law_list[ index ][ 0 ] ;
			if( trace_match( law_art_or_art_law, pattern ) ) {
				const art_list = law_art_or_art_law
					.split( art_pat )
					.filter( item => trace_match( item, art_pat ) && is_plausible_article( item ) ) ;
				if( typeof result_4[ key ] === 'undefined' ) {
					result_4[ key ] = [] ;
				}
				result_4[ key ] = result_4[ key ].concat( art_list ) ;
			}
		} )
	) ;
	if( debug_debug) {
		console.log( 'extract_articles_4	%s', support.truncate( JSON.stringify( result_4 ), truncate_max ) ) ;
	}
	return Object.keys( result_4 ).map( key => [ key ].concat( result_4[ key ] ) ) ;
}

// List the XML tags that may indicate sections.
const tag_list = [ 'title', 'bridgehead', 'emphasis', 'para' ];

// Substitute the tokens in an criminality_facts list.
function substitute_criminality_facts_section_list_list( abstract_criminality_facts_section_list ) {
	const common = '(?:\\b[0-9]+[a-z]*\\b|\\.|:|<\\\/?nr\\b[^>]*>|<\\?linebreak\\?>|de|hoofdstuk|\\s)*' ;
	const pat_list = mk_pat_list( 'criminality_facts_section',
			'<section[^>]*>' + common,
			abstract_criminality_facts_section_list,
			'\\n', false, 'gi' ) ;
	const pat_list_list = tag_list.map( tag =>
		mk_pat_list( 'criminality_facts_' + tag,
			'<' + tag + '[^>]*>' + common,
			abstract_criminality_facts_section_list,
			'(?:-|:|\\.|\\s)*<\/' + tag + '>', false, 'gi' ) ) ;
	return [ pat_list ].concat( pat_list_list ) ;
}

// List the section headings that indicate the discusson of the criminality of the facts
const abstract_criminality_facts_section_list = [
	'strafbaarheid (?:van )?(?:(?:de )?feiten|(?:het )?feit)',
	'strafbaarheid van het bewezen ?verklaarde',
	'strafbaarheid van het bewezenverklaarde en van (?:de )?verdachte',
	'kwalificaties?[^<]*'

] ;
const criminality_facts_section_pat_list_list	= substitute_criminality_facts_section_list_list( abstract_criminality_facts_section_list ) ;

// Substitute the tokens in an criminality_offender list.
function substitute_criminality_offender_section_list_list( abstract_criminality_offender_section_list ) {
	const common = '(?:\\b[0-9]+[a-z]*\\b|\\.|:|<\\\/?nr\\b[^>]*>|<\\?linebreak\\?>|de|hoofdstuk|\\s)*' ;
	const pat_list = mk_pat_list( 'criminality_offender_section',
			'<section[^>]*>' + common,
			abstract_criminality_offender_section_list,
			'\\n', false, 'gi' ) ;
	const pat_list_list = tag_list.map( tag =>
		mk_pat_list( 'criminality_offender_' + tag,
			'<' + tag + '[^>]*>' + common,
			abstract_criminality_offender_section_list,
			'(?:-|:|\\.|\\s)*<\/' + tag + '>', false, 'gi' ) ) ;
	return [ pat_list ].concat( pat_list_list ) ;
}

// List the section headings that indicate the discusson of the criminality of the offender
const abstract_criminality_offender_section_list = [
	'strafbaarheid (?:van )?(?:de )?verdachte',
	'strafbaarheid'
] ;
const criminality_offender_section_pat_list_list	= substitute_criminality_offender_section_list_list( abstract_criminality_offender_section_list ) ;

// Substitute the tokens in an decision list.
function substitute_law_section_list_list( abstract_law_section_list ) {
	const common =	'(?:\\b[0-9]+[a-z]*\\b|\\.|:|<\\\/?nr\\b[^>]*>|<\\?linebreak\\?>|' +
				'beoordeling|de|het|hoofdstuk|thans|toeg?e?paste?|t?oepasselijke?|toepasselijkheid|toepassing|van|\\s)*' ; // Take care of typos
	const pat_list = mk_pat_list( 'law_section',
			'<section[^>]*>' + common,
			abstract_law_section_list,
			'\\n', false, 'gi' ) ;
	const pat_list_list = tag_list.map( tag =>
		mk_pat_list( 'law_' + tag,
			'<' + tag + '[^>]*>' + common,
			abstract_law_section_list,
			'(?:-|:|\\.|\\s)*<(?:\/' + tag + '|\\?linebreak\\?)>', false, 'gi' ) ) ;
	return [ pat_list ].concat( pat_list_list ) ;
}

const abstract_law_section_list = [
	'Bij onderstaande beslissing heeft de rechtbank acht geslagen op de relevante wetsartikel[^<]*',
	'WETTELIJKE *VOOR<\\\/emphasis><emphasis role="bold">SCHRIFTEN',
	'WETTELIJKE *VOORSC<\\\/emphasis> *<emphasis role="bold">HRIFTEN',
	'WETTELIJKE<\\\/emphasis><emphasis[^>]*><emphasis[^>]*> *VOORSC<\\\/emphasis>HRIFTEN',
	'alles afwegend acht de rechtbank de hierna te noemen straff?e?n? passend en geboden.Toepasselijke wettelijke voorschriften',
	'rechtbank heeft gelet op de artikelen[^<]*',
	'gelet is op de artikelen[^<]*',
	'beslissing berust op de artikelen[^<]*',
	'artikelen',
	'op te leggen maatregel berust op artikel[^<]*',
	'gronden voor een straf of maatregel',
	'na te noemen straf(?:fen zijn|is) gegrond[^<]*',
	'oplegging van straf of maatregel is, behalve op de reeds aangehaalde wettelijke voorschriften, gegrond op de artikel[^<]*',
	'recht',
	'strafoplegging',
	'[^<]*artikelen van de wet, welke worden toegepast',
	'wet',
	'wetsartikele?n?[^<]*',
	'wetsbepalinge?n?',
	'wettelijke? *bepalinge?n?',
	'(?:wettelijke?)+ *voorschrifte?n?[^<]*',
	'welke artikelen uit de wet heeft de rechtbank gebruikt\\?'

] ;
const law_section_pat_list_list	= substitute_law_section_list_list( abstract_law_section_list ) ;

// Substitute the tokens in an decision list.
function substitute_decision_section_list_list( abstract_decision_section_list ) {
	const common = '(?:\\b[0-9]+[a-z]*\\b|\\.|:|<\\\/?nr\\b[^>]*>|<\\?linebreak\\?>|de|hoofdstuk|\\s)*' ;
	const pat_list = mk_pat_list( 'decision_section',
			'<section[^>]*>' + common,
			abstract_decision_section_list,
			'\\n', false, 'gi' ) ;
	const pat_list_list = tag_list.map( tag =>
		mk_pat_list( 'decision_' + tag,
			'<' + tag + '[^>]*>' + common,
			abstract_decision_section_list,
			'(?:-|:|\\.|<\\?linebreak\\?>|\\s)*<\/' + tag + '>', false, 'gi' ) ) ;
	return [ pat_list ].concat( pat_list_list ) ;
}

// List the section headings used by the courts to annouce a decision.
const abstract_decision_section_list = [
	'beslissinge?n?(?::|<\\?linebreak\\?>|de|een|en ondertekening|heropening|' +
		'in het kort|kantonrechter|na|over|politierechter|rechtbank|' +
		'rechters-commissarissen|straf|tot|van|veroordeling|voorwaardelijke|\\s)*',
	'(?:oplegging|opgelegde) straf (?:en|of) maatregel',
	'b e s l i s s i n g',
	'BES<\\\/emphasis><emphasis role="bold">LISSING',
	'beschikkende',
	'beschikking(?:van de rechter-commissaris| )*',
	'beschikt als volgt',
	'BESL<\\\/emphasis>I<emphasis role="bold">SSING',
	'beslag[^<]*DE UITSPRAAK',
	'beslissing is gegrond[^<]*DE UITSPRAAK',
	'eslissing',
	'conclusie van de rechtbank',
	'herstel(?:beschikking|vonnis|uitspraak)',
	'legt op de volgende straf(?:fen)?',
	'legt op de volgende straf\\(fen\\) en\\\/of maatregel\\(en\\)',
	'opgelegde straf of maatregel; opgave van de bijzondere redenen, die de straf hebben bepaald of tot de maatregel hebben geleid',
	'r e c h t d o e n d e',
	'rechtdoende',
	'rechter spreekt de beslissing uit',
	'straf en\\\/of maatregel',
	'tussenbeslissing',
	'uitspraak(?:van|de|economische|politierechter|rechtbank|luidt| )*',
	'vervolgens deelt de voorzitter de volgende beslissingen van de rechtbank mee',
	'voorzitter deelt als beslissingen van de rechtbank mee dat de rechtbank'

] ;
const decision_section_pat_list_list	= substitute_decision_section_list_list( abstract_decision_section_list ) ;

// List the essentials of a footnote.
const footnote_section_pat_list		= mk_pat_list( 'footnote', '<footnote id=[^>]*>', [ '[\\s\\S]*' ] , '<\\\/footnote>', false, 'gi' ) ;

// Substitute the tokens in an abstract treatment list.
function substitute_treatment_list( tag, abstract_treatment_list ) {
	const treatment_list = abstract_treatment_list.map( item => item
		.replace( /<dash>/g,		'(?:-|:|\\.|,| )*' )
		.replace( /<feit>/g,		'(?:\\b[0-9\\\/-]+\\b|,|beide|bewezen|de|en|feite?n?|'+
						'het|in|met|onder|primair|parket[-]*nummer|subsidiair|verklaarde|zaak| )+')
		.replace( /<maatregel>/g,	'(?:de|volgende| )*maatregele?n?' )
		.replace( /<verpleging>/g,	'verpleging(?:van|overheids|wege| )*' )
		.replace( /<voor>/g,		'(?:_t_a_v_?|voor|wegens|ter zake van|ten aanzien van)' )
		.replace( /<duur>/g,		'(?:de duur|de tijd|de termijn|gedurende|van|voor| )*' )
		.replace( /<inrichting>/g,	'(?:de|een|het|in| )*' +
						'(?:inrichting|psychiatrisch ziekenhuis)' +
						'(?:\\((?:ISD)[ -]*(?:maatregel)?\\)|daders?|stelselmatige|voor| )*' )
		.replace( /<jeugd_inrichting>/g,'(?:een|in| )*(?:PIJ[ -]*inrichting|inrichting voor (?:jeugdige daders|jeugdigen))' )
		.replace( /<plaatsing>/g,	'(?:de|gelast|tot|van|voorwaardelijke| )*(?:plaatsing)' )
		.replace( /<geplaatst>/g,	'(?:worden|wordt|zal| )*geplaatst' )
		.replace( /<terbeschikking>/g,	'(?:de|worden|wordt|zal| )*ter *beschikking *' +
						'(?:bevel|gesteld|met|stelling|tot|verpleging|worden|wordt|zal| )*' )
		.replace( /<verdachte>/g,	'(?:de|van|aan| )*(?:verdachte|veroordeelde|ter beschikking gestelde)' +
						'(?:\\[|\\]|naam|verdachte|,| )*' )
		.replace( /<maanden>/g,		'(?:\\b[a-z]+(?: [a-z]+ )?(?:[\\(|\\[][0-9]+[\\)|\\]] *)?(?:maand|maanden)|' +
						'\\b[0-9]+ (?:[\\(|\\[][a-z]+(?: [a-z]+)?[\\)|\\]] *)?(?:maand|maanden))' )
		.replace( /<jaren>/g,		'(?:\\b[a-z]+(?: [a-z]+ )?(?:[\\(|\\[][0-9]+[\\)|\\]] *)?(?:jaar|jaren)|' +
						'\\b[0-9]+ (?:[\\(|\\[][a-z]+(?: [a-z]+)?[\\)|\\]] *)?(?:jaar|jaren))' )
		.replace( /<periode>/g,		'(?:\\(|\\[|\\)|\\]|\\b[0-9]+\\b|\\b[a-z]+\\b|\\.|,|:| )*' +
						'\\b(?:uu?r|dag|wee?k|maand|jaa?r)(?:en)?\\b' +
						'(?:\\(|\\[|\\)|\\]|\\b[0-9]+\\b|\\b[a-z]+\\b|\\.|,|:| )*' ) ) ;
	return mk_pat_list( tag, '\\b', treatment_list, '\\b' ) ;
}

// List the imposition of special treatment for minors .
// Legt op de maatregel van plaatsing in een inrichting voor jeugdigen voor de duur van 3 (drie) jaren
const abstract_youth_treatment_list	= [
	'legt op aan <verdachte> <maatregel> betreffende het gedrag van de jeugdige <duur>(<periode>)',
	'beslissing<dash><plaatsing> <jeugd_inrichting> <duur>(<periode>)',
	'gelast <plaatsing> <verdachte> <jeugd_inrichting> <duur>(<periode>)',
	'gelast dat <verdachte> <duur> (<maanden>) <jeugd_inrichting> <geplaatst>',
	'gelast dat <verdachte> <geplaatst> <jeugd_inrichting> <duur>(<periode>)',
	'gelast dat <verdachte> <jeugd_inrichting> <geplaatst> <duur>(<periode>)',
	'gelast dat <verdachte> <voor> <feit> <geplaatst> <jeugd_inrichting> <duur>(<periode>)',
	'legt <verdachte> <maatregel> op <plaatsing> <jeugd_inrichting><dash> <duur>(<periode>)?',
	'legt <verdachte> op <maatregel> van<dash>plaatsing <jeugd_inrichting> <duur>(<periode>)?',
	'legt <verdachte> op<dash><maatregel><dash><plaatsing> <jeugd_inrichting><dash> <duur>(<periode>)?',
	'legt op <maatregel> <plaatsing> <verdachte> <jeugd_inrichting> <duur>(<periode>)?',
	'legt op <maatregel><dash><plaatsing> <jeugd_inrichting> <duur>(<periode>)?',
	'legt op <maatregel><dash><voor> <feit><dash><plaatsing> <jeugd_inrichting> <duur>(<periode>)?',
	'legt op <verdachte><dash><maatregel> <plaatsing> <jeugd_inrichting> <duur>(<periode>)?',
	'legt op <verdachte><dash><voor> <feit><dash><maatregel> <plaatsing> <jeugd_inrichting> <voor>(<periode>)',
	'veroordeelt <verdachte> <plaatsing> <jeugd_inrichting> <duur>(<periode>)?',
	'veroordeelt <verdachte> <voor> <feit> <plaatsing> <jeugd_inrichting> <duur>(<periode>)?',
	'veroordeelt <verdachte> <voor> <feit> tot <maatregel> <plaatsing> <jeugd_inrichting> <duur>(<periode>)?',
	'veroordeelt <verdachte> <voor> <feit> tot<dash><plaatsing> <jeugd_inrichting> <duur>(<periode>)?',
	'veroordeelt <verdachte> tot<dash><maatregel> <plaatsing> <jeugd_inrichting> <duur>(<periode>)?',
	'veroordeelt <verdachte> tot<dash><plaatsing> <jeugd_inrichting> <duur>(<periode>)?'
] ;
const youth_treatment_pat_list	= substitute_treatment_list( 'youth_treatment', abstract_youth_treatment_list ) ;

// List the imposition of special treatment, in e.g. a psychiatric hospitals (abstract).
const abstract_treatment_list	= [
	'beslissing<dash><plaatsing> <inrichting> <duur>(<periode>)',
	'beveelt dat <verdachte> van overheidswege zal worden verpleegd',
	'gelast <plaatsing> <verdachte> <inrichting> <duur>(<periode>)',
	'gelast <terbeschikking> <verdachte> <duur>(<periode>)?',
	'gelast <voor> <feit> <terbeschikking> <verdachte>',
	'gelast dat <verdachte> <duur> (<jaren>) <inrichting> <geplaatst>',
	'gelast dat <verdachte> <duur>(<jaren>)<terbeschikking>',
	'gelast dat <verdachte> <geplaatst> <inrichting> <duur>(<periode>)',
	'gelast dat <verdachte> <inrichting> <geplaatst> <duur>(<periode>)',
	'gelast dat <verdachte> <terbeschikking> <duur>(<periode>)?',
	'gelast dat <verdachte> <voor> <feit> <geplaatst> <inrichting> <duur>(<periode>)',
	'legt <verdachte> <maatregel> op <plaatsing> <inrichting><dash> <duur>(<periode>)?',
	'legt <verdachte> op <maatregel> van<dash>plaatsing <inrichting> <duur>(<periode>)?',
	'legt <verdachte> op<dash><maatregel><dash><plaatsing> <inrichting><dash> <duur>(<periode>)?',
	'legt op <maatregel> <plaatsing> <verdachte> <inrichting> <duur>(<periode>)?',
	'legt op <maatregel><dash><plaatsing> <inrichting> <duur>(<periode>)?',
	'legt op <maatregel><dash><terbeschikking> <duur>(<periode>)?',
	'legt op <maatregel><dash><voor> <feit><dash><plaatsing> <inrichting> <duur>(<periode>)?',
	'legt op <maatregel><dash><voor> <feit><dash><terbeschikking>',
	'legt op <verdachte><dash><maatregel> <plaatsing> <inrichting> <duur>(<periode>)?',
	'legt op <verdachte><dash><voor> <feit><dash><maatregel> <plaatsing> <inrichting> <voor>(<periode>)',
	'maatregel<dash>gelast <voor> <feit> <terbeschikking> van <verdachte>',
	'veroordeelt <verdachte> <plaatsing> <inrichting> <duur>(<periode>)?',
	'veroordeelt <verdachte> <voor> <feit> <plaatsing> <inrichting> <duur>(<periode>)?',
	'veroordeelt <verdachte> <voor> <feit> tot <maatregel> <plaatsing> <inrichting> <duur>(<periode>)?',
	'veroordeelt <verdachte> <voor> <feit> tot gelast dat <verdachte> <terbeschikking> <duur>(<periode>)?',
	'veroordeelt <verdachte> <voor> <feit> tot<dash><plaatsing> <inrichting> <duur>(<periode>)?',
	'veroordeelt <verdachte> tot<dash><maatregel> <plaatsing> <inrichting> <duur>(<periode>)?',
	'veroordeelt <verdachte> tot<dash><plaatsing> <inrichting> <duur>(<periode>)?'

] ;
const treatment_pat_list	= substitute_treatment_list( 'treatment', abstract_treatment_list ) ;

// Substitute the tokens in an abstract detetion or youth detention list.
function substitute_detention_list( tag, abstract_detention_list ) {
	const detention_list = abstract_detention_list.map( item => item
		.replace( /<verdachte>/g,	'(?:aan|de| )*(?:\\[|\\]|\\(verder:|\\)|,|[0-9]|\\b[a-z]\\b|hem|naam|verdachte|veroordeelde| )+' )
		.replace( /<maatregel>/g,	'(?:bijkomende|de|en|of|maatregel\\(?e?n?\\)?|straf\\(?f?e?n?\\)?|volgende|\\\/| )+' )
		.replace( /<van>/g,		'(?:de duur|de tijd|(?:de|een) termijn|gedurende|van|voor| )*' )
		.replace( /<voor>/g,		'(?:_t_a_v_?|aanzien|betrekking|met|ten|ter|tot|van|voor|wegens|zake| )+' )
		.replace( /<gevangenisstraf>/g,	'(?:;|\\b_?[a-z]_?\\b|een|geheel|militaire|o?n?voorwaardelijke|viermaal| )*' +
						'(?:gevangenisstraf(?:voor)?|detentie|hechtenis|hechteins)' ) // Typo in ECLI:NL:RBAMS:2020:1692
		.replace( /<jeugddetentie>/g,	'(?:een|geheel|van|voorwaardelijke| )*(?:tot)?jeugd[ -]*detentie' )
		.replace( /<tot>/g,		'(?:onder toepassing van[^,]*,|overigens|te|dier|zake| )*tot' )
		.replace( /<feit>/g,		'(?:diefstal[^\\)]+|moord|[\\\/,-]|onderzoek[^\\)]+|' +
						'\\(|\\)|\\.|,|\\b[0-9][0-9a-z\\\/-]*\\b|\\b_?[a-z]_?\\b|aanzien|ad|alsmede|alternatief|' +
						'arrest|bewezene?|bij|boven|cumulatief|dagvaardinge?n?|dat|deze|de|dit|drie|' +
						'en|eerste|feite?n?|gelegde?|het|hier|impliciet|informandum|in\\b|' +
						'laste|meer|met|misdrijven|onder|onderdeel|op|overige|parketnummers?|' +
						'primaire?|strafbaa?re?|sub|subsidiaire?|t?o?e?gevoegde|ten|' +
						'tot|tweede|van|verklaarde?|vermelde|vonnis|voormeld|voor|zaak| )+' )
		.replace( /<dash>/g,		'(?:-|:|\\*|,|\\(|\\)|\\.| )*' )
		.replace( /<periode>/g,		'(?:\\(|\\[|\\)|\\]|\\b[0-9]+\\b|\\b[a-z]+\\b|\\.|,|-|:| )*' +
						'\\b(?:uu?r|dag|wee?k|maand|jaa?r)(?:en)?\\b' +
						'(?:\\(|\\[|\\)|\\]|\\b[0-9]+\\b|\\b[a-z]+\\b|\\.|,|-|:| )*' )
		.replace( /<eenheid>/g,		'(?:\\b(?:uu?r|dag|wee?k|maand|jaa?r)(?:en)?\\b)' ) ) ; // PHH?
	return mk_pat_list( tag, '\\b', detention_list, '\\b' ) ;
}

// List the imposition of youth detention sentences
const abstract_youth_detention_list	= [
	'strafoplegging <jeugddetentie> <van>(<periode>)',
	'legt <verdachte> op <jeugddetentie> <van>(<periode>)',
	'legt <verdachte> op <maatregel><dash><jeugddetentie> <van>(<periode>)',
	'legt op <maatregel><dash><jeugddetentie> <van>(<periode>)',
	'legt op <maatregel><dash><voor> <feit><dash><jeugddetentie> <van>(<periode>)',
	'legt op <maatregel><dash>beslissing<dash><voor> <feit><dash><jeugddetentie> <van>(<periode>)',
	'veroordeelt <verdachte> <jeugddetentie> <van>(<periode>)',
	'veroordeelt <verdachte> <voor> <feit> <tot> ([0-9]+ (?:\\([a-z]+\\) *)?<eenheid>) <jeugddetentie>',
	'veroordeelt <verdachte> <voor> <feit> <tot> ([0-9]+ (?:\\([a-z]+\\) *)?<eenheid> en [0-9]+ (?:\\([a-z]+\\) *)?<eenheid>) <jeugddetentie>',
	'veroordeelt <verdachte> <voor> <feit> <tot> ([a-z]+ [0-9]+ <eenheid>) <jeugddetentie>',
	'veroordeelt <verdachte><dash><tot><dash><jeugddetentie><dash><van>(<periode>)',
	'veroordeelt <verdachte><dash><voor> <feit><dash><tot><dash><jeugddetentie> <van>(<periode>)'

] ;
const youth_detention_pat_list	= substitute_detention_list( 'youth_detention', abstract_youth_detention_list ) ;

// List the imposition of detention sentences
const abstract_detention_list	= [
	'beslissing<dash><gevangenisstraf> <van>(<periode>)',
	'beslissing<dash><voor><feit><dash><gevangenisstraf> <van>(<periode>)',
	'legt <verdachte> <maatregel> op<dash><gevangenisstraf> <van>(<periode>)',
	'legt <verdachte> <maatregel> op<dash><voor> <feit><dash><gevangenisstraf> <van>(<periode>)',
	'legt <verdachte> <voor> <feit> <maatregel> op<dash><gevangenisstraf> <van>(<periode>)',
	'legt <verdachte> <voor> <feit> op<dash><gevangenisstraf> <van>(<periode>)',
	'legt <verdachte> op <maatregel><dash><gevangenisstraf> <van>(<periode>)',
	'legt <voor> <feit> op <maatregel><dash><gevangenisstraf> <van>(<periode>)',
	'legt op <gevangenisstraf> <van>(<periode>)',
	'legt op <maatregel><dash><feit><dash><gevangenisstraf> <van>(<periode>)',
	'legt op <maatregel><dash><gevangenisstraf> <van>(<periode>)',
	'legt op <maatregel><dash><voor><dash><feit><dash><gevangenisstraf> <van>(<periode>)',
	'legt op <maatregel><dash>beslissing<dash><gevangenisstraf> <van>(<periode>)',
	'legt op <maatregel><dash>beslissing<dash><voor><feit><dash><gevangenisstraf> <van>(<periode>)',
	'oplegging straf<dash>veroordeelt <verdachte> <tot> <gevangenisstraf> <van>(<periode>)',
	'strafoplegging <gevangenisstraf> <van>(<periode>)',
	'verklaart verdachte hiervoor strafbaar<dash><voor> <feit><dash><gevangenisstraf> <van>(<periode>)',
	'veroordeelt <verdachte> <tot> ([0-9]+ <eenheid>) <gevangenisstraf>',
	'veroordeelt <verdachte> <tot> een (levenslange) gevangenisstraf',
	'veroordeelt <verdachte> <tot><dash><gevangenisstraf> <van>(een <eenheid>)<dash>',
	'veroordeelt <verdachte> <tot><dash><voor> <feit><dash> <gevangenisstraf> <van>(<periode>)',
	'veroordeelt <verdachte> voor de op de dagvaarding van meerderjarige verdachte vermelde <feit> tot <gevangenisstraf> <van>(<periode>)',
	'veroordeelt <verdachte><dash><feit> <tot> <gevangenisstraf> <van>(<periode>)',
	'veroordeelt <verdachte><dash><tot><dash><feit><dash><gevangenisstraf> <van>(<periode>)',
	'veroordeelt <verdachte><dash><tot><dash><gevangenisstraf> <van>(<periode>)',
	'veroordeelt <verdachte><dash><tot><dash><gevangenisstraf><dash><van>(<periode>)',
	'veroordeelt <verdachte><dash><tot><dash><voor> <feit><dash><gevangenisstraf> <van>(<periode>)',
	'veroordeelt <verdachte><dash><voor> <feit><dash><gevangenisstraf> <van>(<periode>)',
	'veroordeelt <verdachte><dash><voor> <feit><dash><tot><dash><gevangenisstraf> <van>(<periode>)',
	'veroordeelt <verdachte><dash><voor> <feit><dash><tot><dash>hechtenis <van>(<periode>)',
	'veroordeelt <verdachte><dash><voor> <tot><dash><gevangenisstraf> <van>(<periode>)'

];
const detention_pat_list	= substitute_detention_list( 'detention', abstract_detention_list ) ;

// Substitute the tokens in an abstract fine list.
function substitute_fine_list( abstract_fine_list ) {
	const fine_list = abstract_fine_list.map( item => item
		.replace( /<dash>/g,		'(?:-|:|\\.|\\*|;| )*' )
		.replace( /<instantie>/g,	'(?:de|het| )*rechtbank' )
		.replace( /<maatregel>/g,	'(?:\\\/|bijkomende|de|en|maatregel\\(?e?n?\\)?|of|tot|schadevergoeding|' +
						'straf\\(?f?e?n?\\)?|volgende| )+' )
		.replace( /<verdachte>/g,	'(?:,|\\[|\\]|[0-9]|_b_v_|aan|bank|bedrijf|betrokkene|de|gestrafte|haar|hem|het|hoofdelijk|'+
						'man|naam|organisatie|rechtspersoon|vennootschap|verdachte|veroo?rdeelde|voornoemd| )+' )
		.replace( /<voor>/g,		'(?:_t_a_v_?|aanzien|inzake|ten|ter|van|voor|wegen?s?|zake| )+' )
		.replace( /<feit>/g,		'(?:\\b[a-z]\\b|alternatief|betrekking|bewezen|de|eerste|en|feite?n?|' +
						'gelegde|het|hiervoor|laste|meer|met|misdrij(?:f|ven)|nog|onder|overtuigend|' +
						'parketnummer *[0-9\\\/-]+|primaire?|strafbaar|\\bsub\\b|subsidiaire?|' +
						'ten|tot|vanwege|van|verklaarde|wettige?|zaak|[\\(\\)0-9\\\/,: -])+' )
		.replace( /<verplichting>/g,	'(?:de|hoofdelijke|betalings| )*verplichting' )
		.replace( /<tot>/g,		'(?:de|in|mitsdien|plaats|tweede|voorts| )*tot' )
		.replace( /<geldboete>/g,	'(?:\\(|\\)|_[a-z]_|betalen|betaling|de|een|gedeeltelijke?|geheel|het|van|'+
						'voorwaardelijke?| )*g?e?l?d?boetes?' )
		.replace( /<betaling>/g,	'(?:de|een|het|hoofdelijke| )*(?:betaling|betalen)' )
		.replace( /<aan_de_staat>/g,	'(?:aan de staat(?: der Nederlanden)?)' )
		.replace( /<ter_ontneming>/g,	'(?:,|daar|dat|door|geschatte|haar|hem|het|om|ontneming|' +
						'ter|van|verkregen|voordeel|wederrechtelijk|zake| )+' )
		.replace( /<van>/g,		'(?:,|-|:|aan|bedrage?n?|behoeve|benadeelde|bovenstaande?|de\\b|' +
						'deze|een|en|geldbedrag|groott?e?|het|hier|hoofdelijk|hoogte|' +
						'immateriele|in|na|op|partije?n?|schadevergoeding|slachtoffers?|' +
						'som|ten|ter|tot\\b|totaal|van|volgende|voor\\b|voornoemde?|zake|zeg|' +
						'\\[ *(?:aangeefster|aangever|adres|agent|apotheek|bank|bedrijf|' +
						'benadeelde|betrokkene|broer|getuige|moeder|naam|nabestaande|' +
						'partij|partner|persoon|slachtoffers?|vader|verbalisant|verdachte|' +
						'zus)[a-z0-9 ]*\\]| )*' )
		.replace( /<bedrag>/g,		'[\\(\\[\\)\\]0-9a-z \\.,:=-]+' ) ) ;
	return mk_pat_list( 'fine', '\\b', fine_list, '\\b' ) ;
}

// List the imposition of fines
const abstract_fine_list	= [
	'<verdachte> <verplichting> opleggen <tot> betaling <aan_de_staat> <van>(<bedrag>)',
	'bepaalt <verplichting> <tot> <betaling> <aan_de_staat> <ter_ontneming> op(<bedrag>)',
	'beslissing<dash><geldboete> <van>(<bedrag>)',
	'beslissing<dash><voor> <feit><dash><geldboete> <van>(<bedrag>)',
	'beveelt dat de opbrengst van het verbeurd verklaarde clubhuis aan de \\[adres\\] na aftrek van een bedrag van euro (<bedrag>) aan de Stichting wordt uitbetaald',
	'legt <ter_ontneming> <verdachte> <verplichting> op <tot> <betaling> <aan_de_staat> <van>(<bedrag>)',
	'legt <verdachte> <geldboete> op <van>(<bedrag>)',
	'legt <verdachte> <maatregel> op<dash><voor> <feit> <geldboete> <van>(<bedrag>)',
	'legt <verdachte> <maatregel> op<dash>inhoudende <verplichting> <aan_de_staat> te betalen <van>(<bedrag>)',
	'legt <verdachte> <ter_ontneming> <verplichting> op <tot> <betaling> <aan_de_staat> <van>(<bedrag>)',
	'legt <verdachte> <verplichting> <tot> <betaling> <aan_de_staat> <van>(<bedrag>)',
	'legt <verdachte> <verplichting> op (dat bedrag) <aan_de_staat> te betalen <ter_ontneming>',
	'legt <verdachte> <verplichting> op <ter_ontneming> <aan_de_staat> te voldoen <van>(<bedrag>)',
	'legt <verdachte> <verplichting> op <tot> <betaling> <aan_de_staat> <ter_ontneming> <van>(<bedrag>)',
	'legt <verdachte> <verplichting> op <tot> <betaling> <aan_de_staat> <van>(<bedrag>)',
	'legt <verdachte> <verplichting> op <tot> <betaling> <van>(<bedrag>)<dash><aan_de_staat>',
	'legt <verdachte> <verplichting> op <tot> <geldboete> <van>(<bedrag>)',
	'legt <verdachte> op<dash><geldboete> <van>(<bedrag>)',
	'legt <verdachte> op<dash><verplichting> <tot> <betaling> <aan_de_staat> <van>(<bedrag>)',
	'legt <voor> <feit> op <maatregel><dash><geldboete> <van>(<bedrag>)',
	'legt op <geldboete> <van>(<bedrag>)',
	'legt op <maatregel><dash><geldboete> <van>(<bedrag>)',
	'legt op <maatregel><dash><voor> <feit><dash><geldboete> <van>(<bedrag>)',
	'legt op <maatregel><dash><voor> <feit><dash>vrijspraak<dash><voor> <feit><dash><geldboete> <van>(<bedrag>)',
	'legt op <verdachte><dash><verplichting> <tot> <betaling> <aan_de_staat> <van>(<bedrag>)',
	'legt op <verdachte><dash><verplichting> <tot> <betaling> <van>(<bedrag>) <aan_de_staat>',
	'stelt <verplichting> <aan_de_staat> <ter_ontneming> op (nihil)',
	'stelt <verplichting> <aan_de_staat> op (nihil)',
	'stelt <verplichting> <tot> <betaling> <aan_de_staat> <ter_ontneming> vast op (nihil)',
	'stelt <verplichting> <tot> <betaling> <aan_de_staat> van het geschatte voordeel op (nihil)',
	'stelt <verplichting> <tot> <betaling> <aan_de_staat> vast op (nihil)',
	'stelt <verplichting> van <verdachte> <tot> <betaling> <aan_de_staat> <ter_ontneming> op (nihil)',
	'stelt het bedrag waarop het door <verdachte> wederrechtelijk verkregen voordeel wordt geschat, vast op (nihil)',
	'strafoplegging <feit><dash><geldboete> <van>(<bedrag>)',
	'veroordeelt <instantie> <verdachte> <tot><dash><geldboete> <van>(<bedrag>)',
	'veroordeelt <verdachte> <ter_ontneming> <tot><dash><geldboete> <van>(<bedrag>)',
	'veroordeelt <verdachte> <tot> <betaling> <aan_de_staat> <ter_ontneming> <van>(<bedrag>)',
	'veroordeelt <verdachte> <tot> <betaling> <aan_de_staat> <van>(<bedrag>)',
	'veroordeelt <verdachte> <tot> <betaling> <van>(<bedrag>) <aan_de_staat>',
	'veroordeelt <verdachte> <tot><dash><voor> <feit> <geldboete> <van>(<bedrag>)',
	'veroordeelt <verdachte> <voor> <feit> <tot> <voor> <feit><dash><geldboete> <van>(<bedrag>)',
	'veroordeelt <verdachte> <voor> <feit> <tot><dash><geldboete> <van>(<bedrag>)',
	'veroordeelt <verdachte><dash><tot><dash><geldboete><dash><van>(<bedrag>)'

] ;
const fine_pat_list		= substitute_fine_list( abstract_fine_list ) ;

// die bestaat uit het verrichten van onbetaalde arbeid
// Substitute the tokens in an abstract labour list.
function substitute_labour_list( abstract_labour_list ) {
	const labour_list = abstract_labour_list.map( item => item
		.replace( /<dash>/g,		'(?:\\.|:|;|-|,|\\*| )*' )
		.replace( /<tot>/g,		'(?:derhalve| )*tot(?:het|verrichten|van| )*' )
		.replace( /<verdachte>/g,	'(?:aan|de|het| )*\\[?(?:voor|naam|van|verdachte|veroordeelde| )*\\]?' )
		.replace( /<taakstraf>/g,	'(?:\\.|:|-|,|\\*|\\(|\\)|TACT|Tools *4U|arbeid|bestaande|' +
						'bestaat|cool|dan|de\\b|deze|die|een|het|in|leerproject|' +
						'limits|maximale|navolgende|o?n?voorwaardelijke|onbetaalde|ook|plus|' +
						'regulier|respect|so|straf|te|uit|van|verlengd|verrichten|' +
						'van|volgen|voorwaardelijke?|vorm|weten|zijnde| )*' +
						'(?:leerstraf|taakstraf|werkstr?af)' +
						'(?:\\.|:|-|,|\\*|\\(|\\)|TACT|Tools *4U|arbeid|bestaande|' +
						'bestaat|conform|cool|dan|de\\b|deze|die|een|eis|het|in|leerstraf|leerproject|' +
						'limits|maximale|navolgende|officier van justitie|o?n?voorwaardelijke|' +
						'onbetaalde|ook|op|plus|regulier|respect|so|straf|taakstraf|te|uit|van|verlengd|' +
						'verrichten|van|volgen|voorwaardelijke?|vorm|werkstr?af|weten|zijnde| )*' )
		.replace( /<van>/g,		'(?:de duur|de tijd|de termijn|gedurende|van|voor| )*' )
		.replace( /<maatregel>/g,	'(?:\\\/|bijkomende|de|en|of|maatregel\\(?e?n?\\)?|(?:hoofd *)?straf\\(?f?e?n?\\)?|volgende| )+' )
		.replace( /<voor>/g,		'(?:wegens|ten|aanzien|van|_t_a_v_?|ter|zake|voor|in|verband|met| )+' )
		.replace( /<vrijspraak>/g,	'(?:Ontslag van alle rechtsvervolging, zijnde het feit niet strafbaar)' )
		.replace( /<feit>/g,		'(?:bewezene?|de|en|eerste|feite?n?|gelegde?|het|in|laste|meer|meest|met|misdrij[vf]e?n?|' +
						'onder|overige|parketnummer|primaire?|strafbaa?re?|subsidiaire?|ten|tweede|van|' +
						'verklaarde|zaak|[0-9][A-Z]?|[\\\/\\.\\[\\]: ,-])+' )
		.replace( /<uren>/g,		'(?:\\b[a-z]+(?: [a-z]+ )?(?:[\\(|\\[][0-9]+[\\)|\\]] *)?(?:uur|uren)|' +
						'\\b[0-9]+ (?:(?:uur|uren) )?(?:[\\(|\\[][a-z]+(?: [a-z]+)?[\\)|\\]] *)?(?:uur|uren))' )
		.replace( /<periode>/g,		'(?:\\(|\\[|\\)|\\]|\\b[0-9]+[a-z]*\\b|\\b[a-z]+\\b|\\.|,|:| )*' +
						'\\b(?:uu?r|dag|wee?k|maand|jaa?r)(?:en)?\\b' +
						'(?:\\(|\\[|\\)|\\]|\\b[0-9]+\\b|\\b[a-z]+\\b|\\.|,|:| )*' ) ) ;
	return mk_pat_list( 'labour', '\\b', labour_list, '\\b' ) ;
}

// List the imposition of labour
const abstract_labour_list	= [
	'BESLISSING<dash><voor><feit><dash><taakstraf> <van>(<periode>)',
	'beslissing <taakstraf> voor de duur van (<periode>)',
	'beslissing<dash><taakstraf> <van>(<periode>)',
	'beslissing<dash><voor> <feit><dash> vrijspraak <voor> <feit><dash><taakstraf> <van>(<periode>)',
	'gelast <taakstraf> <van>(<periode>)',
	'gelast <voor> <feit> <taakstraf> <van>(<periode>)',
	'legt <verdachte> <taakstraf> op<dash><taakstraf> <van>(<periode>)',
	'legt <verdachte> <voor> <feit> <taakstraf> op<dash><taakstraf> <van>(<periode>)',
	'legt <verdachte> <voor> <feit> en <voor> <feit> <taakstraf> <van>(<periode>)',
	'legt <verdachte> op <taakstraf> <van>(<periode>)',
	'legt <voor> <feit> op <maatregel><dash><taakstraf> <van>(<periode>)',
	'legt de rechtbank <verdachte> <taakstraf> op <van>(<periode>)',
	'legt de rechtbank <verdachte> op <taakstraf> <van>(<periode>)',
	'legt op <maatregel><dash><taakstraf><dash><van>(<periode>)',
	'legt op <maatregel><dash><voor> <feit><dash><taakstraf> <van>(<periode>)',
	'legt op <maatregel><dash><voor> <feit><dash><vrijspraak><dash><voor> <feit><dash><taakstraf> <van>(<periode>)',
	'legt op <maatregel><dash>\\(?<voor> <feit>\\)?<dash><taakstraf> <van>(<periode>)',
	'legt op <maatregel><dash>beslissing<dash><taakstraf><dash><van>(<periode>)',
	'legt op <maatregel><dash>beslissing<dash><voor> <feit><dash><taakstraf> <van>(<periode>)',
	'legt op <taakstraf> <van>(<periode>)',
	'strafoplegging <taakstraf> <van>(<periode>)',
	'verklaart <verdachte> hiervoor strafbaar<dash><taakstraf> <van>(<periode>)',
	'veroordeelt <verdachte> <tot> <maatregel><dash><taakstraf> <van>(<periode>)',
	'veroordeelt <verdachte> <tot> <taakstraf><dash><verdachte> moet (<uren>) onbetaalde arbeid verrichten',
	'veroordeelt <verdachte> <tot><dash>(<uren>)<taakstraf>', // <periode> does not work with <taakstraf> at the end
	'veroordeelt <verdachte> <tot><dash><feit> <taakstraf> <van>(<periode>)',
	'veroordeelt <verdachte> <tot><dash><taakstraf><dash><van>(<periode>)',
	'veroordeelt <verdachte> <tot><dash><taakstraf><dash><verdachte> moet (<periode>)',
	'veroordeelt <verdachte> <tot><dash><voor> <feit><dash><taakstraf> <van>(<periode>)',
	'veroordeelt <verdachte> <voor> <feit> <tot><dash><taakstraf><dash><van>(<periode>)',
	'veroordeelt <verdachte> <voor> <feit><verdachte> <tot> <taakstraf> <van>(<periode>)',
	'veroordeelt de rechtbank <verdachte> <voor> <feit> <tot><dash><taakstraf> <van>(<periode>)'

] ;
const labour_pat_list	= substitute_labour_list( abstract_labour_list ) ;

// Substitute the tokens in an abstract admin list.
function substitute_admin_list( abstract_admin_list ) {
	const admin_list = abstract_admin_list.map( item => item
		.replace( /<alsnog>/g,		'alsnog (?:ten|uitvoer|zal|worden|gelegd| )*' )
		.replace( /<beschikking>/g,	'(?:de|vorenstaande|inleidende|volgende|bestreden| )*(?:beschikkinge?n?|beslissinge?n?)' )
		.replace( /<tenuitvoerlegging>/g,	'(?:de|gedeeltelijke?|voorlopige?| )*tenuitvoerlegging' )
		.replace( /<gegrond>/g,		'(?:gedeeltelijk|deels| )*(?:on[ -]*gegrond|gegrond|nietig)' )
		.replace( /<instantie>/g,	'(?:de|het|zijn|hem|haar|behandelende?|betreffende?|enkelvoudige?|meervoudige?|militaire?| )*' +
						'(?:\\[A\\]|directeur|geneesheer|hof|hoge raad|instantie|kamer|kinder[ -]*rechter|' +
						'officiere?n? van justitie|openbaar[ -]*ministerie|politie[ -]*rechter|raadkamer|' +
						'raadshee?re?n?|raadsvrouwe?n?|rechtbank|rechter[ -]*commissaris|' +
						'rechters?|van|verdediging|verschoningskamer|voorzieningenrechter|wrakingskamer)[:, -]*' )
		.replace( /<land>/g,		'[a-z, ]+' )
		.replace( /<levering>/g,	'(?:de )?(?:over[ -]*levering|uit[ -]*levering|uitleveringsdetentie|overleveringsdetentie)' )
		.replace( /<maatregel>/g,	'(?:deel|de|een|deel van|gedeelte *van|het|maatregel (?:tot|van)|niet *ten *uitvoer *gelegde|' +
						'opgelegde|resterende|vervangende|voorlopige?|voorwaardelijke?| )*'+
						'(?:(?:TBS|ISD|PIJ)?[ -]*maatregel|TBS|\\(|\\)|' +
						'dwangmiddel|gedragsaanwijzing|gedurende beschikking is gesteld|geldboete|' +
						'gevangenhouding|gevangenisstraff?e?n?|gijzeling|hechtenis|hoofdstraf|' +
						'inverzekeringstelling|invrijheidss?telling|jeugddetentie|met|of|' +
						'plaatsing in een inrichting voor (?:stelselmatige daders?|jeugdigen)|' +
						'straf|ter *beschikking *stelling|van overheidswege|verpleging|' +
						'voorwaarden|vrijheidsstraff?e?n?|zoals hierboven omschreven|' +
						'zowel het klassieke als het conservatoire beslag| )*' )
		.replace( /<mr>/g,		'(?:[a-z,_ -]+)' )
		.replace( /<dash>/g,		'(?:[:;, \\\/\\.-]+)' )
		.replace( /<nummer>/g,		'(?:[0-9a-z\\\/\\.-]+)' )
		.replace( /<datum>/g,		'(?:_d_d_ )?(?:[0-9]+ [a-z]+ 20[0-9][0-9])' )
		.replace( /<tijd>/g,		'(?:[0-9\\.]+ uur)' )
		.replace( /<ontneming>/g,	'ontneming van (?:het )?wederrechtelijk verkregen voordeel' )
		.replace( /<onbevoegd>/g,	'(?:bevoegd|onbevoegd|niet[ -]*bevoegd)' )
		.replace( /<ontvankelijk>/g,	'(?:\\(|kennelijk|aanstonds|\\)| )*(?:ontvankelijk|on[ -]*ontvankelijk|niet[ -]*ontvankelijk)' )
		.replace( /<partij>/g,		'(?:\\[|\\(|\\b[a-z]\\b|benadeelden?|betrokkenen?|bezwaarden?|eisers?|de|deze|eiseress?e?n?|getuigen?|' +
						'klaagsters?|klagers?|voor|en|achter|naam|opg?eeii?st?e? pers?p?oon|' +
						'persoon[0-9 ]*|partije?n?|slachtoffer[ 0-9]*|ter[ -]*beschikking[ -]*gestelden?|' +
						'verdachten?|veroordeelden?|verzochte|verzoekers?|verzoeksters?|voornoemd|\\]|\\)|,| )*' )
		.replace( /<plaats>/g,		'(?:[a-z]+)' )
		.replace( /<voor>/g,		'(?:met|voor|wegens|ten aanzien van)' )
		.replace( /<van>/g,		'(?:van|voor de duur van)' )
		.replace( /<toe>/g,		'(?:gedeeltelijk|deels|ten dele| )*(?:af|toe)' )
		.replace( /<toelaatbaar>/g,	'(?:on[ -]*toelaatbaar|toelaatbaar)' )
		.replace( /<tot>/g,		'(?:strekkende| )*(?:om|tot)' )
		.replace( /<verzoek>/g,		'(?:de|het|zijn|hem|haar|hun|in beide zaken|schriftelijke|mondelinge| )*' +
						'(?:beklag|beroepe?n?|bezwa?are?n?|hoger beroep|klaag|klachte?n?|' +
						'ontnemings[ -]*vorderinge?n?|oproeping|verzoeke?n?|vorderinge?n?|wrakingsverzoek)' +
						'(?:schrifte?n?)?' )
		.replace( /<termijn>/g,		'(?:de|termijn| )*' )
		.replace( /<periode>/g,		'(?:de|tijd van|duur van|' +
						'(?:\\b[a-z]+\\b(?:\\(|\\b[0-9]+\\b|\\)| )*|\\b[0-9]+\\b(?:\\(|\\b[a-z]+\\b|\\)| )*' +
						'(?:\\b(?:uu?r|dag|wee?k|maand|jaa?r)(?:en)?\\b))| )*' )
		.replace( /<weigert>/g,		'(?:verklaart on[ -]*toelaatbaar|verklaart toelaatbaar|heft op|weigert|staat toe|' +
						'staat niet toe|wijst toe|wijst af)' ) ) ;
	return mk_pat_list( 'admin', '\\b', admin_list, '\\b' ) ;
}

// List phrases related to the administrative processes
const abstract_admin_list	= [
	'<instantie> <nummer> verklaart <partij> <ontvankelijk>',
	'<instantie> <nummer> weigert de gevraagde voorziening',
	'<instantie> <nummer> wijst <verzoek> <toe>',
	'<instantie> <weigert> <verzoek>',
	'<instantie> beeindigt <maatregel>',
	'<instantie> bekrachtigt het vonnis waarvan beroep',
	'<instantie> beveelt de hervatting van <maatregel>',
	'<instantie> is van oordeel dat het beroep op het verschoningsrecht door <instantie> dient te wijken voor het belang van de waarheidsvinding',
	'<instantie> machtigt <instantie> <maatregel> toe te passen jegens <partij>',
	'<instantie> schorst <maatregel> van <partij>',
	'<instantie> schorst de vervolging',
	'<instantie> verklaart <partij> <ontvankelijk>',
	'<instantie> verklaart zich <onbevoegd> om deze beslissing te nemen',
	'<instantie> verklaart zich <onbevoegd> tot het afdoen van het klaagschrift',
	'<instantie> verleent het verzochte verlof',
	'<instantie> verwerpt <verzoek>',
	'<instantie> verwerpt het beroep',
	'<instantie> wijst <verzoek> <toe>',
	'<instantie> wijst <verzoek> <tot> <maatregel> <toe>',
	'<instantie> wijst <verzoek> <tot> het horen van <partij> daarom <toe>',
	'<instantie> wijst <verzoek> <tot> het uitstellen van <maatregel> <toe>',
	'<instantie> zal <verzoek> toewijzen',
	'<partij> zijn <ontvankelijk> in hun bezwaar',
	'<verzoek> <tot> opheffen van <maatregel> wordt daarom afgewezen',
	'<verzoek> is <gegrond>',
	'<verzoek> van <instantie> wordt afgewezen',
	'<verzoek> van <partij> wordt afgewezen',
	'<weigert> <levering> van <levering> van <partij>',
	'<weigert> <levering> van <partij>',
	'<weigert> <verzoek> <maatregel> ex artikel <nummer> WED',
	'<weigert> <verzoek> <tot> <ontneming>',
	'<weigert> <verzoek> <tot> <tenuitvoerlegging>',
	'<weigert> <verzoek> <tot> aanhouding van de behandeling van <verzoek>',
	'<weigert> <verzoek> <tot> achterwege laten van <maatregel>',
	'<weigert> <verzoek> <tot> gedeeltelijke herroeping van <maatregel>',
	'<weigert> <verzoek> <tot> gedeeltelijke herroeping van de voorwaardelijke invrijheidss?telling',
	'<weigert> <verzoek> <tot> herroeping van <maatregel>',
	'<weigert> <verzoek> <tot> hervatting van <maatregel>',
	'<weigert> <verzoek> <tot> het achterwege blijven van <maatregel>',
	'<weigert> <verzoek> <tot> het achterwege laten van <maatregel>',
	'<weigert> <verzoek> <tot> het gelasten van <tenuitvoerlegging> van <maatregel>',
	'<weigert> <verzoek> <tot> het gelasten van een onderzoek',
	'<weigert> <verzoek> <tot> het horen van de getuigen',
	'<weigert> <verzoek> <tot> het vaststellen van voorwaarden',
	'<weigert> <verzoek> <tot> opheffing dan wel schorsing van <levering>',
	'<weigert> <verzoek> <tot> opheffing en schorsing van <levering>',
	'<weigert> <verzoek> <tot> opheffing van <levering>',
	'<weigert> <verzoek> <tot> schadevergoeding',
	'<weigert> <verzoek> <tot> schorsing van <levering>',
	'<weigert> <verzoek> <tot> schorsing van <maatregel>',
	'<weigert> <verzoek> <tot> uitstel van <maatregel>',
	'<weigert> <verzoek> <tot> vaststelling en <ontneming>',
	'<weigert> <verzoek> <tot> verlenging van <maatregel>',
	'<weigert> <verzoek> <tot> verlenging van de termijn gedurende welke <partij> ter beschikking is gesteld',
	'<weigert> <verzoek> <tot> verlenging van het bevel <tot> bewaring van <partij>',
	'<weigert> <verzoek> <tot> wraking',
	'<weigert> <verzoek> jegens <partij> <tot> <maatregel>',
	'<weigert> <verzoek> jegens <partij> <tot> betaling van een bedrag aan de Staat ter <ontneming>',
	'<weigert> <verzoek> van <instantie>',
	'<weigert> <verzoek> van <partij>',
	'<weigert> <verzoek> voor zover dat strekt <tot> toekenning van een vergoeding voor materiele schade',
	'<weigert> alle door <instantie> gedane <verzoek>',
	'<weigert> alle door <partij> gedane <verzoek>',
	'<weigert> de door <land> verzochte <levering> van <partij>',
	'<weigert> de ter terechtzitting van <datum> gedane verzoeken met betrekking <tot> onderzoekshandelingen',
	'<weigert> het beroep op verschoningsrecht',
	'<weigert> het beroep tegen <beschikking>',
	'aanvulling op het verkort vonnis van <datum>',
	'beeindigt <maatregel> van <partij>',
	'beeindigt <maatregel>l <tot> <maatregel> van <partij>',
	'bepaalt<dash>dat <tenuitvoerlegging> van <maatregel> die aan <partij> bij voornoemd vonnis is opgelegd<dash>wordt geschorst',
	'bepaalt<dash>dat <tenuitvoerlegging> van <maatregel> wordt voort?gezet',
	'bepaalt<dash>dat voortzetting van <maatregel> is vereist',
	'bepaalt<dash>dat zij <onbevoegd> is kennis te nemen van <verzoek>',
	'beschikt als volgt<dash><weigert> de vordering gevangenneming',
	'beslissing na voorwaardelijke veroordeling<dash>afwijzing van de vordering',
	'beslissing na voorwaardelijke veroordeling<dash>last tot tenuitvoerlegging van de straf',
	'beslissing wijst <verzoek> <toe>',
	'beslist<dash>dat <maatregel> zal worden voortgezet',
	'beslist<dash>dat de tenuitvoerlegging van <maatregel> wordt voortgezet',
	'beslist<dash>dat de voortzetting van de tenuitvoerlegging van <maatregel> is vereist',
	'beslist<dash>dat het onderzoek wordt geschorst voor onbepaalde tijd',
	'beveelt de gevangenhouding van <partij>',
	'beveelt<dash><tenuitvoerlegging> van de hierboven genoemde <maatregel>',
	'beveelt<dash>dat <partij> <partij> alsnog van overheidswege zal worden verpleegd',
	'beveelt<dash>dat <partij> alsnog van overheidswege zal worden verpleegd',
	'beveelt<dash>dat het onderzoek ter terechtzitting zal worden hervat op een nader te bepalen dag en uur',
	'beveelt<dash>de gevangenneming ter <levering>',
	'beveelt<dash>de gevangenneming van <partij>',
	'beveelt<dash>de schorsing van <levering> van <partij>',
	'beveelt<dash>de schorsing van <maatregel>',
	'beveelt<dash>het ontslag uit de gijzeling van <partij>',
	'Deze bijlage behoort bij het op <datum> gewezen vonnis in de zaak tegen <partij>',
	'deze publicatie is ingetrokken door',
	'gelast <tenuitvoerlegging> van <beschikking>',
	'gelast <tenuitvoerlegging> van <maatregel>',
	'gelast dat <maatregel> <alsnog>',
	'gelast dat <maatregel> van <partij> zal worden hervat',
	'gelast dat <maatregel><dash> <periode><dash><alsnog>',
	'gelast dat <maatregel><dash> te weten <maatregel> <periode><dash><alsnog>',
	'gelast dat <maatregel><dash><maatregel> <van> <periode><dash><alsnog>',
	'gelast dat <maatregel><dash>te weten <maatregel> <van> <periode><dash><alsnog>',
	'gelast de gedeeltelijke herroeping van <maatregel>',
	'gelast de hervatting van <maatregel>',
	'gelast de verlenging van de proeftijd',
	'gelast met onmiddellijke ingang het ontslag van <partij> uit de gijzeling',
	'handhaaft <beschikking> <tot> uitstel van <beschikking> over <levering>',
	'handhaaft de gestelde prejudiciele vragen',
	'handhaaft haar beslissing van <datum><dash>met herstel van het dictum',
	'heft op <levering>',
	'heft op de voorwaardelijke beeindiging van <maatregel>',
	'Heropent de behandeling van de zaak om voormelde reden en schorst het onderzoek',
	'heropent de behandeling van de zaak om voormelde redenen en schorst het onderzoek voor onbepaalde tijd',
	'heropent de behandeling<dash>bepaalt dat <instantie> de zaak als <instantie> voortzet<dash>schorst de behandeling',
	'heropent en schorst de behandeling',
	'heropent en schorst het onderzoek',
	'heropent het gesloten onderzoek en schorst dit voor onbepaalde tijd',
	'Heropent het gesloten onderzoek ter terechtzitting<dash>schorst het onderzoek ter terechtzitting',
	'heropent het gesloten onderzoek<dash>schorst dit',
	'heropent het onderzoek en bepaalt dat de behandeling in <instantie> zal worden aangehouden',
	'heropent het onderzoek en bepaalt dat het onderzoek in <instantie> zal worden voortgezet',
	'heropent het onderzoek en schorst dit',
	'heropent het onderzoek en schorst het onderzoek voor onbepaalde tijd',
	'heropent het onderzoek in deze zaak; schorst het onderzoek ter terechtzitting',
	'heropent het onderzoek onder gelijktijdige schorsing daarvan',
	'heropent het onderzoek ter terechtzitting en schorst dit terstond',
	'heropent het onderzoek ter terechtzitting; schorst het onderzoek ter terechtzitting',
	'heropent het onderzoek ter terechtzitting<dash>schorst het onderzoek ter terechtzitting',
	'heropent het onderzoek ter terechtzitting<dash>schorst het onderzoek voor onbepaalde tijd',
	'heropent het onderzoek ter zitting<dash>schorst het onderzoek ter zitting voor onbepaalde tijd',
	'heropent het onderzoek ter zitting<dash>schorst het onderzoek ter zitting',
	'heropent het onderzoek ter zitting<dash>schorst het onderzoek voor onbepaalde tijd',
	'heropent het onderzoek ter zitting<dash>stelt uit <beschikking> over <levering>',
	'heropent het onderzoek<dash>schorst het onderzoek voor onbepaalde tijd',
	'heropent het op <datum> gesloten onderzoek en schorst dit',
	'heropent het ter terechtzitting van <datum> gesloten onderzoek ter terechtzitting en schorst het onderzoek voor onbepaalde tijd',
	'herroept <maatregel> van <partij>',
	'herstel(?:beschikking|vonnis) gewezen naar aanleiding van <datum>',
	'herstelt de kennelijke misslag in het dictum',
	'herstelt het onder parketnummer <nummer> op <datum> gewezen vonnis',
	'Herstelvonnis gewezen naar aanleiding van het op <datum> door <instantie>',
	'het onderzoek wordt heropend en meteen voor bepaalde tijd<dash>te weten <tot> <datum> te <tijd>, geschorst',
	'kent <toe> aan <partij> een vergoeding uit s Rijks kas',
	'kent <toe> aan <partij> uit s Rijks kas een vergoeding',
	'kent aan <partij> <toe> ten laste van de Staat een bedrag',
	'kent aan <partij> een vergoeding <toe>',
	'kent aan <partij> een vergoeding uit s Rijks kas <toe>',
	'kent aan <partij> een vergoeding van schade en kosten als bedoeld in de artikel <nummer> van het Wetboek van Strafvordering ten laste van de Staat <toe>',
	'kent aan <partij> ten laste van de Staat een bedrag toe',
	'kent aan <partij> ten laste van de staat een vergoeding <toe>',
	'kent aan <partij> ten laste van de Staat een vergoeding ex artikel <nummer> van het Wetboek van Strafvordering <toe>',
	'kent aan <partij> toe een vergoeding ten laste van de Staat',
	'kent aan <partij> toe ten laste van de Staat een bedrag',
	'kent aan <partij> uit s Rijks kas een vergoeding <toe>',
	'kent op grond van artikel <nummer> Sv aan <partij> ten laste van de Staat een vergoeding <toe>',
	'kent toe aan <partij> voornoemd een vergoeding uit s Rijks kas',
	'komt <tot> <beschikking><dash><instantie> wijst <verzoek> <toe>',
	'komt <tot> <beschikking><dash>Wijst? <verzoek> <toe>',
	'neemt het <verzoek> <tot> wraking van <mr> niet in behandeling',
	'Om dezelfde redenen worden ook in de zaken tegen <partij> de verzoeken afgewezen',
	'oordeelt <maatregel> van <partij> onrechtmatig ',
	'op de voet van artikel <nummer> Wetboek van Strafvordering<dash>wijst <verzoek> <toe>',
	'opheffing van het tegen <partij> verleende bevel <tot> <maatregel>',
	'schorst de vervolging van <partij>',
	'schorst het onderzoek <tot> een in overleg met <instantie>',
	'schorst het onderzoek <tot> een in overleg met <partij>',
	'schorst het onderzoek ter terechtzitting voor onbepaalde tijd',
	'stelt dit <verzoek> <tot> wraking buiten behandeling',
	'stelt het <verzoek> <tot> wraking van <mr> wegens kennelijke niet-ontvankelijkheid buiten behandeling',
	'stelt het bedrag van de aan <partij> uit te keren vergoeding uit s Rijks kas vast',
	'ten aanzien van beide <verzoek>: verklaart <partij> <ontvankelijk>',
	'ten aanzien van beide <verzoek><dash>verklaart <partij> niet ontvankelijk',
	'verklaart <gegrond> het door <partij> aangetekende beroep',
	'verklaart <instantie> <ontvankelijk> in de vervolging van <partij>',
	'verklaart <instantie> <ontvankelijk>',
	'verklaart <instantie> <verzoek> <gegrond>',
	'verklaart <instantie> ten aanzien van de feiten 1 en 3 <ontvankelijk>',
	'verklaart <partij> <ontvankelijk> in <verzoek> <tot> wraking',
	'verklaart <partij> <ontvankelijk> in <verzoek>',
	'verklaart <partij> <partij> in <verzoek> <ontvankelijk>',
	'verklaart <toelaatbaar> <levering> aan <land> van <partij>',
	'verklaart <verzoek> <gegrond>',
	'verklaart <verzoek> <ontvankelijk>',
	'verklaart <verzoek> <tot> verschoning <gegrond>',
	'verklaart <verzoek> <tot> wraking <gegrond>',
	'verklaart <verzoek> <tot> wraking <ontvankelijk>',
	'verklaart <verzoek> <tot> wraking tegen <mr> <gegrond>',
	'verklaart <verzoek> <tot> wraking van <instantie> <ontvankelijk>',
	'verklaart <verzoek> <tot> wraking van <mr> <ontvankelijk>',
	'verklaart <verzoek> <tot> wraking van <mr> in de zaak met zaaknummer <ontvankelijk>',
	'verklaart <verzoek> in <partij> <gegrond>',
	'verklaart <verzoek> met betrekking <tot> <verzoek> <tot> het horen van <partij> en <partij> <gegrond>',
	'verklaart <verzoek> tegen <beschikking> van <instantie> <gegrond>',
	'verklaart <verzoek> tegen <maatregel> <gegrond>',
	'verklaart <verzoek> tegen <maatregel> van <datum> <gegrond>',
	'verklaart <verzoek> tegen de inleidende <beschikking> <gegrond>',
	'verklaart <verzoek> ter zake art. <nummer> Sv <gegrond>',
	'verklaart <verzoek> van <instantie> <gegrond>',
	'verklaart <verzoek> van <partij> <gegrond>',
	'verklaart <verzoek> van <partij> tegen <beschikking> van <instantie> <datum> <gegrond>',
	'verklaart <verzoek> van de <partij> tegen voormelde kennisgeving <gegrond>',
	'verklaart dat de strafzaak tegen <partij> onder parketnummer <nummer> niet is geeindigd',
	'verklaart dat de zaak tegen <partij> met bovengemeld parketnummer is geeindigd',
	'verklaart de dagvaarding nietig',
	'verklaart de door <land> verzochte <levering> van <partij> <toelaatbaar>',
	'verklaart de klaagschriften in <instantie> <gegrond>',
	'Verklaart het verzet gegrond<dash>vernietigt het dwangbevel',
	'verklaart het verzet gegrond<dash>vernietigt het tegen <partij> uitgevaardigde dwangbevel',
	'verklaart voormeld <verzoek> <gegrond>',
	'verklaart zich <onbevoegd> <tot> de afdoening van <verzoek>',
	'verklaart zich <onbevoegd> <tot> kennisneming van <verzoek> van <instantie>',
	'verklaart zich <onbevoegd> <tot> kennisneming van <verzoek>',
	'verklaart zich <onbevoegd> <tot> kennisneming van het ten laste gelegde',
	'verklaart zich <onbevoegd> kennis te nemen van <verzoek>',
	'verklaart zich <onbevoegd> om op <verzoek> te beslissen',
	'verklaart zich <onbevoegd> om van de onderhavige strafzaak kennis te nemen',
	'verleent derhalve machtiging aan <instantie> in <plaats> om kennis te nemen',
	'verlengt <maatregel> <van> <partij> <voor> <periode>',
	'verlengt <maatregel> <voor> <periode>',
	'verlengt <termijn> <maatregel> <voor> <periode>',
	'verlengt <termijn> van <maatregel> van <partij> <voor> <periode>',
	'verlengt <termijn> van <maatregel><dash> <voor> <periode>',
	'verlengt de bij voormeld vonnis bepaalde proeftijd met',
	'verlengt de bij voormeld vonnis vastgestelde proeftijd met twee jaar',
	'verlengt de geldigheidsduur van het bevel <tot> gevangenhouding van <partij>',
	'verlengt de proeftijd met een jaar',
	'verlengt de proeftijd van <maatregel> met <nummer> dagen',
	'verlengt de proeftijd<dash>bepaald bij vonnis van <instantie>',
	'verlengt de termijn gedurende welke <partij> *ter beschikking is gesteld',
	'verlengt de termijn gedurende welke <partij> voornoemd ter beschikking is gesteld',
	'verlengt de termijn van <maatregel>',
	'verlengt de termijn welke <partij> ter beschikking is gesteld',
	'verstaat dat de zaak van rechtswege is geeindigd',
	'verstaat dat geen gevangenisstraf meer kan worden opgelegd',
	'verwijst de zaak naar de rol van <datum> voor het nemen van een akte',
	'wijst <verzoek> <tenuitvoerlegging> na voorwaardelijke veroordeling <toe>',
	'wijst <verzoek> <toe> en bepaalt dat <maatregel> geheel achterwege wordt gelaten',
	'wijst <verzoek> <toe> en bepaalt dat <maatregel> van <partij> achterwege blijft',
	'wijst <verzoek> <toe> en beveelt dat <partij> wordt teruggeplaatst in een inrichting',
	'wijst <verzoek> <tot> <maatregel> <toe>',
	'wijst <verzoek> <tot> <ontneming> <toe>',
	'wijst <verzoek> <tot> <tenuitvoerlegging> <toe>',
	'wijst <verzoek> <tot> <tenuitvoerlegging> van <maatregel> <toe>',
	'wijst <verzoek> <tot> afstel van <maatregel> <toe>',
	'wijst <verzoek> <tot> beeindiging van <maatregel> <toe>',
	'wijst <verzoek> <tot> een gedeeltelijke herroeping van <maatregel> <toe>',
	'wijst <verzoek> <tot> gedeeltelijke herroeping van <maatregel> <toe>',
	'wijst <verzoek> <tot> herroeping van <maatregel> <toe>',
	'wijst <verzoek> <tot> herroeping van <maatregel> voor een periode van <nummer> dagen <toe>',
	'wijst <verzoek> <tot> het achterwege blijven van <maatregel> <toe>',
	'wijst <verzoek> <tot> het achterwege laten van <maatregel> <toe>',
	'wijst <verzoek> <tot> invrijheidss?telling van <partij> <toe>',
	'wijst <verzoek> <tot> matiging <toe>',
	'wijst <verzoek> <tot> opheffing van <maatregel> <toe>',
	'wijst <verzoek> <tot> opheffing van <maatregel> van <partij> <toe>',
	'wijst <verzoek> <tot> opheffing van de bijzondere voorwaarden <toe>',
	'wijst <verzoek> <tot> schorsing van het bevel tot <maatregel> <toe>',
	'wijst <verzoek> <tot> toekenning van vergoeding vanwege ondergane voorlopige hechtenis en de kosten van rechtsbijstand <toe>',
	'wijst <verzoek> <tot> uitstel met een termijn van <nummer> dagen <toe>',
	'wijst <verzoek> <tot> uitstel of achterwege blijven van <maatregel> <toe>',
	'wijst <verzoek> <tot> uitstel van <maatregel> <toe>',
	'wijst <verzoek> <tot> uitstel van <maatregel> voor de duur van <nummer> dagen <toe>',
	'wijst <verzoek> <tot> verlenging van <maatregel> af',
	'wijst <verzoek> <tot> verlenging van de proeftijd van <maatregel> <toe>',
	'wijst <verzoek> <tot> verlening van verlof <tot> <tenuitvoerlegging> van lijfsdwang voor de duur van <nummer> dagen <toe>',
	'wijst <verzoek> <tot> verschoning <toe>',
	'wijst <verzoek> <tot> wraking <toe>',
	'wijst <verzoek> <tot> wraking in beide strafzaken als kennelijk ongegrond <toe>',
	'wijst <verzoek> <tot> wraking van <instantie> <toe>',
	'wijst <verzoek> <tot> wraking van <mr> <toe>',
	'wijst <verzoek> <tot> wraking van <mr> wegens kennelijke niet-ontvankelijkheid <toe>',
	'wijst <verzoek> door naar de meervoudige kamer van de rechtbank',
	'wijst <verzoek> en de aanvullende <verzoek> tot wraking <toe>',
	'wijst <verzoek> ex artikel <nummer> lid <nummer> Sv <toe>',
	'wijst <verzoek> ingevolge artikel <nummer> van het Wetboek van Strafrecht <toe>',
	'wijst <verzoek> om wraking van <instantie> <toe>',
	'wijst <verzoek> te verklaren dat de strafzaak jegens <partij> is geeindigd <toe>',
	'wijst <verzoek> ten aanzien van de zaak met parketnummer <nummer> <toe>',
	'wijst <verzoek> van <instantie> <toe>',
	'wijst <verzoek> van <instantie> <tot> verlenging van <maatregel> <toe>',
	'wijst <verzoek> van <instantie> <tot> verlenging van <maatregel> van <partij> <toe>',
	'wijst <verzoek> van <instantie> <tot> verlenging van <termijn> van <maatregel> <voor> <periode> <toe>',
	'wijst <verzoek> van <instantie><dash><tot> <ontneming> <toe>',
	'wijst <verzoek> van <partij> <toe>'

] ;
const admin_pat_list		= substitute_admin_list( abstract_admin_list ) ;

// Substitute the tokens in an abstract acquittal list.
function substitute_acquittal_list( abstract_acquittal_list ) {
	const acquittal_list = abstract_acquittal_list.map( item => item
		.replace( /<dash>/g,		'(?:[:;, \\\/\\.-]+)' )
		.replace( / *<(?:voor|van)> */g,'(?:_t_a_v_|ten|aanzien|te dier|ter|zake|voor|daar|van|met|betrekking|tot| )+' )
		.replace( / *<(?:dat|wat)> */g,	'(?:dat|wat|hetgeen| )+' )
		.replace( / *<datum> */g,	'(?:_d_d_ )?(?:[0-9]+ [a-z]+ 20[0-9][0-9])' )
		.replace( / *<instantie> */g,	'(?:de rechtbank|de meervoudige kamer|de politie[ -]*rechter)[: -]*' )
		.replace( / *<vrij> */g,	'(?:hiervan|daarvan|onvoorwaardelijk|geheel|integraal| )*vrij *' )
		.replace( / *<wordt> */g,	'(?:wordt|zal|worden| )+' )
		.replace( / *<begaan> */g,	'(?:begaan|gepleegd| )+' )
		.replace( / *<nummer> */g,	'(?:[0-9a-z]|\\\/|-|,| )+' )
		.replace( / *<maatregel> */g,	'(?:de|volgende|straf(?:\\(?fen\\)?)?|maatregel(?:\\(?en\\)?)?|en|of|op| )+' )
		.replace( / *<verdachte> */g,	'(?:van|aan|hem|haar|de|\\b[a-z]\\b|instelling|voor|en|achter|naam|nn01|rechtspersoon|' +
						'stichting|verdachte|\\[|\\]|,| )+' )
		.replace( / *<(?:feit|tenlasteisgelegd)> */g,
						'(?:[0-9]\\.?|\\(|\\)|\\\/|\\b[A-Z]\\b|aangenomen|als|alternatief|bewezen|bij|cumulatief|' +
						'dagvaarding|dan|dat|derde|de|deze|dit|een|eerste|en|feite?n?|geacht|gedraging|g?e?legde?|' +
						'gewijzigde|hem|het|hierboven|hiervoor|\\bI+\\b|impliciet|is|keuze|laste|maak|meer|meest|' +
						'met|nog|of|omschreven|onder\\.?|overtuigende?|parketnummer (?:[0-9]|\\\/|-|,| )*|' +
						'primaire?|punt|sub|subsidiaire?|ten|tot|tweede|uiterst|verklaarde?|vermeld|' +
						'vierde|wel|wettig|wordt|zoals|,|-| )+')
		.replace( / *<bewezen> */g,	'(?:hierboven|wettig|en|overtuigend|bewezen|is|verklaard|wordt|geacht|,| )+' ) ) ;
	return mk_pat_list( 'acquittal', '\\b', acquittal_list, '\\b' ) ;
}

// List phrases that indicate acquittal
const abstract_acquittal_list	= [
	'<feit> niet <bewezen><dash>en spreekt <verdachte> <vrij>',
	'<instantie> spreekt <verdachte> <vrij>',
	'bepaalt<dash>dat <voor> <feit> geen <maatregel> <wordt> opgelegd',
	'bepaalt<dash>dat aan <verdachte> geen <maatregel> <wordt> opgelegd',
	'bepaalt<dash>dat geen <maatregel> <wordt> opgelegd',
	'beslissing<dash>bepaalt dat geen <maatregel> wordt opgelegd',
	'beslissing<dash>schuldigverklaring zonder oplegging van <maatregel>',
	'beslist als volgt<dash> <voor> <feit> vrijspraak',
	'de conclusie kan er maar een zijn, namelijk dat <verdachte> <wordt> <vrij>gesproken',
	'de rechtbank<dash><voor><feit>vrijspraak',
	'legt <verdachte> met toepassing van artikel <nummer> Wetboek van Strafrecht geen <maatregel> op',
	'legt op <maatregel><dash> <voor><feit><dash>vrijspraak',
	'niet <bewezen> <dat> <verdachte> <feit> heeft <begaan>?<dash>en spreekt <verdachte> <vrij>',
	'niet <bewezen> <wat> <feit> aan <verdachte> <tenlasteisgelegd><dash>en spreekt <verdachte> <vrij>',
	'niet <bewezen> <wat> <tenlasteisgelegd><dash><verdachte> <wordt> daarvan <vrij>gesproken',
	'niet <bewezen> <wat> <verdachte> <tenlasteisgelegd><dash>en spreekt? <verdachte> <vrij>',
	'niet <bewezen> <wat> <verdachte> <tenlasteisgelegd><dash>spreekt <verdachte> <vrij>',
	'niet <bewezen> <wat> <verdachte> onder <feit> <tenlasteisgelegd><dash>en spreekt <verdachte> <vrij>',
	'niet <bewezen> wordt geacht <feit><dash><verdachte> wordt daarvan vrijgesproken',
	'niet strafbaar <voor> <feit><dash>en ontslaat <verdachte> <van> alle rechtsvervolging',
	'niet strafbaar<dash>en ontslaat <verdachte> <van> alle rechtsvervolging',
	'noch het primair<dash>noch het subsidiair <feit> <bewezen><dash>en spreekt <verdachte> <vrij>',
	'ontslaat <verdachte> <van> alle rechtsvervolging',
	'ontslaat <verdachte> <voor> <feit> van alle rechtsvervolging',
	'spreekt <verdachte> <vrij> van <feit>',
	'spreekt <verdachte> van <feit> <vrij>',
	'ten aanzien van <feit><dash>vrijspraak',
	'verklaart <feit> niet <bewezen><dash>en spreekt <verdachte> <vrij>',
	'verklaart <feit> niet strafbaar en ontslaat verdachte <voor> <feit> van alle rechtsvervolging',
	'verklaart <feit> niet strafbaar<dash>ontslaat <verdachte> <voor> <feit> van alle rechtsvervolging',
	'verklaart <verdachte> (?:hiervoor )?schuldig zonder oplegging van <maatregel>',
	'verklaart <verdachte> daarvoor strafbaar<dash>legt geen <maatregel>',
	'verklaart <verdachte> niet strafbaar <voor> <feit> en ontslaat <verdachte> op dat onderdeel van alle rechtsvervolging',
	'verklaart <verdachte> niet strafbaar en ontslaat <verdachte> voor <feit> van alle rechtsvervolging',
	'verklaart dat <feit> geen strafbaar feit oplevert en ontslaat <verdachte> op dat onderdeel van alle rechtsvervolging',
	'vrijspraak, achtende de rechtbank het tenlastegelegde niet wettig en overtuigend bewezen',
	'vrijspraak<dash>verklaart niet <bewezen>, <dat> <verdachte> <feit> heeft <begaan>'

] ;
const acquittal_pat_list	= substitute_acquittal_list( abstract_acquittal_list ) ;

// Substitute the tokens in an abstract partial aquittal list.
function substitute_partial_acquittal_list( abstract_partial_acquittal_list ) {
	const partial_acquittal_list = abstract_partial_acquittal_list.map( item => item
		.replace( /<dash>/g,		'(?:[:;, \\\/\\.-]+)' )
		.replace( /<verdachte>/g,	'(?:aan|dat|hetgeen|wat| )*(?:haar|hem|verdachte)' )
		.replace( /<feit>/g,		'(?:,|\\b(?:[0-9]{2,}[\\/\\.-])?[0-9]{4,}(?:[\\/\\.-][0-9]{2})?\\b|\\b[0-9][0-9\\.]*\\b|' +
							'aangenomen|aanzien|als|anders|begaan|' +
							'bewezen|dan|de|en|feite?n?|gelegde?|' +
							'heeft|het|hierboven|hiervoor|is|laste|meer|' +
							'of|onder|mishandeling|parketnummers?|primaire?|' +
							'subsidiaire?|ten|van|verklaarde?| )*' ) ) ;
	return mk_pat_list( 'partial_acquittal', '\\b', partial_acquittal_list, '\\b' ) ;
}

const abstract_partial_acquittal_list = [
	'verklaart <feit> niet bewezen en spreekt <verdachte> daarvan vrij',
	'verklaart niet bewezen <verdachte> en spreekt <verdachte> daarvan vrij',
	'verklaart niet bewezen <verdachte> <feit> en spreekt <verdachte> daarvan vrij'
] ;
const partial_acquittal_pat_list	= substitute_partial_acquittal_list( abstract_partial_acquittal_list ) ;

// Article 36e is treated as a keyword because the courts often do not provide the "wettelijke voorschriften"
const abstract_article_36e_list = [
	'<beslissing> \\(ex <artikel_36e>\\)',
	'<beslissing> <berust_op> <artikel_36e>',
	'<maatregel> <berust_op> <artikel_36e>',
	'<rechtbank> heeft gelet op <artikel_36e>',
	'<rechtbank> is van oordeel dat <artikel_36e><dash>dat hoofdelijke veroordeling mogelijk maakt, in kan grijpen in de vaststelling van de omvang van het te ontnemen wederrechtelijk verkregen voordeel',
	'<rechtbank> ontleent aan <feit> het oordeel<dash>dat <verdachte> door <handelen> een voordeel als bedoeld in <artikel_36e> heeft (?:verkregen|gehad)',
	'<rechtbank> is van oordeel dat <verdachte> door <handelen> voordeel als bedoeld in <artikel_36e> heeft gehad'
] ;


const article_36e_list = abstract_article_36e_list.map( item => [ 'article_36e', new RegExp( item
	.replace( /<dash>/g,		'(?:,| )*' )
	.replace( /<maatregel>/g,	'(?:De op te leggen maatregel|De oplegging van de maatregel)' )
	.replace( /<artikel_36e>/g,	'(?:(?:het per [0-9]+ [a-z]+ 20[0-9]+ in werking getreden)?(?:artikel|art\\.?) 36e ' +
						'(?:lid 2|tweede lid|(?:en 36e )?\\(oud\\)|,| )*(?:van het Wetboek van Strafrecht|Sr|\\(Sr\\)| )*)' )
	.replace( /<vonnis>/g,		'(?:(?:de|inhoud van|het|voornoemde| )*vonnis(?:en|het|strafdossier| )*)' )
	.replace( /<beslissing>/g,	'(?:(?:Deze|de| )*beslissing)' )
	.replace( /<rechtbank>/g,	'(?:(?:De| )*rechtbank)' )
	.replace( /<berust_op>/g,	'(?:is(?:derhalve| )*gegrond op|berust op)' )
	.replace( /<feit>/g,		'(?:(?:de|die|genoemde?|het|hierboven|in de strafzaak|inhoud van|opgesomde?|' +
						'vermelde?|vernoemde?|voormelde?|voornoemde?| )*' +
						'(?:bewijsmiddelen|vonnis)(?:en|het|strafdossier| )*)' )
	.replace( /<verdachte>/g,	'(?:de| )*\\[?(?:verdachte|veroordeelde)\\]?' )
	.replace( /<handelen>/g,	'(?:(?:begaan|bewezenverklaarde?|deze|de|die|dit|' +
						'handelen|het|middel|plegen|van|vermelde?|' +
						'vernoemde?|voormelde?|voornoemde?| )*' +
						'(?:diefstall?e?n?|feite?n?)(?:[0-9]+| )*)' ), 'i' ) ] ) ;

// Convert a string into an age
function parse_age( txt ) {
	var value ;
	if( typeof txt !== 'undefined' ) {
		const normalised_txt = txt
			.replace( /  */g, ' ' )
			.toLowerCase() ;
		const integral_list = normalised_txt.match( /^ *([0-9,\. ]+)/i ) ;
		const word_list = normalised_txt.match( /^([a-z]*)/i ) ;
		if( integral_list ) {
			value = integral = Number.parseInt( integral_list[ 1 ].replace( /[,\. ]/g, '' ) ) ;
		} else if( word_list ) {
			try{
				value = support.word2number( word_list[ 1 ] ) ;
			} catch( e ) {
				value = undefined ;
			}
		}
	}
	if( debug ) {
		console.log( 'parse_age( %s ) -> %f', txt, value ) ;
	}
	if( typeof value === 'undefined' || isNaN( value ) ) {
		if( debug ) {
			console.log( 'parse_age( %s ) -> %s', txt, value ) ;
		}
		return undefined ;
	} else {
		return value ;
	}
}

// Convert a string into an amount in euros.
function parse_euro_cent( txt ) {
	var value ;
	if( typeof txt !== 'undefined' ) {
		const normalised_txt = txt
			.replace( /(?:\(afgerond\)|hoofdsom|zeggen?|voluit|:|\beuro\b|\bEUR\b\.?)/g, ' ' )
			.replace( /  */g, ' ' )
			.toLowerCase() ;
		const nihil_list = normalised_txt.match( /^ *nihil\b/ ) ;
		const integral_fraction_list = normalised_txt.match( /^ *([0-9,\. ]+?)([\.,](?:-+|=+|\d\d\b))/i ) ;
		const integral_list = normalised_txt.match( /^ *([0-9,\. ]+)/i ) ;
		const word_list = normalised_txt.match( /^([a-z]*)/i ) ;
		if( nihil_list ) {
			value = 0 ;
		} else if( integral_fraction_list ) {
			const integral = Number.parseInt( integral_fraction_list[ 1 ].replace( /[,\. ]/g, '' ) ) ;
			const fraction = Number.parseInt( integral_fraction_list[ 2 ].replace( /[,\. ]/g, '' ) ) ;
			if( isNaN( fraction ) ) {
				value = integral ;
			} else {
				value = integral + fraction / 100 ;
			}
		} else if( integral_list ) {
			value = integral = Number.parseInt( integral_list[ 1 ].replace( /[,\. ]/g, '' ) ) ;
		} else if( word_list ) {
			try{
				value = support.word2number( word_list[ 1 ] ) ;
			} catch( e ) {
				value = undefined ;
			}
		}
	}
	if( debug ) {
		console.log( 'parse_euro_cent( %s ) -> %f', txt, value ) ;
	}
	if( typeof value === 'undefined' || isNaN( value ) ) {
		if( debug ) {
			console.log( 'parse_euro_cent( %s ) -> %s', txt, value ) ;
		}
		return undefined ;
	} else {
		return value ;
	}
}

// List patterns representing the different ways in which courts prescribe a period.
const float_pat		= /^([0-9\.,]+) *([\.,] *\d+\b)(?:\(|\)|\[|\]|\b[a-z]+\b|,|-| )*(\b(?:uu?r|dag|wee?k|maand|jaa?r)(?:en)?\b)/ ;
const int_pat		= /([0-9\.,]+)(?:\(|\)|\[|\]|\b[a-z]+\b|,|-| )*(\b(?:uu?r|dag|wee?k|maand|jaa?r)(?:en)?\b)/ ;
const word_pat		= /((?:\b[a-z]+\b| |-){1,10})(?:\(|\)|\[|\]|\b[a-z]+\b|\b[0-9]+\b|,|-| ){0,10}(\b(?:uu?r|dag|wee?k|maand|jaa?r)(?:en)?\b)/ ;

const life_pat_list	= mk_pat_list( 'life',		'^', [ 'levenslange?' ], '' ) ;
const int_pat_list	= mk_pat_list( 'int',		'^', [ int_pat.source ], '' ) ;
const word_pat_list	= mk_pat_list( 'word',		'^', [ word_pat.source ], '' ) ;

const float_pat_list	= mk_pat_list( 'float',		'^', [ float_pat.source ], '' ) ;
const int_int_pat_list	= mk_pat_list( 'int_int',	'^', [ int_pat.source + ' *en * ' + int_pat.source ], '' ) ;
const word_word_pat_list= mk_pat_list( 'word_word',	'^', [ word_pat.source + ' *en * ' + word_pat.source ], '' ) ;
const word_int_pat_list	= mk_pat_list( 'word_int',	'^', [ word_pat.source + ' *en * ' + int_pat.source ], '' ) ;
const int_word_pat_list	= mk_pat_list( 'int_word',	'^', [ int_pat.source + ' *en * ' + word_pat.source ], '' ) ;

const single_pat_list	= [ life_pat_list, int_pat_list, word_pat_list ].flat() ;
const double_pat_list	= [ float_pat_list, int_int_pat_list, word_word_pat_list, word_int_pat_list, int_word_pat_list ].flat() ;

// Convert a string into a duration in hours, days, weeks, months or years.
function parse_duration( txt ) {
	var value, unit ;
	if( typeof txt !== 'undefined' ) {
		const normalised_txt = txt
			.replace( /(uren|dagen|weken|maanden|jaren)met/, '$1 met' )
			.replace( /^van/, '' )
			.replace( / *(?:voluit|maximaal|zeggen?)[: ]*/i, '' )
			.replace( /\b(?:en|met|een|,| )*(?:bepaalt|beveelt|bevel|proeftijd|(?:geheel )?voorwaardelijk|waarvan).*/i, '' )
			.toLowerCase() ;
		if( debug ) {
			console.log( 'parse_duration_0( %s )', normalised_txt ) ;
		}
		const single_match_list	= trace_match_list( normalised_txt, single_pat_list ) ;
		const double_match_list	= trace_match_list( normalised_txt, double_pat_list ) ;
		if( debug_debug) {
			console.log( 'single_match_list', single_match_list ) ;
			console.log( 'double_match_list', double_match_list ) ;
		}
		if( double_match_list ) {
			if( double_match_list.id.match( /^float_\d$/ ) ) {
				const int = Number.parseInt( double_match_list[ 1 ].replace( /[,\. ]/g, '' ) ) ;
				const fraction = Number.parseInt( double_match_list[ 2 ].replace( /[,\. ]/g, '' ) ) ;
				if( isNaN( fraction ) ) {
					value = int ;
				} else {
					const power = Math.pow(10, fraction.toString().length ) ;
					value = int + fraction / power ;
				}
				unit = support.normalise_unit( double_match_list[ 3 ] ) ;
			} else if( double_match_list.id.match( /^int_int_\d$/ ) ) {
				const first_value = Number.parseInt( double_match_list[ 1 ].replace( /[,\. ]/g, '' ) ) ;
				const first_unit = double_match_list[ 2 ] ;
				const second_value = Number.parseInt( double_match_list[ 3 ].replace( /[,\. ]/g, '' ) ) ;
				const second_unit = double_match_list[ 4 ] ;
				const hours = support.unit2hours( first_value, first_unit ) + support.unit2hours( second_value, second_unit ) ;
				value = Math.round( support.hours2unit( hours, second_unit ) ) ;
				unit = support.normalise_unit( second_unit ) ;
			} else if( double_match_list.id.match( /^word_word_\d$/ ) ) {
				const first_unit = double_match_list[ 2 ] ;
				const second_unit = double_match_list[ 4 ] ;
				try{
					const first_value = support.word2number( double_match_list[ 1 ] ) ;
					const second_value = support.word2number( double_match_list[ 3 ] ) ;
					const hours = support.unit2hours( first_value, first_unit ) + support.unit2hours( second_value, second_unit ) ;
					value = Math.round( support.hours2unit( hours, second_unit ) ) ;
					unit = support.normalise_unit( second_unit ) ;
				} catch( e ) {
					value = undefined ;
				}
				unit = support.normalise_unit( second_unit ) ;
			} else if( double_match_list.id.match( /^word_int_\d$/ ) ) {
				const first_unit = double_match_list[ 2 ] ;
				const second_unit = double_match_list[ 4 ] ;
				try{
					const first_value = support.word2number( double_match_list[ 1 ] ) ;
					const second_value = Number.parseInt( double_match_list[ 3 ].replace( /[,\. ]/g, '' ) ) ;
					const hours = support.unit2hours( first_value, first_unit ) + support.unit2hours( second_value, second_unit ) ;
					value = Math.round( support.hours2unit( hours, second_unit ) ) ;
					unit = support.normalise_unit( second_unit ) ;
				} catch( e ) {
					value = undefined ;
				}
				unit = support.normalise_unit( second_unit ) ;
			} else if( double_match_list.id.match( /^int_word_\d$/ ) ) {
				const first_unit = double_match_list[ 2 ] ;
				const second_unit = double_match_list[ 4 ] ;
				try{
					const first_value = Number.parseInt( double_match_list[ 1 ].replace( /[,\. ]/g, '' ) ) ;
					const second_value = support.word2number( double_match_list[ 3 ] ) ;
					const hours = support.unit2hours( first_value, first_unit ) + support.unit2hours( second_value, second_unit ) ;
					value = Math.round( support.hours2unit( hours, second_unit ) ) ;
					unit = support.normalise_unit( second_unit ) ;
				} catch( e ) {
					value = undefined ;
				}
				unit = support.normalise_unit( second_unit ) ;
			} else {
				console.log( 'unknown double_match_list %s', double_match_list.id ) ;
			}
		} else if( single_match_list ) {
			if( single_match_list.id.match( /^life_\d$/ ) ) {
				value = 1 ;
				unit = 'levenslang' ;
			} else if( single_match_list.id.match( /^int_\d$/ ) ) {
				value = int = Number.parseInt( single_match_list[ 1 ].replace( /[,\. ]/g, '' ) ) ;
				unit = support.normalise_unit( single_match_list[ 2 ] ) ;
			} else if( single_match_list.id.match( /^word_\d$/ ) ) {
				try{
					value = support.word2number( single_match_list[ 1 ] ) ;
				} catch( e ) {
					value = undefined ;
				}
				unit = support.normalise_unit( single_match_list[ 2 ] ) ;
			} else {
				console.log( 'unknown single_match_list %s', single_match_list.id ) ;
			}
		}
	}
	if( debug ) {
		console.log( 'parse_duration_2( %s ) -> {%s, %s}', txt, value, unit ) ;
	}
	if( typeof value === 'undefined' || isNaN( value ) || typeof unit === 'undefined' ) {
		if( debug ) {
			console.log( 'parse_duration_3( %s ) -> {%s, %s}', txt, value, unit ) ;
		}
		return { value: undefined, unit: undefined } ;
	} else {
		return { value: value, unit: unit } ;
	}
}

// List the categories for the variable Onderzoek
const case_list		= [
	['Abarth_I',		/\bAbarth *I\b/i],
	['Abarth_II',		/\bAbarth *II\b/i],
	['Abarth_III',		/\bAbarth *III\b/i],
	['Abedejo',		/10[ -]*Abedejo\b/i],
	['Aberdeen',		/10[ -]*Aberdeen\b/i],
	['Air_Holland',		/\bAir Holland\b/i],
	['Albali',		/10[ -]*Albali\b/i],
	['Alburtis',		/11[ -]*Alburtis\b/i],
	['ALFA',		/31[ -]*ALFA17\b/i],
	['ALTRADE',		/09[ -]*ALTRADE\b/i],
	['Amahai',		/\bAmahai\b/i],
	['Amstel',		/09[ -]*Amstel\b/i],
	['Amuse',		/30[ -]*Amuse18\b/i],
	['Angel',		/13[ -]*Angel\b/i],
	['Anker',		/09[ -]*Anker\b/i],
	['Apollo17',		/09[ -]*Apollo17\b/i],
	['ARARAT',		/04[ -]*ARARAT\b/i],
	['ARIA',		/13[ -]*ARIA\b/i],
	['Assen',		/09[ -]*Assen\b/i],
	['Astoria',		/26[ -]*Astoria\b/i],
	['Atik',		/13[ -]*Atik\b/i],
	['Aurora',		/09[ -]*Aurora\b/i],
	['Azie',		/31[ -]*Azie17\b/i],
	['Bammis',		/13[ -]*Bammis\b/i],
	['BANK',		/0*95[ -]*BANK\b/i],
	['BARI',		/161[ -]*BARI\b/i],
	['BASTARDO',		/26[ -]*BASTARDO\b/i],
	['Baudette',		/13[ -]*Baudette\b/i],
	['Beat',		/14[ -]*Beat\b/i],
	['Beugel',		/13[ -]*Beugel\b/i],
	['Bhutia',		/\bBhutia\b/i],
	['Bieslook',		/25[ -]*Bieslook\b/i],
	['Bijvoet',		/03[ -]*Bijvoet\b/i],
	['Biscoe',		/13[ -]*Biscoe\b/i],
	['Bison',		/09[ -]*Bison\b/i],
	['Blacksburg',		/26[ -]*Blacksburg\b/i],
	['Blauw',		/09[ -]*Blauw\b/i],
	['Blauwvis',		/09[ -]*Blauwvis\b/i],
	['Boni',		/09[ -]*Boni11\b/i],
	['Boonville',		/\bBoonville\b/i],
	['BORIUM',		/169[ -]*BORIUM\b/i],
	['BORRA',		/08[ -]*BORRA\b/i],
	['Bosduif',		/09[ -]*Bosduif\b/i],
	['Bosnie',		/\bBosnie\b/i],
	['Bosuil',		/09[ -]*Bosuil\b/i],
	['BOTO',		/13[ -]*BOTO\b/i],
	['Boxer',		/08[ -]*Boxer2\b/i],
	['BRDUSTY',		/09[ -]*BRDUSTY\b/i],
	['BREIFEL',		/09[ -]*BREIFEL\b/i],
	['BRFossil',		/09[ -]*BRFossil\b/i],
	['BrLeeuw',		/09[ -]*BrLeeuw\b/i],
	['BRListel',		/09[ -]*BRListel\b/i],
	['Brookhaven',		/26[ -]*Brookhaven\b/i],
	['BRUG',		/13[ -]*BRUG\b/i],
	['Bruiloft',		/09[ -]*Bruiloft12\b/i],
	['Buddleja',		/13[ -]*Buddleja\b/i],
	['Budget',		/09[ -]*Budget\b/i],
	['Burgos',		/162[ -]*Burgos\b/i],
	['BURTON',		/03[ -]*BURTON\b/i],
	['CAEN',		/162[ -]*CAEN\b/i],
	['CAMERON',		/26[ -]*CAMERON\b/i],
	['CAP',			/13[ -]*CAP\b/i],
	['CARGERIE',		/27[ -]*CARGERIE\b/i],
	['Catch',		/09[ -]*Catch14\b/i],
	['Cayenne',		/12[ -]*Cayenne\b/i],
	['Cicero',		/26[ -]*Cicero\b/i],
	['CIMI',		/13[ -]*CIMI\b/i],
	['Closer',		/26[ -]*Closer\b/i],
	['Coalinga',		/13[ -]*Coalinga\b/i],
	['Coel',		/0*31[ -]*Coel\b/i],
	['COLEUS',		/13[ -]*COLEUS\b/i],
	['Combo',		/09[ -]*Combo\b/i],
	['Comfort',		/\bComfort\b/i],
	['Commodore',		/\bCommodore\b/i],
	['Conroe',		/26[ -]*Conroe\b/i],
	['COSBY',		/09[ -]*COSBY\b/i],
	['Crockett',		/\bCrockett\b/i],
	['Croft',		/\bCroft\b/i],
	['CUCULUS',		/09[ -]*CUCULUS16\b/i],
	['Cupid',		/08[ -]*Cupid\b/i],
	['Cyllene',		/13[ -]*Cyllene\b/i],
	['Davits',		/10[ -]*Davits\b/i],
	['DEBET',		/09[ -]*DEBET15\b/i],
	['Deccan',		/10[ -]*Deccan\b/i],
	['Delft',		/09[ -]*Delft\b/i],
	['Delft',		/15[ -]*Delft100\b/i],
	['Delftplus',		/09[ -]*Delftplus\b/i],
	['DeLucht',		/26[ -]*DeLucht\b/i],
	['DeStrubben',		/26[ -]*DeStrubben\b/i],
	['Deventer',		/09[ -]*Deventer\b/i],
	['DeVink',		/26[ -]*DeVink\b/i],
	['DeWierde',		/26[ -]*DeWierde\b/i],
	['Dinar',		/13[ -]*Dinar\b/i],
	['Dinara',		/12[ -]*Dinara\b/i],
	['Doega',		/09[ -]*Doega\b/i],
	['Doetje',		/09[ -]*Doetje\b/i],
	['DON',			/13[ -]*DON\b/i],
	['Donar',		/10[ -]*Donar\b/i],
	['Donkey',		/\bDonkey\b/i],
	['Doorn',		/09[ -]*Doorn\b/i],
	['DREK',		/08[ -]*DREK\b/i],
	['Dreumel',		/12[ -]*Dreumel\b/i],
	['Drome',		/11[ -]*Drome\b/i],
	['Drooghok',		/09[ -]*Drooghok\b/i],
	['DUERO',		/11[ -]*DUERO\b/i],
	['Duiker',		/03[ -]*Duiker\b/i],
	['DUIM',		/09[ -]*DUIM\b/i],
	['Duister',		/03[ -]*Duister\b/i],
	['Dwemer',		/13[ -]*Dwemer\b/i],
	['Ebetsu',		/13[ -]*Ebetsu\b/i],
	['Eden',		/13[ -]*Eden\b/i],
	['Effer',		/13[ -]*Effer\b/i],
	['EMCRTO',		/17[ -]*EMCRTO\b/i],
	['Enclave',		/13[ -]*Enclave\b/i],
	['Fairborn',		/13[ -]*Fairborn\b/i],
	['Fantasie',		/09[ -]*Fantasie13\b/i],
	['Flatonia',		/13[ -]*Flatonia\b/i],
	['Fluor',		/26[ -]*Fluor\b/i],
	['FOSSIL',		/09[ -]*FOSSIL\b/i],
	['FRAXINUS',		/13[ -]*FRAXINUS\b/i],
	['Freesia',		/13[ -]*Freesia\b/i],
	['FRIGO',		/0*96[ -]*FRIGO\b/i],
	['Fris',		/09[ -]*Fris\b/i],
	['Frisco',		/13[ -]*Frisco\b/i],
	['Galjoen',		/26[ -]*Galjoen\b/i],
	['GAMA',		/0*31[ -]*GAMA\b/i],
	['Gatesi',		/13[ -]*Gatesi\b/i],
	['Gaudi',		/09[ -]*Gaudi\b/i],
	['GILERA',		/09[ -]*GILERA\b/i],
	['Goudvink',		/09[ -]*Goudvink\b/i],
	['GRAND',		/0*93[ -]*GRAND\b/i],
	['Gravesac',		/26[ -]*Gravesac\b/i],
	['Grimes',		/26[ -]*Grimes\b/i],
	['Groot',		/08[ -]*Groot\b/i],
	['Hailsham',		/13[ -]*Hailsham\b/i],
	['Hakijzer',		/13[ -]*Hakijzer\b/i],
	['HATI',		/09[ -]*HATI\b/i],
	['HEREN',		/0*95[ -]*HEREN\b/i],
	['Hermon',		/09[ -]*Hermon\b/i],
	['Hert',		/10[ -]*Hert\b/i],
	['Heuvel',		/10[ -]*Heuvel\b/i],
	['Hilaria',		/13[ -]*Hilaria\b/i],
	['Hillside',		/13[ -]*Hillside\b/i],
	['Hoedster',		/13[ -]*Hoedster\b/i],
	['Holmium',		/26[ -]*Holmium\b/i],
	['Hoofddorp',		/09[ -]*Hoofddorp\b/i],
	['Hoogvliet',		/09[ -]*Hoogvliet\b/i],
	['Hooper',		/26[ -]*Hooper\b/i],
	['IFTALOY',		/09[ -]*IFTALOY\b/i],
	['IFTMONTY',		/09[ -]*IFTMONTY\b/i],
	['IFTVERDI',		/09[ -]*IFTVERDI\b/i],
	['IJsberg',		/\bIJsberg\b/i],
	['IJshamer',		/13[ -]*IJshamer\b/i],
	['Imin',		/13[ -]*Imin\b/i],
	['INDU',		/09[ -]*INDU\b/i],
	['Ingooien',		/13[ -]*Ingooien\b/i],
	['Inktrol',		/13[ -]*Inktrol\b/i],
	['INTER',		/0*31[ -]*INTER\b/i],
	['Ippon',		/09[ -]*Ippon15\b/i],
	['IRISATIE',		/13[ -]*IRISATIE\b/i],
	['IRMO',		/13[ -]*IRMO\b/i],
	['Jager',		/09[ -]*Jager\b/i],
	['Janebell',		/26[ -]*Janebell\b/i],
	['JohnsonCity',		/26[ -]*JohnsonCity\b/i],
	['JUNCO',		/03[ -]*JUNCO\b/i],
	['Kachia',		/13[ -]*Kachia\b/i],
	['Kale',		/13[ -]*Kale\b/i],
	['Kalifornsky',		/\bKalifornsky\b/i],
	['Kalkton',		/13[ -]*Kalkton\b/i],
	['Kauw',		/08[ -]*Kauw\b/i],
	['Kaygrey',		/26[ -]*Kaygrey\b/i],
	['Kiem',		/13[ -]*Kiem\b/i],
	['Klip',		/09[ -]*Klip16\b/i],
	['Koelruit',		/13[ -]*Koelruit\b/i],
	['Kogge',		/26[ -]*Kogge\b/i],
	['KOLOA',		/13[ -]*KOLOA\b/i],
	['KOM',			/13[ -]*KOM\b/i],
	['KOPER',		/09[ -]*KOPER\b/i],
	['KOPER',		/26[ -]*KOPER\b/i],
	['Kotter',		/26[ -]*Kotter\b/i],
	['Kourion',		/13[ -]*Kourion\b/i],
	['KRAB',		/31[ -]*KRAB18\b/i],
	['KRAT',		/0*94[ -]*KRAT\b/i],
	['Krauwel',		/13[ -]*Krauwel\b/i],
	['Kruidvat',		/09[ -]*Kruidvat\b/i],
	['KUIFAREND',		/08[ -]*KUIFAREND\b/i],
	['Kutztown',		/26[ -]*Kutztown\b/i],
	['KWINI',		/09[ -]*KWINI\b/i],
	['Lagrein',		/26[ -]*Lagrein\b/i],
	['Lancashire',		/26[ -]*Lancashire\b/i],
	['LANDAL',		/25[ -]*LANDAL\b/i],
	['LEEUW',		/09[ -]*LEEUW\b/i],
	['LEIGRAAF',		/26[ -]*LEIGRAAF\b/i],
	['Lek',			/09[ -]*Lek18\b/i],
	['Leyburn',		/13[ -]*Leyburn\b/i],
	['Liesbos',		/26[ -]*Liesbos\b/i],
	['Lifan',		/03[ -]*Lifan\b/i],
	['Lilium',		/13[ -]*Lilium\b/i],
	['Livonia',		/26[ -]*Livonia\b/i],
	['Lockhart',		/26[ -]*Lockhart\b/i],
	['Loodpan',		/13[ -]*Loodpan\b/i],
	['Lostant',		/13[ -]*Lostant\b/i],
	['Lupus',		/09[ -]*Lupus\b/i],
	['LUTWYCHE',		/27[ -]*LUTWYCHE\b/i],
	['Maan',		/09[ -]*Maan15\b/i],
	['Maanvis',		/11[ -]*Maanvis\b/i],
	['Maas',		/11[ -]*Maas\b/i],
	['Manet',		/11[ -]*Manet\b/i],
	['Mango',		/09[ -]*Mango\b/i],
	['Mango',		/25[ -]*Mango\b/i],
	['Maplevalley',		/26[ -]*Maplevalley\b/i],
	['Marsman',		/\bMarsman\b/i],
	['MarsmanII',		/\bMarsmanII\b/i],
	['Mathaak',		/13[ -]*Mathaak\b/i],
	['Maywood',		/26[ -]*Maywood\b/i],
	['MEENT',		/0*32[ -]*MEENT\b/i],
	['Meerkoet',		/\bMeerkoet\b/i],
	['Megaliet',		/13[ -]*Megaliet\b/i],
	['Meiberg',		/26[ -]*Meiberg\b/i],
	['Melogale',		/26[ -]*Melogale\b/i],
	['Mercado',		/08[ -]*Mercado\b/i],
	['Metroid',		/\bMetroid\b/i],
	['Micro',		/31[ -]*Micro18\b/i],
	['Milan',		/26[ -]*Milan\b/i],
	['Minus',		/09[ -]*Minus13\b/i],
	['Mirabel',		/13[ -]*Mirabel\b/i],
	['Moer',		/13[ -]*Moer\b/i],
	['Mono',		/09[ -]*Mono15\b/i],
	['Monument',		/26[ -]*Monument\b/i],
	['Moore',		/26[ -]*Moore\b/i],
	['Mortel',		/13[ -]*Mortel\b/i],
	['Natrix',		/15[ -]*Natrix\b/i],
	['Natur',		/13[ -]*Natur\b/i],
	['NEBELHOORN',		/08[ -]*NEBELHOORN\b/i],
	['Nefelien',		/13[ -]*Nefelien\b/i],
	['Niger',		/14[ -]*Niger\b/i],
	['Nijntje',		/\bNijntje\b/i],
	['Nijver',		/10[ -]*Nijver\b/i],
	['NIXON',		/12[ -]*NIXON\b/i],
	['NOCIS',		/09[ -]*NOCIS\b/i],
	['NOORD',		/09[ -]*NOORD\b/i],
	['Noord',		/09[ -]*Noord16\b/i],
	['Noord',		/09[ -]*Noord\b/i],
	['Oceans',		/13[ -]*Oceans\b/i],
	['Octans',		/10[ -]*Octans\b/i],
	['Offside',		/13[ -]*Offside\b/i],
	['Ohey',		/13[ -]*Ohey\b/i],
	['Oliekan',		/13[ -]*Oliekan\b/i],
	['OLPE',		/13[ -]*OLPE\b/i],
	['Oogappel',		/13[ -]*Oogappel\b/i],
	['Oosterhout',		/09[ -]*Oosterhout\b/i],
	['Opkomst',		/13[ -]*Opkomst\b/i],
	['Oranje',		/31[ -]*Oranje17\b/i],
	['Ordino',		/13[ -]*Ordino\b/i],
	['Orland',		/26[ -]*Orland\b/i],
	['Oscoda',		/13[ -]*Oscoda\b/i],
	['Ostara',		/13[ -]*Ostara\b/i],
	['Overloon',		/13[ -]*Overloon\b/i],
	['Overspelt',		/13[ -]*Overspelt\b/i],
	['Panchax',		/13[ -]*Panchax\b/i],
	['PARIJS',		/09[ -]*PARIJS\b/i],
	['Parka',		/25[ -]*Parka\b/i],
	['Parkiet',		/14[ -]*Parkiet\b/i],
	['Patiel',		/26[ -]*Patiel\b/i],
	['PAVLOV',		/13[ -]*PAVLOV\b/i],
	['Pecan',		/26[ -]*Pecan\b/i],
	['PEER',		/09[ -]*PEER\b/i],
	['Pegasi',		/13[ -]*Pegasi\b/i],
	['Pelikaan',		/14[ -]*Pelikaan\b/i],
	['Pellaea',		/13[ -]*Pellaea\b/i],
	['Pelotas',		/13[ -]*Pelotas\b/i],
	['Peoria',		/26[ -]*Peoria\b/i],
	['Peseta',		/25[ -]*Peseta\b/i],
	['Piano',		/09[ -]*Piano\b/i],
	['Piano',		/09[ -]*PIANO17\b/i],
	['Pinch',		/26[ -]*Pinch\b/i],
	['PITTSTON',		/13[ -]*PITTSTON\b/i],
	['Polder',		/09[ -]*Polder\b/i],
	['Poort',		/25[ -]*Poort\b/i],
	['POPTANG',		/13[ -]*POPTANG\b/i],
	['Por',			/13[ -]*Por\b/i],
	['PRAAG',		/08[ -]*PRAAG\b/i],
	['Puli',		/13[ -]*Puli\b/i],
	['QRVALB',		/27[ -]*QRVALB06\b/i],
	['Quartair',		/13[ -]*Quartair\b/i],
	['Rakker',		/13[ -]*Rakker\b/i],
	['REDHILL',		/13[ -]*REDHILL\b/i],
	['REGEN',		/0*96[ -]*REGEN\b/i],
	['REX',			/13[ -]*REX\b/i],
	['Riesling',		/26[ -]*Riesling\b/i],
	['Rijk',		/09[ -]*Rijk14\b/i],
	['Roepie',		/09[ -]*Roepie\b/i],
	['Roerdomp',		/09[ -]*Roerdomp\b/i],
	['Rolwolk',		/13[ -]*Rolwolk\b/i],
	['Rome',		/08[ -]*Rome\b/i],
	['Roof',		/09[ -]*Roof\b/i],
	['Rooibos',		/13[ -]*Rooibos\b/i],
	['Roos',		/09[ -]*Roos\b/i],
	['RTBRAM',		/0*31[ -]*RTBRAM\b/i],
	['RTUNI',		/0*32[ -]*RTUNI\b/i],
	['Runner',		/09[ -]*Runner\b/i],
	['Safe',		/26[ -]*Safe\b/i],
	['SAMARA',		/03[ -]*SAMARA\b/i],
	['Sandston',		/25[ -]*Sandston\b/i],
	['SARA',		/0*31[ -]*SARA\b/i],
	['Saraland',		/13[ -]*Saraland\b/i],
	['Sassenheim',		/26[ -]*Sassenheim\b/i],
	['Schalmei',		/13[ -]*Schalmei\b/i],
	['Selderij',		/25[ -]*Selderij\b/i],
	['Sherlock',		/13[ -]*Sherlock\b/i],
	['Shogun',		/13[ -]*Shogun\b/i],
	['SINOPESA',		/13[ -]*SINOPESA\b/i],
	['Ska',			/14[ -]*Ska\b/i],
	['Skipers',		/13[ -]*Skipers\b/i],
	['Slag',		/13[ -]*Slag\b/i],
	['SMIT',		/0*32[ -]*SMIT\b/i],
	['SMUL',		/0*96[ -]*SMUL\b/i],
	['Somerville',		/26[ -]*Somerville\b/i],
	['Sortie',		/10[ -]*Sortie\b/i],
	['Spalter',		/13[ -]*Spalter\b/i],
	['Spons',		/13[ -]*Spons\b/i],
	['SPOTLIJSTER',		/08[ -]*SPOTLIJSTER\b/i],
	['Staalmeesters',	/07[ -]*Staalmeesters\b/i],
	['Stamford',		/10[ -]*Stamford\b/i],
	['Start',		/14[ -]*Start\b/i],
	['Steen',		/09[ -]*Steen13\b/i],
	['Ster16',		/09[ -]*Ster16\b/i],
	['STUK',		/14[ -]*STUK\b/i],
	['Subra',		/13[ -]*Subra\b/i],
	['Superior',		/26[ -]*Superior\b/i],
	['Sylvaner',		/26[ -]*Sylvaner\b/i],
	['TAAK',		/0*96[ -]*TAAK\b/i],
	['Tahoka',		/13[ -]*Tahoka\b/i],
	['TANDEM',		/26[ -]*TANDEM\b/i],
	['TandemII',		/26[ -]*TandemII\b/i],
	['TANGO',		/65[ -]*TANGO\b/i],
	['Tarangire',		/26[ -]*Tarangire\b/i],
	['TEBERIO',		/13[ -]*TEBERIO\b/i],
	['THOLEN',		/13[ -]*THOLEN\b/i],
	['Thomond',		/01[ -]*Thomond\b/i],
	['Tienbaan',		/26[ -]*Tienbaan\b/i],
	['Tifra',		/08[ -]*Tifra\b/i],
	['TIKTAK',		/0*94[ -]*TIKTAK\b/i],
	['Tilburg',		/09[ -]*Tilburg\b/i],
	['Tolnegen',		/26[ -]*Tolnegen\b/i],
	['Trajanus',		/13[ -]*Trajanus\b/i],
	['Trivor',		/11[ -]*Trivor\b/i],
	['UBA',			/13[ -]*UBA\b/i],
	['Udzungwa',		/26[ -]*Udzungwa\b/i],
	['Uithoorn',		/09[ -]*Uithoorn\b/i],
	['Ulawa',		/13[ -]*Ulawa\b/i],
	['Valk',		/08[ -]*Valk\b/i],
	['Vidalia',		/13[ -]*Vidalia\b/i],
	['Videlia',		/13[ -]*Videlia\b/i],
	['Vinger',		/09[ -]*Vinger\b/i],
	['VIOOL',		/09[ -]*VIOOL\b/i],
	['VLAM',		/31[ -]*VLAM18\b/i],
	['Vliegtuig',		/07[ -]*Vliegtuig\b/i],
	['VOGEL',		/03[ -]*VOGEL18\b/i],
	['Waitaki',		/13[ -]*Waitaki\b/i],
	['WAK',			/0*32[ -]*WAK\b/i],
	['WATER',		/08[ -]*WATER206\b/i],
	['WEB',			/0*94[ -]*WEB\b/i],
	['Weymouth',		/13[ -]*Weymouth\b/i],
	['Whaler',		/13[ -]*Whaler\b/i],
	['WICKER',		/09[ -]*WICKER\b/i],
	['Wierook',		/09[ -]*Wierook\b/i],
	['WIJK',		/0*97[ -]*WIJK\b/i],
	['Willemsbos',		/26[ -]*Willemsbos\b/i],
	['Wing',		/03[ -]*Wing\b/i],
	['Winsor',		/09[ -]*Winsor\b/i],
	['Wodan',		/10[ -]*Wodan\b/i],
	['Worland',		/25[ -]*Worland\b/i],
	['Wurtulla',		/27[ -]*Wurtulla\b/i],
	['Xanne',		/10[ -]*Xanne\b/i],
	['Yakima',		/13[ -]*Yakima\b/i],
	['YERING',		/27[ -]*YERING\b/i],
	['YMKEVER',		/12[ -]*YMKEVER\b/i],
	['ZEGEL',		/0*96[ -]*ZEGEL\b/i],
	['Zelda',		/13[ -]*Zelda\b/i],
	['Ziezo',		/09[ -]*Ziezo\b/i],
	['Zoetermeer',		/09[ -]*Zoetermeer\b/i],
	['Zomergem',		/13[ -]*Zomergem\b/i]
] ;

// List the categories for the variable Tools:
const tool_list		= [
	['DarkWebMonitor',	/\bdark *web *monitor\b/i],
	['Chainalysis',		/\bchainalysis\b/i],
	['Elliptic',		/\belliptic\b/i],
	['Ciphertrace',		/\bciphertrace\b/i],
	['Bitrank',		/\bbitrank\b/i],
	['Qlue',		/\bqlue\b/i]
] ;

// List the main banks
const bank_list		= [
	['ABNAmro',		/\bABN[ -]*Amro\b/i],
	['ASN',			/\bASN[ -]*(?:Bank|rekening|pas|kaart|betaal[ -]*(?:pas|kaart|rekening))\b/i ],
	['Bunq',		/\bBunq\b/i],
	['DHB',			/\bDHB[ -]*(?:Bank|rekening|pas|kaart|betaal[ -]*(?:pas|kaart|rekening))\b/i ],
	['ING',			/\bING[ -]*(?:Bank|rekening|pas|kaart|betaal[ -]*(?:pas|kaart|rekening))\b/i ],
	['Knab',		/\bKnab\b/i],
	['MasterCard',		/\bMaster[ -]*card\b/i ],
	['MoneyGram',		/\bMoney[ -]*Gram\b/i],
	['PayPal',		/\bPay[ -]*Pal\b/i],
	['Rabo',		/\bRabo[ -]*(?:Bank|rekening|pas|kaart|betaal[ -]*(?:pas|kaart|rekening))\b/i ],
	['Regio',		/\bRegio[ -]*(?:Bank|rekening|pas|kaart|betaal[ -]*(?:pas|kaart|rekening))\b/i ],
	['SNS',			/\bSNS[ -]*(?:Bank|rekening|pas|kaart|betaal[ -]*(?:pas|kaart|rekening))\b/i ],
	['Triodos',		/\bTriodos[ -]*(?:Bank|rekening|pas|kaart|betaal[ -]*(?:pas|kaart|rekening))\b/i ],
	['Visa',		/\bVisa[ -]*(?:Bank|rekening|pas|kaart|betaal[ -]*(?:pas|kaart|rekening))\b/i ],
	['WesternUnion',	/\bWestern[ -]*Union\b/i],
	['Bankieren',		/\binternet[ -]*(?:bankieren|rekening)\b/i]
] ;

// List the most prominent market places
const market_list	= [
	'abraxas',
	'agora',
	'alpha[ -]*bay',
	'cloud9',
	'dark[ -]*(?:net|web)?',
	'dream',
	'empire',
	'evolution',
	'hansa',
	'middle[ -]*earth',
	'nessus',
	'nucleus',
	'ondergrondse',
	'online',
	'outlaw',
	'pandora',
	'silk[ -]*road(?:[0-9\.]*)?',
	'valhalla',
	'wall[ -]*street'
] ;

// List the categories for the variable Expertise_verdachte.
const expertise_defendant_list = [
	['TOR',			/\b(?:the *onion *router|tor|tormail|torchat|onion|hidden[ -]*server)\b/i],
	['BlackPass',		/\bblackpass\b/i],
	['Bullet_proof',	/\bbullet[ -]*proof\b/i],
	['Crypto_currency',	/\bmonero\b|\bethereum\b|\bbitcoins?\b|\bbitcoin[ -]*(?:fora|forums?|clusters?|adres?e?n?|addresse?s?|markte?n?|markets?|wallets?)\b/i],
	['DDoS',		/\b(?:ddos|booter|denial[ -]*of[ -]*service|DoS[ -]*aanval)\b/i],
	['Darkweb',		/\b(?:dark[ -]*net\b|dark[ -]*web)(?:[ -]*winkel)?\b/i],
	['Darkweb',		/\bonline marktplaats[^\.]Silk Road\b/i],
	['Exchange',		/\b(?:Kraken.com|NEXT.exchange|bitonic|bitconic|anycoin|coinvert)\b/i],
	['Exchange',		/\bbitcoin[ -]*(?:exchange[rs]*(?:[ -]*dienst)?|wisselkantoo?re?n?|ATM|handelaa?re?n?|beurs)\b/i],
	['Exit',		/\bexit[ -]*node\b/i],
	['Hacking',		/\b(?:gehackte?|hacken|hackers?|hacking)\b/i],
	['ICO',			/\b(?:initial coin offering|ico)\b/i],
	['Malware',		/\b(?:malware|malwareprogramma|exploit|remote *access *tool|bitcryptor|torrat)\b/i],
	['Market',		/\b(?:black[ -]*market[ -]*reloaded|webshop voor drugs)\b/i],
	['Market',		/\b(?:webshop voor drugs|marktplaats [^\.]* verdovende middelen)\b/i],
	['Market',		new RegExp( '\\b(?:' + market_list.join( '|' ) + ')[ -]*(?:markets?|markte?n?|markt[ -]*plaatse?n?)\\b', 'i' )],
	['Market',		new RegExp( '\\b(?:markets?|markte?n?|markt[ -]*plaatse?n?)[ -]*(?:' + market_list.join( '|' ) + ')\\b', 'i' )],
	['MetaSploit',		/\bmetasploit\b/i],
	['Mining',		/\b(?:(?:mining|miners?)[^\.]*(?:blockchain|bitcoins?)|(?:blockchain|bitcoins?)[^\.]*(?:mining|miners?))\b/i],
	['Mining',		/\b(?:bitcoins?|ether|ethereum)[ -]*(?:(?:heeft )?gemined|miners?|mining)\b/i],
	['Mining',		/\b(?:miner|mining)[ -]*(?:capaciteit|dienste?n?|rig|pools?|processe?s?|van (?:bitcoin|ethere?u?m?))\b/i],
	['Mining',		/\b(?:scrypt\.cc|antpool\.com|f2pool\.com|slushpool\.com)\b/i],
	['Mining',		/\bminers[^\.]*validiteit van transacties te controleren\b/i],
	['Mixing',		/\b(?:bitcoins?|ethere?u?m?)[ -]*(?:(?:heeft )gemixed|mixers?|mixing)\b/i],
	['Mixing',		/\b(?:mixer|mixing)[ -]*(?:dienste?n?|rig|processe?s?)\b/i],
	['Mixing',		/\b(?:sharedcoin|smartmix\.io|bitcoin-laundry\.com|anonymix\.io|bitcoinmixer\.org|smartmixer\.io|blender\.io|cryptomixer\.io|www\.privcoin\.io|bitblender\.io|bitcoinfog\.info)\b/i],
	['Phishing',		/\b(?:phishers?|phishing(?:website|sites?|mails?|fraude)?|gephishte?|phishen)\b/i],

	['Fraud',		/\b(?:\(?internet[\) -]*(?:oplichting|oplichters?))\b/i],
	['Fraud',		/\b(?:identiteits|internet|marktplaats|phishing|cyber)[ -]*fraude\b/i],
	['Fraud',		/\b(?:namaakwebshops?|nepsite)\b/i ],
	['Fraud',		/\b(?:\(?cyber\)?[ -]*oplichting(?:szaken)?)\b/i ],
	['Fraud',		/\boplichting en misbruik van (?:zijn|haar) identiteit via (?:het )?internet\b/i],
	['Fraud',		/\boplichting gepleegd (?:had )?met misbruik van DIGID\b/i],
	['Fraud',		/\boplichting (?:pleegde )?via (?:het )?internet\b/i],
	['Fraud',		/\boplichting zich uitstrekte tot meerdere webshops\b/i],
	['Fraud',		/\boplichting,* door op (?:het )?internet goederen (?:te koop )?aan te bieden\b/i],
	['Fraud',		/\boplichtingspraktijke?n? op (?:het )?internet\b/i],
	['Fraud',		/\boplichtingszaa?ke?n? via (?:het )?internet\b/i],

	['Ransomware',		/\bransomware\b/i],
	['Sniffing',		/\b(?:packet[ -]*)?(?:sniffer|sniffing)\b/i],
	['Wallet',		/\b(?:online|bitcoin|paycoin|opalcoin|multibit|ledger|blockchain|blokchain|prive|ingelogde|digitale?)[ -]*wallets?\b/i],
	['Wallet',		/\b(?:trezor|electrum|Virwox[ -]*account)\b/i],
	['Wallet',		/\b(?:wallets?[^\.]bitcoins?|bitcoins?[^\.]*wallets?)\b/i],
	['Wallet',		/\bwallet[ -]*(?:bestand|add?ress?|clusters?)\b/i],
	['Wallet',		/\bwallet[^\.]*(?:public|private) *keys?\b/i]
] ;

// List patterns indicating a phone.
const mobile_list	= [
	'Blackberry',
	'GSM',
	'Huawei',
	'Nokia',
	'Samsung[ -]*(?:Galaxy|telefoons?)',
	'WhatsApp[ -]*gesprekk?e?n?',
	'feature[ -]*phones?',
	'iPhones?',
	'mobiele[ -]*(?:telefoons?|toestell?e?n?)',
	'mobieltjes?',
	'simkaart',
	'smart[ -]*phones?'
] ;

// List standard internet related words; these are categories for the variable Internet.
const internet_list	= [
	['Browser',		/\b(?:browsers?|chrome|edge|firefox|internet[ -]*explorer)\b/i ],
	['Communication',	/\b(?:skype|virtual[ -]*private[ -]*network|wi[ -]*fi)\b/i ],
	['Company',		/\b(?:apple|ibm|microsoft)\b/i ],
	['Generic',		/\b(?:computer|online|internet[a-z]*(?<!tap)|tablets?|web[ -]*sites?|partition)\b/i ],
	['Hardware',		/\b(?:floppy[ -]*disk|hard[ -]*disks?|hardware|iphone|laptops?|pc|routers?|mobieltje|mobiele[ -]*telefoon|usb)\b/i ],
	['Mail',		/\b(?:e[ -]*mail|e[ -]*mail[ -]*bericht|gmail|forum|hotmail|outlook|protonmail)\b/i ],
	['Mobile',		new RegExp( '\\b(?:' + mobile_list.join( '|' ) + '?)\\b', 'i' ) ],
	['Moneymule',		/\b(?:money[ -]*mu?les?|geld[ -]*ezels?|kat[ -]*vangers?)\b/i],
	['Payment',		/\b(?:paypal|alipay)\b/i ],
	['Phone',		/\b(?:satt?ell?iettelefoons?|telefoon[a-z]*(?<!tap))\b/i ],
	['Protocol',		/\b(?:dns|ftp|https?|ssh|tls)\b/i ],
	['Search',		/\b(?:bing|google|wikipedia|yahoo)\b/i ],
	['Security',		/\b(?:disk[ -]*encryption|inlog[ -]*codes?|inlog[ -]*gegevens?|passwords?|smart[ -]*cards?|wachtwoorde?n?)\b/i ],
	['Service',		/\b(?:digid|hosting)\b/i ],
	['Shop',		/\b(?:alibaba|airbnb|amazon|bol\.com|booking\.com|expedia|marktplaats\.nl|uber)\b/i ],
	['SocialNetwork',	/\b(?:facebook|hyves|instagram|linkedin|pinterest|snapchat|reddit|tiktok|twitter|tumblr|youtube)\b/i ],
	['Software',		/\b(?:data[ -]*base|data[ -]*structuur|software)\b/i ],
	['System',		/\b(?:android|linux|macos|windows|docker|virtualbox|vmware)\b/i ],
	['VPN',			/\b(?:virtual[ -]*private[ -]*network|VPN|nord[ -]*vpn|surf[ -]*shark|private[ -]*vpn|vypr[ -]*vpn|proton[ -]*vpn|express[ -]*vpn|pure[ -]*vpn|safer[ -]*vpn|zenmate|hide[ -]*my[ -]*ass|cyber[ -]*ghost|ip[ -]*vanish|hot[ -]*spot[ -]*shield)\b/i ],
	['Video',		/\b(?:dailymotion|netflix|vimeo)\b/i ],
	['WhatsApp',		/whatsapp/i ]

] ;

// List patterns indicating encrypted mobiles.
const crypto_list	= [
	'cryptofoon',
	'phantom[ -]*secure',
	'(?:PGPs|PGPtjes?)',
	'(?:PGP|encryptie|versleutelde|encrypted|crypto)[ -]*(?:telefoon[a-z]*|gesprekk?e?n?|toestell?e?n?|mobiel[a-z]*)'
] ;
const mobile_crypto_list = mobile_list.concat( crypto_list ) ;

// List confidentiality related keywords; these are categories for the variable Vertrouwelijkheid
const confidentiality_list = [
	['PGP-1BC',		/\bno1bc\b/i ],
	['PGP-BlackBerry',	/\b(?:BB met PGP|Blackberry *?s? met PGP|BlackBerry (?:Wit)? *PGP|PGP[ -]*BB|PGP[ -]*Blackberry *s?)\b/i ],
	['PGP-Certgate',	/\b(?:Certgate|CGcard|CGphone)\b/i ],
	['PGP-Encro',		/\bEncro[ -]*chat\b/i ],
	['PGP-Ennet',		/\bEnnet[ -]*com\b/i ],
	['PGP-GhostPGP',	/\b(?:Ghost[ -]ECC|Ghost[ -]*PGP)\b/i ],
	['PGP-IronChat',	/\b(?:Iron[ -]*Chat|Iron[ -]*Phone|Black[ -]*Box[ -]*Security)\b/i ],
	['PGP-Nokia',		/\bNokia *?s? met PGP\b/i ],
	['PGP-PhantomSecure',	/\b(?:Phantom[ -]*Secure|phantomsecurepgp)\b/i ],
	['PGP-Safe',		/\bPGP[ -]*Safe\b/i ],
	['PGP-Sky',		/\bSky[ -]*ECC\b/i ],
	['PGP-telefoon',	new RegExp ( '\\b(?:' + crypto_list.join( '|' ) + ')\\b', 'i' ) ],
	['HushMail',		/\bHush[ -]*Mai\b/i ],
	['Protocol',		/\b(?:PGP|PGPnaa?me?n?|PGPsleutels?|PGPsoftware)\b/i ],
	['ProtonMail',		/\bProton[ -]*Mail\b/i ],
	['SecureDrop',		/\bSecure[ -]*Drop\b/i ],
	['Signal',		/\b(?:telefoon bevat onder meer de applicatie Signal|via de app Signal een bericht|communiceerde via Signal|Signal *protocol)\b/i ],
	['Telegram',		/\btelegram *(?:channels?|kanaal|groups?|groepe?n?|messages?|berichte?n?|protocol)\b/i ]
] ;

// List the categories for the variable Expertise_rechtbank:
const expertise_court_list = [
	['NFI',			/\b(?:NFI|Nederlands Forensisch Instituut|digitaal forensisch|forensisch technisch)\b/i],
	['TMFI',		/\b(?:TMFI|The Maastricht Forensic Institute)\b/i],
	['TNO',			/\b(?:TNO|Nederlandse Organisatie voor Toegepast Natuurwetenschappelijk Onderzoek)\b/i],
	['ExpertWitness',	/\bdeskundige\b[^\.]*\b(?:NFI|TMFI|TNO|van Wegberg)\b/i]
] ;

// List patterns indicative of investigating a laptop.
const laptop_list	= [
	'(?:bevindingen|onderzocht|onderzoek)(?:aan|Lenovo|de|een| )*laptop',
	'(?:Macbook Air|laptop) in beslag genomen',
	'verdachte is een laptop aangetroffen',
	'harde schijven in beslag genomen'
] ;

// List patterns indicative of breaking the protection of a mobile phone.
const readout_list	= [
	'AIVD werd geopend',
	'uit te lezen',
	'uitge?lezen',
	'in beslag genomen',
	'gekraakt',
	'digitaal onderzoek',
	'uitlezen'
] ;

// List words indicative of a phone being close to a cell tower.
const radiate_list	= [
	'aangestraald',
	'aanstraalde',
	'aanstralen'
];

// List the indications that law enforcement has used its special powers; these are categories for the variable Opsporing
const investigation_list = [
	['Infiltratie',		/\b(?:126[ -]*h(?:van|het| )*(?:Sv|Wetboek(?:van| )*Strafvordering))\b/i],
	['Infiltratie',		/\bpolitie[ -]*(?:infiltratie|infiltrante?n?)\b/i ],

	['Pseudo-koop',		/\b(?:126[ -]*i(?:van|het| )*(?:Sv|Wetboek(?:van| )*Strafvordering))\b/i],
	['Pseudo-koop',		/\bpseudo[ -]*(?:\(?(?:aan|proef|ver)[ -]*\)?(?:koop|kope[rn])|koop|kope[rn]|dienstverlening)\b/i ],

	['Tap-plaatsen',	/\b(?:126[ -]*l(?:van|het| )*(?:Sv|Wetboek(?:van| )*Strafvordering))\b/i],
	['Tap-gesprek',		/\b(?:opgenomen|tap|OVC)[ -]*gesprekk?e?n?\b/i],
	['Tap-bericht',		/\b(?:opgenomen|tap|OVC)[ -]*berichte?n?\b/i],
	['Tap-gesprek',		/\b(?:tapp?e?n?(?:op|dit|van|de|het|zijn|haar| )*(?:telefoon(?:gesprekk?e?n?)?|gesprekk?e?n?|smart[ -]*phones?|GSM|mobiele[ -]*telefoons?|mobieltjes?))\b/i],
	['Tap-bericht',		/\b(?:tapp?e?n?(?:op|dit|van|de|het|zijn|haar| )*(?:IP[ -]*adres|internet(?:verkeer|aansluiting|verbinding)?))\b/i],
	['Tap-bericht',		/\b(?:IP[ -]*adres|internet(?:verkeer|aansluiting|verbinding)?)[ -]*tapp?e?n?\b/i],
	['Tap-gesprek',		/\b(?:telefoons?|telefoonnummers?|telefoongesprekk?e?n?|gesprekk?e?n?|smart[ -]*phones?|GSM|mobiele[ -]*telefoons?|mobieltjes?)[ -]*(?:afluisteren|afgeluisterd|getapte?|tapp?e?n?)\b/i],
	['Tap-gesprek',		/\b(?:afluisteren|afgeluisterd|tappen|getapte?) *(?:telefoons?|telefoonnummers?|telefoongesprekk?e?n?|gesprekk?e?n?|smart[ -]*phones?|GSM|mobiele[ -]*telefoons?|mobieltjes?)\b/i],
	['Tap-plaatsen',	/\b(?:aanbrengen|(?:in)?zetten|plaatsen|leggen|vorderen)(?:van|de|een| )*tapp?e?n?\b/i],
	['Tap-plaatsen',	/\b(?:tapp?e?n?(?:was|is| )*(?:geplaatst|gezet|gelegd|afgegeven|gevorderd))\b/i],
	['Tap-plaatsen',	/\b(?:bevel|vordering|opsporingsmiddel|opsporingsbevoegdheid|machtiging *(?:verleend)?|gemachtigd|rechtmatigheid)(?:om|te|van|tot| )*tapp?e?n?\b/i],
	['Tap-opname',		/\b(?:(?:opname|opnemen)(?:van|vertrouwelijke| )*(?:tele[ -]*)?communicatie|gesprekken)\b/i],
	['Tap-opname',		/\b(?:meeluisterende opsporingsinstanties|data observaties?)\b/i],
	['Tap-opname',		/\b(?:(?:tappen|getapt)[^\.]*door de politie?)\b/i],

	['Uitlezen',		/\b(?:Cellebrite|UFED)\b/i ],
	['Uitlezen',		/\b(?:AccessData|Magnet[ -]*Forensiscs?)\b/i ],
	['Uitlezen',		/\b(?:Microsystemation|XRY)\b/i ],
	['Uitlezen',		new RegExp( '\\b(?:' + mobile_crypto_list.join( '|' ) + ')[^\\.]*(' + readout_list.join( '|' ) + ')\\b', 'i' ) ],
	['Uitlezen',		new RegExp( '\\b(?:' + readout_list.join( '|' ) + ')[^\\.]*(' + mobile_crypto_list.join( '|' ) + ')\\b', 'i' ) ],
	['Uitlezen',		new RegExp( '\\b(?:' + laptop_list.join( '|' ) + ')\\b', 'i' ) ],

	['Localiseren',		/\b(?:126[ -]*ma?(?:van|het| )*(?:Sv|Wetboek(?:van| )*Strafvordering))\b/i],
	['Localiseren',		new RegExp( '\\b(?:' + radiate_list.join( '|' ) + ')[^\\.]*zendmaste?n?\\b', 'i' ) ],
	['Localiseren',		new RegExp( '\\bzendmaste?n?[^\\.]*(?:' + radiate_list.join( '|' ) + ')\\b', 'i' ) ],
	['Localiseren-IMSI',	/\b(?:IMSI[ -]*catcher)\b/i ],

	['Legaal-hacken',	/\b(?:126nba(?:van|het| )*(?:Sv|Wetboek(?:van| )*Strafvordering))\b/i],
	['Legaal-hacken',	/\bpolitie[ -]*(?:hackers?|hacking|hacks?)\b/i ],
	['Legaal-hacken',	/\bDigital[ -]*Intrusion[ -]*Team\b/i ],

	['Aanhouding',		/\b(?:(?:geobserveerd|observeerde|observaties?)[^\.]*(?:plaatsgevonden|verdachte|perceel|woning|politie))\b/i ],
	['Aanhouding',		/\b(?:Ennetcom-dataset werd doorzocht)\b/i ],
	['Aanhouding',		/\b(?:Meld Misdaad Anoniem)\b/i ],
	['Aanhouding',		/\b(?:Team Criminele Inlichtingen|TCI[ -]*melding)\b/i ],
	['Aanhouding',		/\b(?:camera[^\.]*verdachte[^\.]*schiet)\b/i ],
	['Aanhouding',		/\b(?:dodelijke[^\.]*schoten)\b/i ],
	['Aanhouding',		/\b(?:gewelddadige incidenten)\b/i ],
	['Aanhouding',		/\b(?:medewerker(?:van|het| )*transportbedrijf[^\.]*verdachte zending)\b/i ],
	['Aanhouding',		/\b(?:melding[^\.]*verdacht[^\.]*transport[a-z]*)\b/i ],
	['Aanhouding',		/\b(?:meldkamer[^\.]*persoon[^\.]*neergeschoten)\b/i ],
	['Aanhouding',		/\b(?:observaties?[^\.]*peilbakengegevens?)\b/i ],
	['Aanhouding',		/\b(?:poginge?n?(?:tot| )*(?:afpersing|oplichting|omkoping))\b/i ],
	['Aanhouding',		/\b(?:slachtoffer[^\.]*aangifte[^\.]*geweld)\b/i ],
	['Aanhouding',		/\b(?:verbalisant[^\.]*ANPR[^\.]*hit)\b/i ],
	['Aanhouding',		/\b(?:verbalisante?n?[^\.]*woning[^\.]*betreden|meldinge?n?[^\.]*doorzoeking[^\.]*woninge?n?)\b/i ],
	['Aanhouding',		/\b(?:verdachte[^\.]*staande gehouden)\b/i ],
	['Aanhouding',		/\b(?:verdachte[^\.]*woning van aangeefster)\b/i ],
	['Aanhouding',		/\b(?:vliegtuig[^\.]*stopteken[^\.]*Marechaussee)\b/i ],
	['Aanhouding',		/\b(?:woning[^\.]*verdachte[^\.]*doorzocht)\b/i ],
	['Aanhouding',		/\b(?:spookburgeractie|handhavingsactie)\b/i ],

	['Internationaal',	/\b(?:Landelijk Internationaal Rechtshulp Centrum|LIRC)\b/i ],
	['Internationaal',	/\b(?:ontving[^\.]*FIOD[^\.]*Amerikaanse autoriteiten)\b/i ]
] ;

// List terrorism related categories of the variable Terrorisme.
const terrorism_list	= [
	['UCC',			/\b(?:United Cyber Caliphate)\b/i ],
	['Jihad',		/\b(?:jihad|jihadistische?|jihadisme)\b/i ],
	['terrorisme',		/\b(?:terrorisme|terroristische?)\b/i ],
	['Al-Qaida',		/\b(?:Al[ -]*Qaeda|Al[ -]*Qaidah|Al[ -]*Qaida)\b/i ],
	['ISIS',		/\b(?:ISIS|Islamic state|ISIL|Daesh)\b/i ],
] ;

// List sentencing guidelines. This is a combination of the lovs_list and the om_list.
const lovs_list = [
	['LOVS',		/\b(?:Landelijk Overleg Vakinhoud Strafrecht|LOVS|orientatiepunte?n?)/ ]
] ;

const om_list = [
	[ '2020R003', 'misbruik seksueel beeldmateriaal' ],
	[ '2020R006', 'jeugd en adolescenten' ],
	[ '2017R002', 'strafrechtelijke aanpak schoolverzuim' ],
	[ '2020R001', 'vuurwerkdelicten' ],
	[ '2018R001', 'cybercrime' ],
	[ '2019A003', 'meerderjarigen' ],
	[ '2019R013', 'rijden onder invloed', 'rijden tijdens een rijverbod' ],
	[ '2020R002', 'voorbereiding/bevordering synthetische drugs' ],
	[ '2019R009', 'zware mishandeling' ],
	[ '2015R038', 'zakkenrollerij' ],
	[ '2015R052', 'witwassen' ],
	[ '2015R030', 'Winkeltijdenwet' ],
	[ '2018R020', 'Wet wegvervoer goederen' ],
	[ '2018R014', 'Wet vervoer gevaarlijke stoffen ten aanzien van vervoer over de weg' ],
	[ '2018R015', 'Wet personenvervoer 2000' ],
	[ '2015R003', 'wederspannigheid' ],
	[ '2011R033', 'Warenwet' ],
	[ '2016R017', 'wapens en munitie' ],
	[ '2018R018', 'vliegen onder invloed' ],
	[ '2015R015', 'vernieling' ],
	[ '2012R012', 'verkrachting' ],
	[ '2019R014', 'verkeersongevallen en verlaten plaats ongeval' ],
	[ '2018R008', 'verduistering in dienstbetrekking' ],
	[ '2015R042', 'valsheid in geschrift' ],
	[ '2015R024', 'valse of vervalste kentekenplaten' ],
	[ '2015R005', 'valse aangifte' ],
	[ '2015R045', 'vals/vervalst reisdocument' ],
	[ '2016R013', 'uitgeven enz. van vals geld' ],
	[ '2018R017', 'Telecommunicatiewet' ],
	[ '2018R019', 'strafrechtelijke aansprakelijkheid voor de aanvoer van niet of onjuist gedocumenteerde vreemdelingen' ],
	[ '2015R034', 'straatroof' ],
	[ '2016R002', 'sociale zekerheidsfraude' ],
	[ '2019R003', 'smaad(schrift) en lasterlijke aanklacht' ],
	[ '2018R006', 'ram- en plofkraken' ],
	[ '2015R047', 'seksueel misbruik van minderjarigen' ],
	[ '2015R006', 'schennis van de eerbaarheid' ],
	[ '2019R006', 'poging doodslag' ],
	[ '2019R008', 'poging zware mishandeling' ],
	[ '2016R001', 'overvallen' ],
	[ '2017R006', 'opruiing' ],
	[ '2019R001', 'oplichting' ],
	[ '2018R013', 'Opiumwet, softdrugs' ],
	[ '2019R011', 'Opiumwet, harddrugs' ],
	[ '2019R002', 'openlijke geweldpleging' ],
	[ '2015R036', 'onwettig verblijf' ],
	[ '2018R003', 'onttrekking minderjarige aan wettig gezag' ],
	[ '2015R004', 'niet opvolgen ambtelijk bevel of gedragsaanwijzing' ],
	[ '2019R015', 'motorrijtuig (doen) besturen tijdens ontzegging' ],
	[ '2019R010', 'mobiel banditisme' ],
	[ '2019R007', 'mishandeling' ],
	[ '2015R039', 'metaaldiefstal' ],
	[ '2018R002', 'mensensmokkel' ],
	[ '2016R003', 'mensenhandel' ],
	[ '2015R037', 'meineed' ],
	[ '2018R011', 'Luchtvaartwetgeving' ],
	[ '2016R010', 'Kinderpornografie' ],
	[ '2015R050', 'kindermishandeling' ],
	[ '2015R023', 'joyriding' ],
	[ '2015R013', 'inbraken overig' ],
	[ '2015R001', 'discriminatie' ],
	[ '2015R049', 'huiselijk geweld' ],
	[ '2015R033', 'inbraak', 'insluiping' ],
	[ '2016R016', 'heling' ],
	[ '2016R014', 'diefstal voertuigen' ],
	[ '2015R048', 'belaging' ],
	[ '2015R046', 'aanranding der eerbaarheid' ],
	[ '2015R002', 'huis- en lokaalvredebreuk' ],
	[ '2019R016', 'feitgecodeerde misdrijven en overtredingen' ],
	[ '2015R017', 'dierenmishandeling', 'dierenverwaarlozing' ],
	[ '2015R043', 'brandstichting' ],
	[ '2018R016', 'Binnenvaart' ],
	[ '2013R017', 'bij meerderjarige veelplegers' ],
	[ '2019R004', 'belediging' ],
	[ '2019R005', 'bedreiging' ],
	[ '2015R054', 'art. 248b Sr' ],
	[ '2018R009', '(winkel)diefstal/verduistering' ]
] ;
const guideline_list = lovs_list.concat( om_list.map( item_list => {
	const key = item_list[ 0 ] ;
	return [ key, new RegExp ( '\\b(?:aanwijzing|richtlijn) (?:en kader )?(?:(?:voor )strafvordering )?' +
		'(?:\\(' + key + '\\)|' + item_list.join( '|' ) + ')', 'i' ) ] ;
} ) ) ;

// List intensity related keywords
const intensity_list = [
	['Grootschalig',	/\b(?:grootschalig|grootscheeps|omvangrijk)/ ],
	['Frequent',		/\b(?:meermalen|meermalig|frequent|herhaaldelijki|veelvuldig)/ ],
	['Langdurig',		/\b(?:langdurig|hardnekkig|gedurende geruime tijd|gedurende langer?e? tijd)/ ]
] ;

// List threat related keywords
const threat_list = [
	['Gevaar',		/\b(?:(?:bedreiging|gevaar|risico)(?:van|voor|de| )*(?:gemeenschap|samenleving|maatschappij)|beroepscrimineel)\b/ ],
	['Witwassen',		/\b(?:witassen|witgewassen|witwas[a-z]*|witwissen)\b/ ],
	['Vuurwapen',		/\b(?:aanvals|hand|vuist|schouder)?vuurwapen[a-z]*\b/ ]
] ;

// Extract keywords from the text
function extract_keywords( keyword_list, txt ) {
	const result = keyword_list
		.map( item => trace_match( txt, item[ 1 ] ) ? item[ 0 ] : '' )
		.filter( item => item !== '' ) ;
	return result.length > 0 ? support.uniq( result ) : undefined ;
}

// Patterns to extract how many people with certain roles were involved
const role_prefix_txt	= '(?:\\(|\\[) *(?:de|onbekend *gebleven|aanduiding *voor|roepnaam|voornaa?me?n?|voorletters? *en *achternaam|achternaam|familienaam|naam|schuilnaam[0-9 ]*|bijnaam[0-9 ]*|alias[0-9 ]*|nickname[0-9 ]*)? *' ;
const role_postfix_txt	= ' *(?:organisatie|bedrijf|rechtspersoon)? *([0-9A-Z]*)[ \\.]*(?:\\)|\\])'

const defendant_pat	= new RegExp( role_prefix_txt + '(?:hoofdverdachten?|verdachten?|veroordeelden?|daders?)' + role_postfix_txt, 'gi' ) ;
const codefendant_pat	= new RegExp( role_prefix_txt + '(?:medeverdachten?|medeveroordeelden?|mededaders?)' + role_postfix_txt, 'gi' ) ;
const reporter_pat	= new RegExp( role_prefix_txt + '(?:aangevers?|aangeefsters?)' + role_postfix_txt, 'gi' ) ;
const witness_pat	= new RegExp( role_prefix_txt + '(?:getuigen?|betrokkenen?|betrokken *partije?n?)' + role_postfix_txt, 'gi' ) ;
const victim_pat	= new RegExp( role_prefix_txt + '(?:slachtoffers?|benadeelden?|benadeelde *partije?n?)' + role_postfix_txt, 'gi' ) ;

// Patterns with special phrases that the courts have used at least once to indicate multiple co-defendants
// This is used for example in ECLI:NL:RBAMS:2019:8506
const multiple_codefendant_pat	= /\bverdachte[, ]*(?:en|en\/of)? *(?:zijn|haar)? *(?:medeverdachten?|mededaders?)\b/i ;

// Extract role counters
function extract_roles( role_pat, txt ) {
	var max ;
	const result = trace_match( txt, role_pat ) ;
	if( debug_debug) {
		console.log( 'extract_roles	%s', support.truncate( JSON.stringify( result ), truncate_max ) ) ;
	}
	if( result !== null ) {
		result.map( item => {
			var seq = Number.parseInt( item.replace( /[\[\]\(\)a-zA-Z]/g, ' ' ) ) ;
			if( isNaN( seq ) ) {
				seq = 1 ;
			}
			if( typeof max === 'undefined' || seq > max ) {
				max = seq ;
			}
		} )
	}
	return max ;
}

// Markup ignored.
const markup_list	= [
	'align', 'bridgehead', 'colspec', 'depth', 'emphasis', 'entry', 'fileref', 'footnote-ref', 'format', 'imagedata',
	'imageobject', 'informaltable', 'inlinemediaobject', 'itemizedlist', 'listitem', 'mediaobject', 'nr', 'orderedlist', 'para', 'parablock',
	'paragroup', 'row', 'rs:para', 'scale', 'tbody', 'tgroup', 'table', 'textobject', 'title', 'width', 'superscript', '\\?linebreak\\?'
] ;

const full_markup_list = [
	'section', 'footnote', 'uitspraak', 'uitspraak.info', 'inhoudsindicatie', 'open-rechtspraak' ].concat( markup_list ) ;

// Strip all XML
function strip_XML( xml_txt ) {
	const stripped_XML =
		xml_txt
		// Ignore all markup related elements that are embedded in content
		.replace( /<\/?emphasis[^>]*>/gi, '' )
		.replace( /<nr>[^<]*<\/nr>/gi, ' ' )
		.replace( new RegExp ( '<\\\/?(' + full_markup_list.join( '|' ) + ')[^>]*>', 'gi' ), ' ' )
		// replace hyphen and non-breaking hyphen, and other by dash
		.replace( /[‐‑▪◦]/g, '-' )
		// Remove special characters
		.replace( /[… ]/g, ' ' )
		// Strip all manner of quotes
		.replace( /[–]|[“”]|['‘'’]|[  ]/g, ' ' )
		// Replace some periods by underscores to ensure that a period ends a sentence.
		.replace( /\b(www)\.+([a-z]*)\.+([a-z]*)/gi, '_$1_$2_$3_' )
		.replace( /\b([a-z])\.+([a-z])\.+([a-z])\.+([a-z])\.+([a-z])\.+([a-z])\.+/gi, '_$1_$2_$3_$4_$5_$6_' )
		.replace( /\b([a-z])\.+([a-z])\.+([a-z])\.+([a-z])\.+([a-z])\.+/gi, '_$1_$2_$3_$4_$5_' )
		.replace( /\b([a-z])\.+([a-z])\.+([a-z])\.+([a-z])\.+/gi, '_$1_$2_$3_$4_' )
		.replace( /\b([a-z])\.+([a-z])\.+([a-z])\.+/gi, '_$1_$2_$3_' )
		.replace( /\b([a-z])\.+([a-z])\.+/gi, '_$1_$2_' )
		.replace( /\b([a-z])\.+/gi, '_$1_' )
		.replace( /\b(adj|af[bkz]|bij[lvz]|blz|bv|c[af]|d[rs]|drs|d[hi]r|e[dm]|enz|etc|evt)\.+/gi, '_$1_' )
		.replace( /\b(fig|ge[bhtsz]|ing|i[dr]|j[lr])\.+/gi, '_$1_' )
		.replace( /\b(mevr|min|ml[nd]|mr|mrs|m[vw]|n[lr])\.+/gi, '_$1_' )
		.replace( /\b(prof|pp|re[fd])\.+/gi, '_$1_' )
		// Normalise some symbols
		.replace( /€/g, ' euro ' )
		// Strip whitespace
		.replace( /\s+/g, ' ' )
		.trim() ;
	return stripped_XML ;
}

// Read and parse an XML file
function strip_and_parse_XML( filename, xml_txt ) {
	const uitspraak = xml_txt.replace( /.*<(?:conclusie|uitspraak) id=[^>]*>(.*)<\/(?:conclusie|uitspraak)>.*/s, '$1' ) ;
	const data = {
		Filename:		filename,
		ECLI:			xml_txt.replace( /.*<dcterms:identifier>\s*(ECLI:NL:[^<]*)<\/dcterms:identifier>.*/s, '$1' ).trim(),
		Datum_uitspraak:	xml_txt.replace( /.*<dcterms:date rdfs:label="(?:Uitspraakdatum|Datum genomen)">\s*([0-9]+)-([0-9]+)-([0-9]+)\s*<\/dcterms:date>.*/s, '$3-$2-$1' ),
		Instelling:		xml_txt.replace( /.*<dcterms:creator rdfs:label="Instantie"[^>]*>([^<]*)<\/dcterms:creator>.*/s, '$1' ).trim(),
		Zaaknummer:		xml_txt.replace( /.*<psi:zaaknummer rdfs:label="Zaaknr">([^<]*)<\/psi:zaaknummer>.*/s, '$1' ).replace( /\s+/g, ' ' ).trim(),
		Type:			xml_txt.replace( /.*<dcterms:type[^>]*>([^>]*)<\/dcterms:type>.*/s, '$1' ).trim(),
		Locatie:		xml_txt.replace( /.*<dcterms:spatial rdfs:label="Zittingsplaats">([^<]*)<\/dcterms:spatial>.*/s, '$1' ).trim(),
		Rechtsgebieden:		xml_txt.replace( /.*<dcterms:subject rdfs:label="Rechtsgebied"[^>]*>([^<]*)<\/dcterms:subject>.*/s, '$1' ).split( / *[;,] */ ).map( item => item.trim() ),
		Taal:			xml_txt.replace( /.*<dcterms:language>([^<]*)<\/dcterms:language>.*/s, '$1' ).trim(),
		Inhoudsindicatie:	strip_XML( xml_txt.replace( /.*<inhoudsindicatie[^>]*>(.*)<\/inhoudsindicatie>.*/s, '$1' ) ),
		Uitspraak:		strip_XML( uitspraak ),
		Sections:		uitspraak.split( /<\/section>/ ).map( item => strip_XML( item ) ).filter( item => item.trim().length > 0 )
	}
	Object.keys( data ).map( key => {
		const value = JSON.stringify( data[ key ] ) ;
		if( value.match( /<\?xml/ ) ) {
			delete data[ key ] ;
		}
	} ) ;
	if( debug ) {
		console.log( 'strip_and_parse_XML -> %s', support.truncate( JSON.stringify( data ), truncate_max ) ) ;
	}

	// Handle the most important sections of the court case.
	var toc = {} ;
	toc.decision =			trace_match_list_list( xml_txt, decision_section_pat_list_list, true ) ;
	toc.footnote =			trace_match_list( xml_txt, footnote_section_pat_list, true ) ;
	toc.end =			Object.assign( [''], { index: xml_txt.length } ) ; // Sentinel

	// Check if additional articles have been mentioned in the text before the decision.
	const truncated_xml_txt = toc.decision ? xml_txt.slice( 0, toc.decision.index ) : xml_txt ;
	toc.law =			trace_match_list_list( truncated_xml_txt, law_section_pat_list_list, true ) ;
	toc.criminality_facts =		trace_match_list_list( truncated_xml_txt, criminality_facts_section_pat_list_list, true ) ;
	toc.criminality_offender =	trace_match_list_list( truncated_xml_txt, criminality_offender_section_pat_list_list, true ) ;

	Object.keys( toc ).map( key => {
		if( toc[ key ] ) {
			if( debug ) {
				console.log( '## %s: %s [%d]%s', filename, key, toc[ key ].index, support.truncate( JSON.stringify( toc[ key ] ), truncate_max ) ) ;
			}
		} else {
			if( debug ) {
				console.log( '## %s: %s not found', filename, key ) ;
			}
			delete toc[ key ] ;
		}
	} ) ;
	const sorted_key_list = Object.keys( toc ).sort( (a, b ) => toc[ a ].index - toc[ b ].index ) ;
	const from_to_key_list = support.zip( sorted_key_list.slice( 0, -1 ), sorted_key_list.slice( 1 ) ) ;
	from_to_key_list.map( ( [ from_key, to_key] ) => {
		toc[ from_key ].txt = strip_XML( xml_txt.slice( toc[ from_key ].index, toc[ to_key ].index ) ) ;
	} ) ;
	if( typeof toc.law !== 'undefined' ) {
		if( typeof toc.criminality_facts === 'undefined' ) {
			if( ! toc.law.txt.match( /[0-9]/ ) && typeof toc.decision !== 'undefined' ) { // Heuristic
				// 2017/02/ECLI_NL_RBOBR_2017_918.xml:
				// toc.law= "Toepasselijke-wetsartikelen."
				// toc.decision= "De beslissing is gegrond op de artikelen 10, 27, 57 van het Wetboek van Strafrecht"
				data.law = toc.law.txt + ' ' + toc.decision.txt ;
			} else {
				data.law = toc.law.txt ;
			}
		} else {
			if( toc.law.index < toc.criminality_facts.index ) {
				data.law = toc.law.txt + ' ' + toc.criminality_facts.txt ;
			} else {
				data.law = toc.criminality_facts.txt + ' ' + toc.law.txt ;
			}
		}
	}
	if( typeof toc.decision !== 'undefined' ) {
		data.decision = toc.decision.txt ;
	}
	return data ;
}

// The collection of heuristics
class Heuristics {
	// Create the object
	constructor( xml_filename, error_message ) {
		this.Filename = xml_filename ;
		this.Error_list = typeof error_message === 'undefined' ? [] : [ error_message ] ;
	}

	// Record an error
	Error( message ) {
		this.Error_list.push( message ) ;
		return this ;
	}

	// Initialise and validate the object
	Initialise( jsonObj ) {
		Object.assign( this, jsonObj ) ;
		if( typeof this.Uitspraak === 'undefined' ) {
			this.Error( 'uitspraak not found (fatal)' ) ;
		} else {
			if( this.Uitspraak.match( /Deze publicatie is ingetrokken door/ ) ) {
				this.Error( 'publication withdrawn (fatal)' ) ;
			} else if( this.Uitspraak.match( /(?:The Hague District Court|District Court of The Hague)/i ) ) {
				this.Error( 'publication in English (fatal)' ) ;
			} else if( ( this.Inhoudsindicatie || '' ).match( /Het vonnis in deze zaak is op een nieuwe manier opgebouwd./i ) ) {
				this.Error( 'non-standard case from RBROT (warning)' ) ;
			}
		}
		if( debug ) {
			console.log( 'dataObj	%s', JSON.stringify( this, null, '  ' ) ) ;
		}
		return this ;
	}

	// Analyse the year of birth
	Year_of_birth( ) {
		const txt = this.Uitspraak || '' ;
		const demographics = trace_match( txt, demographics_pat ) ;
		if( demographics ) {
			const year_match = trace_match( demographics[ 0 ], year_pat ) ;
			if( debug_debug) {
				console.log( 'Year_of_birth	%s, %s', JSON.stringify( year_match ), support.truncate( txt, truncate_max ) ) ;
			}
			// Often the year of birth is missing because of anonymisation by the courts.
			if( year_match && year_match.length === 2 ) {
				const year = Number.parseInt( year_match[ 1 ] ) ;
				if( year < 1930 || year > 2005 ) {
					// Sometime the year is not plausible
					this.Error( "'" + year + "' not plausible as year of birth (warning)" ) ;
				} else {
					this.Geboortejaar = year ;
				}
			}
		}
		const current_year = Number.parseInt( this.Datum_uitspraak.replace( /\d\d-\d\d-(\d\d\d\d)/, '$1' ) ) ;
		if( typeof this.Inhoudsindicatie !== 'undefined' && ! isNaN( current_year ) ) {
			const txt = this.Inhoudsindicatie ;
			const age_match = trace_match( txt, age_pat ) ;
			const filtered_age_match = age_match ? age_match.filter( item => item != null ) : [] ;
			if( debug_debug) {
				console.log( '%s	Age?	%s, %s',
					this.ECLI, JSON.stringify( filtered_age_match ), support.truncate( txt, truncate_max ) ) ;
			}
			if( filtered_age_match.length === 2 ) {
				const txt = filtered_age_match[ 0 ] ;
				const jarige_match = trace_match( txt, jarige_pat ) ;
				if( debug_debug) {
					console.log( '%s	Age??	%s, %s',
						this.ECLI, JSON.stringify( jarige_match ), support.truncate( txt, truncate_max ) ) ;
				}
				if( jarige_match && jarige_match.length === 1 ) {
					const age = parse_age( filtered_age_match[ 1 ] ) ;
					if( typeof age !== 'undefined' ) {
						if( age < 15 || age > 90 ) {
							this.Error( "'" + age + "' not plausible as age (warning)" ) ;
						} else {
							if( debug_debug) {
								console.log( '%s	Age!	%d, %s',
									this.ECLI, current_year - age, support.truncate( txt, truncate_max ) ) ;
							}
							const geboorte_jaar = current_year - age ;
							if( typeof this.Geboortejaar === 'undefined' ) {
								this.Geboortejaar = geboorte_jaar ;
							} else if( geboorte_jaar > 2000 && Math.abs( this.Geboortejaar - geboorte_jaar ) > 1 ||
								geboorte_jaar <= 2000 && Math.abs( this.Geboortejaar - geboorte_jaar ) > 5 ) {
								this.Error( "'" + geboorte_jaar + "' and '" + this.Geboortejaar + "' discrepancy in year of birth (warning)" ) ;
							}
						}
						if( typeof this.Geslacht === 'undefined' && trace_match( txt, /vrouw/ ) ) {
							if( debug_debug) {
								console.log( '%s	Vrouw!	%s', this.ECLI, support.truncate( txt, truncate_max ) ) ;
							}
							this.geslacht = 'vrouw' ;
						}
					}
				}
			}
		}
		return this ;
	}

	// Analyse the country of birth
	Country_of_birth( ) {
		const txt = this.Uitspraak || '' ;
		const demographics = trace_match( txt, demographics_pat ) ;
		if( demographics ) {
			// Often the country of birth is missing because of anonymisation by the courts,
			// but we should always be able to detect whether the defendant was born abroad.
			this.Geboorteland = trace_match( demographics[ 0 ], country_pat ) ? 'buitenland' : 'Nederland' ;
		}
		return this ;
	}

	// Analyse gender
	Gender( ) {
		if( typeof this.Geslacht === 'undefined' ) {
			const txt = this.Uitspraak || '' ;
			if( trace_match_list( txt, gender_pat_list ) ) {
				this.Geslacht = 'vrouw' ;
			} else {
				this.Geslacht = 'man' ;
			}
		}
		return this ;
	}

	// Analyse the keywords
	Keywords( ) {
		const txt = this.Uitspraak || '' ;
		this.Onderzoek = extract_keywords( case_list, txt ) ;
		this.Tools = extract_keywords( tool_list, txt ) ;
		this.Bank = extract_keywords( bank_list, txt ) ;
		this.Richtlijnen = extract_keywords( guideline_list, txt ) ;
		this.Expertise_verdachte = extract_keywords( expertise_defendant_list, txt ) ;
		this.Internet = extract_keywords( internet_list, txt ) ;
		this.Terrorisme = extract_keywords( terrorism_list, txt ) ;
		this.Intensiteit = extract_keywords( intensity_list, txt ) ;
		this.Gevaar = extract_keywords( threat_list, txt ) ;
		this.Article_36e = extract_keywords( article_36e_list, txt ) ;
		const normalised_txt = txt
			.replace( /\bart\./g, 'artikel' )
			.replace( /\(*\b(?:gpg|gnu[ -]*privacy[ -]*guard|gnupg|gpg4win|openpgp|pgp|GNU Privacy Assistant|pretty[ -]*good[ -]*privacy)\b\)*/gi, 'PGP' )
			.replace( /Nederlands Forensisch Instituut \(hierna: NFI\)/gi, 'NFI' ) ;
		this.Vertrouwelijkheid = extract_keywords( confidentiality_list, normalised_txt ) ;
		this.Expertise_rechtbank = extract_keywords( expertise_court_list, txt ) ;
		this.Opsporing = extract_keywords( investigation_list, normalised_txt ) ;
		return this ;
	}

	// Analyse how many of relevant roles are mentioned, like defendant, victim, witness etc
	Roles( ) {
		const txt = this.Uitspraak || '' ;
		const aantal_verdachten = extract_roles( defendant_pat, txt ) ;
		const aantal_medeverdachten = extract_roles( codefendant_pat, txt ) ;
		var aantal = 1 ;
		if( typeof aantal_medeverdachten === 'undefined' ) {
			aantal = aantal_verdachten ;
		} else if( typeof aantal_verdachten === 'undefined' ) {
			aantal = aantal_medeverdachten + 1 ;
		} else {
			aantal = Math.max( aantal_verdachten, aantal_medeverdachten + 1 ) ;
		}
		if( aantal === 1 && trace_match( txt, multiple_codefendant_pat ) ) {
			aantal = 2 ;
		}
		if( aantal > 75 ) {
			this.Error( "'" + aantal + "' not plausible as number of defendants (warning)" )
		} else {
			this.Verdachten_aantal = aantal ;
		}
		aantal = extract_roles( reporter_pat, txt ) ;
		if( aantal > 300 ) {
			this.Error( "'" + aantal + "' not plausible as number of reporters (warning)" )
		} else {
			this.Aangevers_aantal = aantal ;
		}
		aantal = extract_roles( witness_pat, txt ) ;
		if( aantal > 150 ) {
			this.Error( "'" + aantal + "' not plausible as number of witnesses (warning)" )
		} else {
			this.Getuigen_aantal = aantal ;
		}
		aantal = extract_roles( victim_pat, txt ) ;
		if( aantal > 600 ) {
			this.Error( "'" + aantal + "' not plausible as number of victims (warning)" )
		} else {
			this.Slachtoffers_aantal = aantal ;
		}
		return this ;
	}

	// Analyse first / repeat offender
	Offender( ) {
		const txt = this.Uitspraak || '' ;
		const offender_section	= trace_match_list( txt, offender_pat_list ) ;
		const first_offender	= trace_match_list( txt, first_offender_pat_list ) ;
		const repeat_offender	= trace_match_list( txt, repeat_offender_pat_list ) ;
		if( offender_section ) {
			if( debug_debug) {
				console.log( 'offender_section	%s', support.truncate( JSON.stringify( offender_section ), truncate_max ) ) ;
			}
			this.Recidive = trace_match( offender_section[ 0 ], yes_no_pat ) ? 'Eerste keer' : 'Recidive' ;
		} else if( first_offender ) {
			if( debug_debug) {
				console.log( 'first_offender	%s', support.truncate( JSON.stringify( first_offender ), truncate_max ) ) ;
			}
			this.Recidive = 'Eerste keer' ;
		} else if( repeat_offender ) {
			if( debug_debug) {
				console.log( 'repeat_offender	%s', support.truncate( JSON.stringify( repeat_offender ), truncate_max ) ) ;
			}
			this.Recidive = 'Recidive' ;
		}
		return this ;
	}

	// Analyse applicable laws
	Applicable_laws( ) {
		var law_section ;
		law_section = this.law ;
		// Laws are missing for acquittals.
		if( typeof law_section === 'undefined' ) {
			return this.Error( 'law section not found (warning)' ) ;
		}
		var unsorted_result = extract_articles( law_section ) ;
		const result = unsorted_result.map( item => item.slice( 0, 1 ).concat( support.uniq( item.slice( 1 ) ).sort() ) ) ;
		if( result.length > 0 ) {
			this.Wettelijke_voorschriften = result ;
		}
		return this ;
	}

	// Analyse the decision of the court
	Decision( ) {
		var decision_section ;
		decision_section = this.decision ;
		if( typeof decision_section === 'undefined' ) {
			return this.Error( 'decision section not found (warning)' ) ;
		}
		const partial_acquittal = trace_match_list( decision_section, partial_acquittal_pat_list ) ;
		if( partial_acquittal ) {
			const count = Math.max( 1, partial_acquittal[ 0 ].split( /\b[0-9][0-9\/\.-]*\b/ ).length - 1 ) ;
			if( debug ) {
				console.log( 'partial_acquittal: %s (%d)', JSON.stringify( partial_acquittal ), count ) ;
			}
			this.Gedeeltelijk_vrij = count ;
		} else {
			this.Gedeeltelijk_vrij = 0 ;
		}
		const sanction_pat_list = [ youth_treatment_pat_list, treatment_pat_list, youth_detention_pat_list, detention_pat_list, labour_pat_list, fine_pat_list ].flat() ;
		const sanction = trace_match_list( decision_section, sanction_pat_list ) ;
		const no_sanction_pat_list = [ admin_pat_list, acquittal_pat_list ].flat() ;
		const no_sanction = trace_match_list( decision_section, no_sanction_pat_list ) ;
		var index ;
		if( sanction ) {
			index = sanction.index ;
			if( sanction.id.match( /youth_treatment/ ) ) {
				const youth_treatment_duration = parse_duration( sanction[ 1 ] ) ;
				this.Beslissing = [ {'soort': 'gedragsbeinvloedendemaatregel', 'aantal': youth_treatment_duration.value, 'eenheid': youth_treatment_duration.unit} ] ;
			} else if( sanction.id.match( /treatment/ ) ) {
				const treatment_duration = parse_duration( sanction[ 1 ] ) ;
				this.Beslissing = [ {'soort': 'terbeschikkingstelling', 'aantal': treatment_duration.value, 'eenheid': treatment_duration.unit} ] ;
			} else if( sanction.id.match( /youth_detention/ ) ) {
				const youth_detention_duration = parse_duration( sanction[ 1 ] ) ;
				this.Beslissing = [ {'soort': 'jeugddetentie', 'aantal': youth_detention_duration.value, 'eenheid': youth_detention_duration.unit} ] ;
			} else if( sanction.id.match( /detention/ ) ) {
				const detention_duration = parse_duration( sanction[ 1 ] ) ;
				this.Beslissing = [ {'soort': 'gevangenisstraf', 'aantal': detention_duration.value, 'eenheid': detention_duration.unit} ] ;
			} else if( sanction.id.match( /fine/ ) ) {
				const fine_amount = parse_euro_cent( sanction[ 1 ] ) ;
				this.Beslissing = [ {'soort': 'geldboete', 'aantal': fine_amount, 'eenheid': 'euro'} ] ;
			} else if( sanction.id.match( /labour/ ) ) {
				const labour_duration = parse_duration( sanction[ 1 ] ) ;
				this.Beslissing = [ {'soort': 'taakstraf', 'aantal': labour_duration.value, 'eenheid': labour_duration.unit} ] ;
			} else {
				console.log( '%s: unknown sanction %s', this.Filename, sanction ) ;
				process.exit( 1 ) ;
			}
		} else if ( no_sanction ) {
			index = no_sanction.index ;
			if( no_sanction.id.match( /admin/ ) ) {
				this.Beslissing = [ {'soort': 'admin'} ] ;
			} else if( no_sanction.id.match( /acquittal/ ) ) {
				this.Beslissing = [ {'soort': 'vrijspraak'} ] ;
			} else {
				console.log( '%s: unknown no_sanction %s', this.Filename, no_sanction ) ;
				process.exit( 1 ) ;
			}
		} else {
			return this.Error( 'decision text not found (warning)' ) ;
		}
		return this ;
	}

	// Delete the temporary fields
	Finalise( ) {
		if( this.Error_list.length === 0 ) {
			delete this.Error_list ;
		}
		if( ! sections ) {
			delete this.Sections ;
			delete this.law ;
			if( typeof this.decision !== 'undefined' ) {
				this.Taakstraf = this.decision.match( /(taakstraf|werkstraf|leerstraf)/i ) ? 1 : 0 ;
			}
			delete this.decision ;
			delete this.Article_36e ;
		} else if( typeof this.Sections !== 'undefined' ) {
			if( typeof this.law !== 'undefined' && this.Sections.indexOf( this.law ) >= 0 ) {
				this.law = 'See ' + support.truncate( this.law, 80 ) ;
			}
			if( typeof this.decision !== 'undefined' && this.Sections.indexOf( this.decision ) >= 0 ) {
				this.decision = 'See ' + support.truncate( this.decision, 80 ) ;
			}
		}
		delete this.Uitspraak ;
		delete this.footnote ;
		return this ;
	}
}


function main() {
	var result = [] ;
	var output_filename ;
	var input_filename_list = [] ;
	for( var i = 2; i < process.argv.length; i++ ) {
		const arg = process.argv[ i ] ;
		if( arg === '-d' ) {
			debug = true ;
		} else if( arg === '-dd' ) {
			debug = true ;
			debug_debug = true ;
		} else if( arg === '-t' ) {
			trace = true ;
		} else if( arg === '-s' ) {
			sections = true ;
		} else if( arg === '-o' ) {
			i++ ;
			if( i < process.argv.length ) {
				output_filename = process.argv[ i ] ;
			}
		} else {
			input_filename_list.push( arg ) ;
		}
	}
	if( input_filename_list.length === 0 || typeof output_filename === 'undefined' ) {
		console.error( 'usage: node %s input_file... -o output_file [-d] [-dd] [-t] [-s]', process.argv[ 1 ] ) ;
		process.exit( 1 ) ;
	}
	input_filename_list.map( filename => {
		const unlatinized_txt = support
			.read_txt( filename )
			.replace( /(<\?linebreak\?>|\s)+/g, ' ' ) ;
		const xml_txt = latinize( unlatinized_txt ) ;

		const uitspraak = [ ...xml_txt.matchAll(/<uitspraak[^\.]/g) ] ;
		const conclusion = [ ...xml_txt.matchAll(/<conclusion/g) ] ;
		const info = [ ...xml_txt.matchAll(/<uitspraak.info/g) ] ;
		const header = {
			section: [ ...xml_txt.matchAll(/<section[^>]*>[\s\S]{0,100}/g) ],
			title: [ ...xml_txt.matchAll(/<title[^>\/]*>[\s\S]{0,100}<\/title>/g) ],
			bridge: [ ...xml_txt.matchAll(/<bridgehead[^>\/]*>[\s\S]{0,100}<\/bridgehead>/g) ],
			emphasis: [ ...xml_txt.matchAll(/<emphasis[^>\/]*>[\s\S]{0,100}<\/emphasis>/g) ],
			para: [ ...xml_txt.matchAll(/<para\b[^>\/]*>[\s\S]{0,100}<\/para>/g) ]
		}
		console.log( '## %s: %d uitspraak, %d conclusion, %d info, %d section, %d title, %d bridge, %d emphasis, %d para',
			filename, uitspraak.length, conclusion.length, info.length,
			header.section.length, header.title.length, header.bridge.length, header.emphasis.length, header.para.length ) ;
		if( debug ) {
			Object.keys( header ).map( key => {
				const max = 20 ;
				const list = header[ key ];
				list.slice( 0, max ).map( item => {
					console.log( '##  %d: %s', item.index, support.truncate( JSON.stringify( item[ 0 ] ), truncate_max ) ) ;
				} ) ;
				if( list.length > max ) {
					console.log( '##  ...%d', list[ list.length - 1 ].index )
				}
			} )
		}

		const jsonObj = strip_and_parse_XML( filename, xml_txt ) ;
		if( debug ) {
			console.log( 'jsonObj	%s', filename, JSON.stringify( jsonObj, null, '  ' ) ) ;
		}
		const data = new Heuristics( filename )
			.Initialise( jsonObj )
			.Year_of_birth( )
			.Country_of_birth( )
			.Gender( )
			.Keywords( )
			.Roles( )
			.Offender( )
			.Applicable_laws( )
			.Decision( ) ;

		const Internet_WhatsApp = ( data.Internet || [] ).filter( item => item.match( /WhatsApp/ ) ).length > 0 ;
		const Vertrouwelijkheid_PGP = ( data.Vertrouwelijkheid || [] ).filter( item => item.match( /^PGP-/ ) ).length > 0 ;
		const WhatsApp_PGP = ( Internet_WhatsApp !== Vertrouwelijkheid_PGP ? 'pgp' : '' ) ;
		if( typeof data.Wettelijke_voorschriften === 'undefined' && typeof data.Article_36e !== 'undefined' ) {
			if( debug ) {
				console.log( '%s> %s: article 36e Sr treated specially', WhatsApp_PGP, data.Filename ) ;
			}
			data.Wettelijke_voorschriften = [["Sr","36e"]] ;
		}
		if( debug ) {
			if( typeof data.decision !== 'undefined' && typeof data.Beslissing === 'undefined' ) {
				console.log( '%s> %s: decision text present but result missing %s',
					WhatsApp_PGP, data.Filename, JSON.stringify( data.decision ) ) ;
			}
			if( typeof data.Wettelijke_voorschriften === 'undefined' && typeof data.Beslissing === 'undefined' ) {
				console.log( '%s> %s: both decision and legal grounds missing',
					WhatsApp_PGP, data.Filename ) ;
			}
			if( typeof data.Wettelijke_voorschriften !== 'undefined' && typeof data.Beslissing === 'undefined' ) {
				console.log( '%s> %s: decision missing but legal grounds present %s',
					WhatsApp_PGP, data.Filename, JSON.stringify( data.Wettelijke_voorschriften ) ) ;
			}
			if( typeof data.Wettelijke_voorschriften === 'undefined' && typeof data.Beslissing !== 'undefined' ) {
				console.log( '%s> %s: legal grounds missing but decision present %s',
					WhatsApp_PGP, data.Filename, JSON.stringify( data.Beslissing ) ) ;
			}
		}
		data.Finalise( ) ;
		result.push( data ) ;
	} ) ;
	support.write_JSON( output_filename, result ) ;
}

main( ) ;
